//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#ifndef X_CLIP_H
#define X_CLIP_H

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include "outputWindow.h"

enum  ClipStatus {NORMAL_E, IGNOREMASK_E, IGNOREIMAGE_E, IGNOREBOTH_E, };
enum  ShapeMode  {SOLID_E, HOLLOW_E,};
enum  ShapeStyle {TRANSPARENT_E, OPAQUE_E, MASK_E, };
enum  ShapeType  {CIRCLE_E, ELLIPSE_E,RECTANGULAR_E,STAR_E,GEAR_E,SPRAY_E,
		  MESH_E, DIAGONAL_E, POLYGON_ES_E,};

class Clip {
protected:
  ClipStatus     clipStatus;
  Display       *Dpy;
  GC             gc;
  Pixmap         shapeMask;
  Widget         drawingArea;
  XGCValues      vals;
  XImage        *image;
  char          *clipName;
  char          *diskFileName;
  char          *shapeFileName;
  int            IsClone;        // == 0 no, == 1 yes
  int            RefCounter;
  int            bytesperline;
  int            defaultImageType;
  int            depth;
  int            dsp_x,dsp_y,dsp_width, dsp_height;
  int            linepad;
  int            ncolors;
  int            width, height;
  unsigned char *bit_data;
public:
  Clip(Display *d,Widget);
  Clip();
  virtual    ~Clip();

  ClipStatus  getClipStatus()	     { return clipStatus; }
  Display    *getDisplay()	     { return Dpy;}
  GC	      GetGC()		     { return gc;}
  Pixmap      getShapeMask()	     { return shapeMask; }
  Widget      getDrawingArea()	     { return drawingArea;}
  XImage     *GetImage()	     { return image;}
  char	     *getClipName()	     { return clipName?clipName:"";}
  char	     *getDiskFileName()	     { return diskFileName?diskFileName:"";}
  char	     *getShapeFileName()     { return shapeFileName?shapeFileName:"";}
  int	      Height()		     { return height;}
  int	      Width()		     { return width;}
  int	      getNcolors()	     { return ncolors; }
  int         isClone()		     { return IsClone; }

  void        deleteShapeMask();
  char	     *getClipStatusInString();
  int         getRefCounter();
  void	      SetGC(GC g);
  void        setDisplay(Display *);
  void	      setClipStatus(ClipStatus p);
  void	      setShapeMask(Pixmap p);
  void	      setNcolors(int);
  void	      setDiskFileName(char *s);
  void        setShapeFileName(char *s);
  void	      setClipName(char *s);
  void        incrRefCounter();
  void        decrRefCounter();
  void	      writeOut();
  void	      writeOut(char *fileName);
  void	      writeOutMask();
  void	      writeOutMask(char *fileName);
  void        deallocateString();
  void        copyto(Clip *rtn);
  XImage     *copyImage();
  Pixmap      copyMask();
  void	      DisplayImage(OutputWindow *w);
  void	      DisplayImage(OutputWindow *w,int x, int y, int shapemaskbg = -1);
  void	      DisplayImage(OutputWindow *w,int x, int y, int w, int h, int shapemaskbg = -1);
  void	      DisplayImage(OutputWindow *w,Pixmap w,int x, int y, int shapemaskbg = -1);
  void	      DisplayImage(OutputWindow *w,Pixmap w,int x, int y, int w, int h, int shapemaskbg = -1);


  virtual void MakeImage(int w, int h, int bg, int fg, int wi, int hi);
  virtual void MakeImageAndDisplay(OutputWindow *w,int w, int h, int bg, 
				   int fg, int wi, int hi);
  virtual void DrawText(char *s,Widget win,int wbox,int hbox,int xloc,int yloc,
			int justification,int size,
			unsigned long fg, unsigned long bg,XImage *);
  virtual void DrawText(char s, Widget win,int wbox,int hbox,int xloc,int yloc,
			int justification,int size,
			unsigned long fg, unsigned long bg,XImage *);
  virtual void PutPixel(int x,  int y, unsigned long v);
  virtual void buildXimage();
  virtual Clip *dup();
};

#endif
