//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "clipList.h"
#include "util.h"


static char *fieldSkip(char *p)
{
  //Search for end of field.
  
  while( *p && *p != ',' && *p != '\n' )
    ++p;
  
  //Make sure field is terminated by a null character.
  if( *p ) *p++ = 0;
  
  //Return pointer to beginning of next field, or null character
  //if we have just skipped the last field.
  return(p);
}

ClipList::ClipList(char *s,ClipsTable *ct)
:size(0),current(NULL),head(NULL),dir(FORWARD_E),clipsPool(ct)
{
  Clip *clp;
  char *p = s;
  char *c;
  ClipListEntry *cle;
  while(p){
    if(strcmp("",p) == 0)
      break;
    c = p;
    p = fieldSkip(p);
    clp = clipsPool->findClip(c);
    if(clp){
      cle = new ClipListEntry();
      cle->clip = clp;
      cle->next = NULL;
      cle->prev = NULL;
      if(head == NULL){
	head = cle;
	current = head;
      }else{
	current->next = cle;
	current = current->next;
      }
      size++;
    }else
      insertStringtoInfo("Clip <%s> not found, ignore\n",c); 
  }
  if(current)
    current->next = head;
  current = head;
}

ClipList::ClipList(const ClipList &cl){
  dir = cl.dir;
  size = cl.size;
  if(!cl.head)
    return;
  ClipListEntry *walk = cl.head;
  ClipListEntry *cle = new ClipListEntry();
  cle->clip = cl.head->clip;
  head = current = cle;
  while(walk->next != cl.head){
    if(walk->next){       // may not need this if since it is a circluar list
      cle =  new ClipListEntry();
      cle->clip = walk->next->clip;
      current->next = cle;
      current = cle;
      walk = walk->next;
    }
  }
  cle->next = head;
  if(dir == BACKWARD_E)
    reverse();
}

ClipList::~ClipList(){
  if(head){
    ClipListEntry *cle;
    current = head->next;
    while(current != head){
      cle = current;
      current = current->next;
      delete cle;
    }
    delete head;
  }
}

int   ClipList::Size(){
  return size;
}

void   ClipList::reset(){
  current = head;
}

void   ClipList::reverse(){
  if(head->prev == NULL){    // haven't built the reverse list
    ClipListEntry *cle = head->next;
    current = head;
    if(cle){
      while(cle != head){
	cle->prev = current;
	cle = cle->next;
	current = current->next;
      }
      head->prev = current;
    }
  }
  if(dir == FORWARD_E)
    dir = BACKWARD_E;
  else
    dir = FORWARD_E;
}

Clip *ClipList::nextClip(){
  ClipListEntry *cle = current;
  if(dir == FORWARD_E)
    current = current->next;
  else
    current = current->prev;
  return cle->clip;
}

void   ClipList::writeOut(FILE *fp){
  ClipListEntry *cle;
  cle = head->next;
  fprintf(fp,"%s",head->clip->getClipName());
  while(cle != head){
    fprintf(fp,",%s",cle->clip->getClipName());
    cle = cle->next;
  }
}

char *ClipList::getListofNames(){
  char *rtn;
  if(head == NULL){
    rtn = new char[4];
    rtn[0] = '\0';
    return rtn;
  }
  ClipListEntry *cle = head->next;

  // find out how many characters is need
  int k = 0;
  k = k + strlen(head->clip->getClipName()) + 1;
  while(cle != head){
    k = k + strlen(cle->clip->getClipName()) + 1;
    cle = cle->next;
  }
  rtn = new char[k+1];
  cle = head->next;
  sprintf(rtn,"%s",head->clip->getClipName());
  while(cle != head){
    sprintf(&rtn[strlen(rtn)],",%s",cle->clip->getClipName());
    cle = cle->next;
    if(strlen(rtn) > k+1){
      insertStringtoInfo("List of clip names is too long");
      break;
    }
  }
  return rtn;
}

void ClipList::incrRefCounter(){
  ClipListEntry *cle = head->next;
  head->clip->incrRefCounter();
  while(cle != head){
    cle->clip->incrRefCounter();
    cle = cle->next;
  }
}

void ClipList::decrRefCounter(){
  ClipListEntry *cle = head->next;
  head->clip->decrRefCounter();
  while(cle != head){
    cle->clip->decrRefCounter();
    cle = cle->next;
  }
}




