//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "xet.h"
#include "clipsTable.h"
#include <libc.h>
#include <string.h>
#include "util.h"
#include "cloneImage.h"

extern Xet   *XET;

ClipsTable::ClipsTable(int k)
:Hash(k),dup_index(0),size(k)
{
}

ClipsTable::~ClipsTable(){
}

int ClipsTable::Size(){
  return occupy();
}

int   ClipsTable::Howmany(){
  return count;
}

Clip *ClipsTable::dup(char *name){
  Clip *ls = findClip(name);
  Clip *rtn;
  char buf[100];
  if(ls){
    rtn = ls->dup();
    sprintf(buf,"%s_%d",name,dup_index++);
    rtn->setClipName(buf);
    addClip(buf,rtn);
  }else
    XET->popupModalInfoBox("It is impossible, clip <%s> not found",name); 
  return rtn;
}

Clip *ClipsTable::clone(char *name){
  Clip *ls = findClip(name);
  Clip *rtn;
  char buf[100];
  if(ls){
    rtn = new CloneImage(ls);
    sprintf(buf,"%s_%d",name,dup_index++);
    rtn->setClipName(buf);
    addClip(buf,rtn);
  }else
    XET->popupModalInfoBox("It is impossible, clip <%s> not found",name); 
  return rtn;
}

void ClipsTable::renameKey(char *Old, char *New){
  ENTRY item;
  item.key = Old;
  ENTRY *rtn;
  if((rtn = hsearch(item,DELETE_E))){
    char *n = rtn->key;
    Clip *l = (Clip *) rtn->data;
    if(n)
      delete n;
    addClip(New,l);
    delete rtn;
  }else
    printf("It is impossible,Clip <%s> not found\n",Old);
}

// both items are deleted by the deleteClips member function
void ClipsTable::addClip(char *name,Clip *c){
  ENTRY item;
  item.key = name;
  ENTRY *rtn;
  if((rtn = hsearch(item,FIND_E))){
    insertStringtoInfo("Clip <%s> already exist \n",name);
  } else {
    ENTRY item;
    item.key = new char[strlen(name)+1];
    strcpy(item.key,name);
    item.data = (char *) c;
    hsearch(item,ENTER_E);
    char tmpbuf[256];
    sprintf(tmpbuf,CLIP_FORMAT,
	    c->getClipName(),
	    c->Width(),
	    c->Height(),
	    (c->isClone() == 1) ? "Link" : "Orig",
	    c->getRefCounter(),
	    (c->getShapeMask() == 0) ? "No Mask" : "Has Mask",
	    c->getClipStatusInString());
    XET->configClipsInfo(tmpbuf);
  }
}

int ClipsTable::deleteClip(char *name){
  ENTRY item;
  item.key = name;
  ENTRY *rtn;
  if((rtn = hsearch(item,DELETE_E))){
    char *n = rtn->key;
    Clip *l = (Clip *) rtn->data;
      if(n) delete n;
      if(l) delete l;
    delete rtn;
    return 1;
  }
  return 0;
}
int ClipsTable::deleteClipMask(char *name){
  ENTRY item;
  item.key = name;
  ENTRY *rtn;
  if((rtn = hsearch(item,FIND_E))){
    Clip *l = (Clip *) rtn->data;
    l->deleteShapeMask();
  }
  return 0;
}

Clip *ClipsTable::findClip(char *key){
  ENTRY item;
  item.key = key;
  ENTRY *entry = hsearch(item,FIND_E);
  if(entry){
    return (Clip *) entry->data;
  }
  else
    return NULL;
}

char *ClipsTable::concateAllNames(){
  int i,k;
    // find out how many chararcter is needed
  for(i=k=0; i < length; i++){
    if(table[i].key != NULL){
      k = k + strlen(table[i].key) + 1;
    }
  }
  char *rtn = new char[k+1];
  rtn[0] = '\0';
  for(i=0; i < length; i++){
    if(table[i].key != NULL){
      sprintf(&rtn[strlen(rtn)],"%s,",table[i].key);
    }
  }
  return rtn;
}
  
void ClipsTable::buildClipsXmList(Widget w){
  char buf[100];
  int i,j;	
  char *c;
  char **tmp = new char *[length];
  for(i=j=0; i < length; i++){
    if(table[i].key != NULL){
      sprintf(buf,CLIP_FORMAT,table[i].key,
	      ((Clip *)(table[i].data))->Width(),
	      ((Clip *)(table[i].data))->Height(),
	      (((Clip *)(table[i].data))->isClone() == 1) ? "Link" : "Orig",
	      ((Clip *)(table[i].data))->getRefCounter(),
	      (((Clip *)(table[i].data))->getShapeMask() == 0) ? "No Mask" : "Has Mask",
	      ((Clip *)(table[i].data))->getClipStatusInString());
      
      c = new char[strlen(buf)+1];
      strcpy(c,buf);
      tmp[j++] = c;
    }
  }
  SortString(tmp,j);
  deleteAllItemsOfXmList(w);
  for(i=0;i<j;i++){
    PutTextInListWidget(w,tmp[i]);
    delete tmp[i];
  }
  delete tmp;
}

void ClipsTable::writeOut(FILE *fp){
  unsigned int i;	/* Counts table slots */
  Pixmap    pm;
  char c_buf[100],m_buf[100];
  fprintf(fp,"## list of clips\n");
  for(i = 0; i < length; i++){
    c_buf[0] = m_buf[0] = '\0';
    if(table[i].key != NULL){
      if(((Clip *)(table[i].data))->isClone() == 1)
	continue;
      ((Clip *)(table[i].data))->writeOut();
      if(strcmp("",((Clip *)(table[i].data))->getDiskFileName()) == 0)
	sprintf(c_buf,"%s.ppm",((Clip *)(table[i].data))->getClipName());
      else
	sprintf(c_buf,"%s",skipDirectoryPath(((Clip *)(table[i].data))->getDiskFileName()));
      pm = ((Clip *)(table[i].data))->getShapeMask();
      if(pm){
	((Clip *)(table[i].data))->writeOutMask();
	if(strcmp("",((Clip *)(table[i].data))->getShapeFileName()) == 0)
	  sprintf(m_buf,"%s.mask",((Clip *)(table[i].data))->getClipName());
	else
	  sprintf(m_buf,"%s",skipDirectoryPath(((Clip *)(table[i].data))->getShapeFileName()));
      }
      fprintf(fp,"clip:%s:%s:%s:%d:%d \n",
	      table[i].key,
	      c_buf,m_buf,
	      ((Clip *)(table[i].data))->Width(),
	      ((Clip *)(table[i].data))->Height());
      
    }
  }
  // now write out clone clips
  char filename[256];
  for(i = 0; i < length; i++){
    c_buf[0] = m_buf[0] = '\0';
    if(table[i].key != NULL){
      if(((CloneImage *)(table[i].data))->isClone() == 0)
	continue;
      pm = ((CloneImage *)(table[i].data))->getShapeMask();
      if(pm){
	sprintf(m_buf,"%s.mask",((CloneImage *)(table[i].data))->getClipName());
	sprintf(filename,"%s/%s",EFFECTIVE_DIRECTORY,m_buf);
	((CloneImage *)(table[i].data))->writeOutMask(filename);
      }
      fprintf(fp,"cloneclip:%s:%s:%s:\n",
	      table[i].key,
	      ((CloneImage *)(table[i].data))->getOriginalClip()->getClipName(),
	      m_buf);
    }
  }
  fprintf(fp,"## end of clips listing\n");
}

void   ClipsTable::clearEverything(){
  Clip *l;
  char *n;
  unsigned int i;	/* Counts table slots */
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      n = table[i].key;
      l = (Clip *) table[i].data;
      if(n)
	delete n;
      if(l)
	delete l;
      table[i].key = NULL;
      table[i].data = NULL;
    }
  }
  count = 0;
  XET->configClipsInfo(NULL);
}
