//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "colorImage.h"
#include "util.h"
#include "color.h"

extern Color      *currentCmap;
ColorImage::ColorImage(Display *d,Widget w)
     : Clip(d,w)
{
}

ColorImage::~ColorImage() {
  if(image)
    XDestroyImage(image);
  deallocateString();
}


// make an Ximage which contains all 256 colors, dimension depends on
// the input parameters 
// cw -- number of pixels in each column
// rw -- number of pixels in each row
// c -- number of columns
// r -- number of rows
// if (c * r) < 256   ====> only display this much
// the image \(width and height\) are calculated
// border -- the width of border in pixel.
// bc -- border color
void ColorImage::MakeImage(int c, int r, int cw, int rw, int border,int bc){
  rowWidth = rw;
  columnWidth = cw;
  nc = c;
  nr = r;
  borderWidth = border;
  borderPixval = bc;
  int x,y;
  height =  r * (border + rw) + border;
  width  =  c * (border + cw) + border;
  if(width <= 0 || height <= 0){
    image = NULL;
    insertStringtoInfo("Can not make color panel, some parameters are unreasonable\n");
    return;
  }
  depth =  DefaultDepth(Dpy, DefaultScreen(Dpy));
  linepad = 8 - (width % 8);
  bit_data = (unsigned char *)new char[((width + linepad) * height) + 1];
  bytesperline = (width + linepad) * depth / 8;
  image = XCreateImage(Dpy, DefaultVisual(Dpy,DefaultScreen(Dpy)), depth,
		       ZPixmap, 0, (char *)bit_data,
		       (width + linepad), height, 8, bytesperline);
  int pixval = 0;
  int xv,xr,yv,yr;
  for(y=0;y<height;y++){
    yv = y / (rw+border);
    yr = y % (rw+border);
    for(x = 0;x<width ;x++){
      xv = x / (cw+border);
      xr = x % (cw+border);
      pixval = xv+yv*c;
      if(xr < border || yr < border)
	PutPixel(x,y,bc);
      else
	PutPixel(x,y,(pixval > 255) ? 255 : pixval);
    }
  }
}

void ColorImage::cacheCmap(){
  int ncolors = 256;
  int i;
  for(i=0;i<ncolors;i++){
    p[i].pixel = i;
    p[i].flags = DoRed | DoGreen | DoBlue;
  }
  XQueryColors(Dpy,currentCmap->getcmap(),p,ncolors);
}

XColor ColorImage::getPixval(int x, int y){
  if(x < 0 || y < 0 || x > width || y > height)
    return p[0];
  int xv,xr,yv,yr;
  yv = y / (rowWidth+borderWidth);
  yr = y % (rowWidth+borderWidth);  
  xv = x / (columnWidth+borderWidth);
  xr = x % (columnWidth+borderWidth);
  if(xr < borderWidth || yr < borderWidth)
    return p[borderPixval];
  int rtn = xv+yv*nc;
  if(rtn < 0)
    return p[0];
  if(rtn > 255)
    return p[255];
  return p[rtn];
}

