//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "util.h"
#include <libc.h>
#include <osfcn.h>
#include <sys/param.h>

char *CURRENT_DIRECTORY;
char *DEFAULT_BACKGROUND_FILE;
char *EFFECTIVE_DIRECTORY;
char *MESSAGE_WINDOW;
char *PRIVATE_COLORMAP;
char *XET_CURRENT_COLOR_MAP_FILE;
char *XET_DEFAULT_COLOR_MAP;
char *XET_DEFAULT_COLOR_MAP_FILE;
char *XET_ROOT_DIRECTORY;
char *XET_WORKING_DIRECTORY;

int   DEFAULT_MAX_LOCUS_POINTS;
int   MIN_HASH_TABLE_SIZE;
int   SNAPSHOT_BLINKING_COUNT;
int   XET_OVERRIDE_FLAG;
int   MAX_SLOTS_IN_CLIP_TABLE;
int   MAX_SLOTS_IN_LOCUS_TABLE;
int   MAX_SLOTS_IN_SCHEDULER;

char *CLIP_FORMAT;
char *CLIP_FORMAT_SCAN;
char *CLIP_FORMAT_SCAN_NAME;
char *LOCUS_FORMAT;
char *LOCUS_FORMAT_SCAN;
char *LOCUS_FORMAT_SCAN_NAME;
char *MOVIE_FORMAT;
char *MOVIE_FORMAT_SCAN;
char *SCHEDULER_FORMAT;
char *SCHEDULER_FORMAT_SCAN;
char *SCHEDULER_FORMAT_SCAN_NAME;
char *SPOOLER_FORMAT;
char *SPOOLER_FORMAT_SCAN;

static void read_xetrc_file(){
  char filebuf[200],buffer[200];
  char line[BUFSIZ+1];
  char *p;
  char f1[100],f2[100],f3[100];
  sprintf(filebuf,"%s/.xetrc",getenv("HOME"));
  if(FileReadable(filebuf)){
    FILE *fp = fopen(filebuf,"r");
    if(fp == NULL)
      return;
    for(;;){
      if((p = fgets(line, BUFSIZ, fp)) == NULL)
	break;     // end of file 
      if(line[0] == '#')
	continue;  // hit comments
      if(isBlankString(p) == 1)
	continue;
      sscanf(line,"%s %s %s",f1,f2,f3);
      if(strcmp(f1,"XET_ROOT_DIRECTORY") == 0){
	ALLOC_STRCPY(XET_ROOT_DIRECTORY,f3);
	continue;
      }
      if(strcmp(f1,"XET_WORKING_DIRECTORY") == 0){
	if(f3[0] == '/'){
	  ALLOC_STRCPY(XET_WORKING_DIRECTORY,f3);
	}else{
	  sprintf(buffer,"%s/%s",CURRENT_DIRECTORY,f3);
	  ALLOC_STRCPY(XET_WORKING_DIRECTORY,buffer);
	}
	continue;
      }
      if(strcmp(f1,"XET_DEFAULT_COLOR_MAP_FILE") == 0){
	if(f3[0] == '/'){
	  ALLOC_STRCPY(XET_DEFAULT_COLOR_MAP_FILE,f3);
	}else{
	  if(XET_ROOT_DIRECTORY){
	    sprintf(buffer,"%s/data/colormap/%s",XET_ROOT_DIRECTORY,f3);
	  }else{
	    sprintf(buffer,"%s",f3);
	  }
	  ALLOC_STRCPY(XET_DEFAULT_COLOR_MAP_FILE,buffer);
	}
	continue;
      }
      if(strcmp(f1,"DEFAULT_BACKGROUND_FILE") == 0){
	ALLOC_STRCPY(DEFAULT_BACKGROUND_FILE,f3);
	continue;
      }
      if(strcmp(f1,"DEFAULT_MAX_LOCUS_POINTS") == 0){
	DEFAULT_MAX_LOCUS_POINTS = atoi(f3);
	continue;
      }

      if(strcmp(f1,"XET_LOAD_OVERRIDE_FLAG") == 0){
	if(f3[0] == 'y' || f3[0] == 'Y')
	  XET_OVERRIDE_FLAG = 1;
	else
	  XET_OVERRIDE_FLAG = 0;
	continue;
      }
      if(strcmp(f1,"MESSAGE_WINDOW") == 0){
	ALLOC_STRCPY(MESSAGE_WINDOW,f3);
	continue;
      }
      if(strcmp(f1,"PRIVATE_COLORMAP") == 0){
	ALLOC_STRCPY(PRIVATE_COLORMAP,f3);
	continue;
      }
    }
    fclose(fp);
  }
}
int initialize(){
// global configureable parameters

  char buffer[MAXPATHLEN+2];

  XET_ROOT_DIRECTORY = NULL;
  XET_OVERRIDE_FLAG = 1;

  getwd(buffer);
  CURRENT_DIRECTORY = new char[strlen(buffer)+1];
  strcpy(CURRENT_DIRECTORY,buffer);

  sprintf(buffer,"%s/%s",CURRENT_DIRECTORY,"xxxet");
  XET_WORKING_DIRECTORY   = new char[strlen(buffer)+1];
  strcpy(XET_WORKING_DIRECTORY,buffer);

  XET_DEFAULT_COLOR_MAP_FILE     = NULL;
  XET_CURRENT_COLOR_MAP_FILE     = NULL;
  EFFECTIVE_DIRECTORY            = NULL;
  STRCPY(DEFAULT_BACKGROUND_FILE, "background.ppm");
  STRCPY(PRIVATE_COLORMAP,"no");
  STRCPY(MESSAGE_WINDOW,"yes");
  MAX_SLOTS_IN_CLIP_TABLE        = 200;
  MAX_SLOTS_IN_LOCUS_TABLE       = 50;
  DEFAULT_MAX_LOCUS_POINTS       = 2000;
  MAX_SLOTS_IN_SCHEDULER         = 100;
  SNAPSHOT_BLINKING_COUNT        = 6;
  MIN_HASH_TABLE_SIZE            = 10;

  STRCPY(LOCUS_FORMAT         , "   %s   <%d, %d points>");
  STRCPY(CLIP_FORMAT          , "   %-20s   (%-3d x %-3d)    <%s, %d, %s, %s>");
  STRCPY(SCHEDULER_FORMAT     , "   %-20s   %-7s    <%d, %d, %s>     ");
  STRCPY(SPOOLER_FORMAT       , "   %s -- %s");
  STRCPY(MOVIE_FORMAT         , "   %-20s   %s   %d  <%s>");
  STRCPY(MOVIE_FORMAT_SCAN    , "%s   %s   %s   %d ");
  STRCPY(LOCUS_FORMAT_SCAN    , "%s   %s   <%d, %d points>");
  STRCPY(LOCUS_FORMAT_SCAN_NAME    , "%s   %s");
  STRCPY(CLIP_FORMAT_SCAN     , "%s   %s   (%d x %d)  <%s,%s,%s>");
  STRCPY(CLIP_FORMAT_SCAN_NAME  , "%s   %s");
  STRCPY(SCHEDULER_FORMAT_SCAN, "%s   %s   %s   <%d, %d, %s > ");
  STRCPY(SCHEDULER_FORMAT_SCAN_NAME, "%s   %s  %s");
  STRCPY(SPOOLER_FORMAT_SCAN  , "%s   %s -- %s");

  // read the ~/.xetrc and set appropriated global variables
  // it may overwrite default values

  read_xetrc_file();

  if(DirectoryExistOrCreate(XET_WORKING_DIRECTORY) == -1){
    printf("can not access/create %s\n",XET_WORKING_DIRECTORY);
    return(-1);
  }
  
  if(XET_ROOT_DIRECTORY == NULL){
    char *xetroot = getenv("XET_ROOT_DIRECTORY");
    if(xetroot){
      ALLOC_STRCPY(XET_ROOT_DIRECTORY,xetroot);
    }else{
      printf("Can not find Xet root directory, use (1) $HOME/.xetrc, or (2) setenv XET_ROOT_DIRECTORY to specify Xet root directory\n");
      exit(0);
    }
  }
  
  // now adjust/specifiy the XET_DEFAULT_COLOR_MAP_FILE
  if(XET_ROOT_DIRECTORY){
    if(XET_DEFAULT_COLOR_MAP_FILE == NULL){
      sprintf(buffer,"%s/data/colormap/stdcmap0.ppm",XET_ROOT_DIRECTORY);
      if(FileReadable(buffer)){
	ALLOC_STRCPY(XET_DEFAULT_COLOR_MAP_FILE,buffer);
      }
    }else{
      if(XET_DEFAULT_COLOR_MAP_FILE[0] != '/'){
	sprintf(buffer,"%s/data/colormap/%s",XET_ROOT_DIRECTORY,XET_DEFAULT_COLOR_MAP_FILE);
	if(FileReadable(buffer))
	  ALLOC_STRCPY(XET_DEFAULT_COLOR_MAP_FILE,buffer);
      }
    }
  }
  return 1;
}


