//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "font.h"

#include <string.h>
#include <libc.h>

GFont::GFont(Display *d,int m)
:Dpy(d),maxFontInList(m),realFontName(NULL)
{
  screen = DefaultScreen(Dpy);
}
GFont::~GFont() {
  delete realFontName;
}
int GFont::Width(char *s){
  if(xfs)
    return XTextWidth(xfs,s,strlen(s));
  else
    return NULL;
}
int GFont::Height(char *){
  if(xfs)
    return xfs->ascent + xfs->descent;
  else
    return NULL;
}
int GFont::SetFont(char *pattern){
  int tcount;
  char **fonts;
  fonts = XListFonts(Dpy,pattern,maxFontInList,&tcount);
  if(fonts){
    ALLOC_STRCPY(realFontName,fonts[0]); // always pick up the first font
    XFreeFontNames(fonts);
  }else{
    // can not locate a font so default to 9x15
    ALLOC_STRCPY(realFontName,"9x15");
  }
  return 1;
}
int GFont::IsScalableFont(char *name){
  int i, field;
  if((name == NULL) || (name[0] != '-'))
    return 0;  // false
  for(i=field=0;name[i] != '\0';i++){
    if(name[i] == '-') {
      field++;
      if((field == 7) || (field == 8) || (field == 12))
	if((name[i+1] != '0') || (name[i+2] != '-'))
	  return 0; // false
    }
  }
  if(field != 14)
    return 0;
  else
    return 1;
}
XFontStruct *GFont::LoadQueryScalableFont(int size){

  int i,j,field;
  char newname[500];
  int res_x, res_y;
  xfs = 0;
  if((realFontName == NULL) || (realFontName[0] != '-'))
    goto D99;  // false
  if(IsScalableFont(realFontName)){
    // calculate screen resolution in dots per inch 25.4mm = i inch
    res_x = (int)(DisplayWidth(Dpy, screen)/(DisplayWidthMM(Dpy,screen)/25.4));
    res_y = (int)(DisplayHeight(Dpy, screen)/(DisplayHeightMM(Dpy,screen)/25.4));
    // copy the font name, changing the scalable fields as we do soooo
    for(i=j=field=0;realFontName[i] != '\0' && field <= 14; i++){
      newname[j++] = realFontName[i];
      if(realFontName[i] == '-'){
	field++;
	switch(field){
	case 7: // pixel size
	case 12:  // average width
	  // change from "-0-" to "-*-"
	  newname[j] = '*';
	  j++;
	  if(realFontName[i+1] != '\0')
	    i++;
	  break;
	case 8: // point size
	  //change from -0- to -<size>-
	  sprintf(&newname[j], "%d", size);
	  while(newname[j] != '\0')
	    j++;
	  if(realFontName[i+1] != '\0')
	    i++;
	  break;
	case 9: // x-resolution
	case 10: // y-resolution
	  // change from an unspecified resolution to res_x or res_y
	  sprintf(&newname[j], "%d", (field==9) ? res_x : res_y);
	  while(newname[j] != '\0') j++;
	  while((realFontName[i+1] != '-') && (realFontName[i+1] != '\0'))
	    i++;
	  break;
	}
      }
    }
    newname[j] = '\0';
    // if there aren't 14 field, it isn't a well formed name
    if(field != 14)
      goto D99;
    xfs = XLoadQueryFont(Dpy,newname);
  }else{
    xfs = XLoadQueryFont(Dpy,realFontName);
  }
 D99:
  if(xfs == 0)
    xfs = XLoadQueryFont(Dpy,"9x15");
  return xfs;
}



