//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

// This section of code is orginiated from hsearch.c in BSD distribution
// Since I need to have 
//    1. multiple hash tables in the same process
//    2. to be able to delete items in the hash table
//    3. to be able to grow the size of the table when the table is full
//    4. I need it is in C++ 
// I have to modify the hsearch.c code
//
//                              Ko-Haw Nieh


#include "hash.h"
#include <libc.h>
#include <string.h>

Hash::Hash(int size)
:count(0),first(1)
{
  initialize(size);
}
void Hash::initialize(int size)
{
  unsigned int unsize;	/* Holds the shifted size */
  unsize = size;	/* +1 for empty table slot; -1 for ceiling */
  length = 1;		/* Maximum entries in table */
  m = 0;		/* Log2 length */
  while(unsize) {
    unsize >>= 1;
    length <<= 1;
    m++;
  }
  table = new TABELEM [length * sizeof(TABELEM)];
  // must clean the table first
  for(int i=0;i<length;i++){
    table[i].key = NULL;
    table[i].data = NULL;
  }
}
Hash::~Hash()
{
  delete table;
}
void Hash::resize(){
  resize(2*length);
}
void Hash::resize(int size){
  // save the old table and length
  TABELEM  *old_table = table;
  unsigned int old_length = length;
  count = 0;
  first = 1;
  initialize(size);
  // move the stuff from old to new table
  for(int i = 0; i < old_length; i++){
    if(old_table[i].key == NULL)
      continue;
    hsearch(old_table[i], ENTER_E);
  }
  delete old_table;
}

ENTRY *Hash::hsearch(ENTRY item, ACTION_E action)
{
  unsigned int i;	/* Insertion index */
  unsigned int c;	/* Secondary probe displacement */
  
  prcnt = 1;
  
  /* D1: */ 
  i = hashm(item.key);	/* Primary hash on key */
  
  /* D2: */
  if(table[i].key == NULL)	/* Empty slot? */
    goto D6;
  else if(strcmp(table[i].key, item.key) == 0){	/* Match? */
    if(action == DELETE_E){
      count--;
      // caller is responsible to delete this item
      TABELEM *rtn = new TABELEM;
      rtn->key =  table[i].key;
      rtn->data = table[i].data;
      table[i].key = NULL;	
      table[i].data = NULL;
      return(rtn);
    }else
      return(&table[i]);
  }
  /* D3: */
  c = hash2m(item.key);	/* No match => compute secondary hash */
 D4: 
  i = (i + c) % length;	/* Advance to next slot */
  prcnt++;
  
  /* D5: */
  if(table[i].key == NULL)	/* Empty slot? */
    goto D6;
  else if(strcmp(table[i].key, item.key) == 0){	/* Match? */
    if(action == DELETE_E){
      count--;
      // caller is responsible to delete this item
      TABELEM *rtn = new TABELEM;
      rtn->key =  table[i].key;
      rtn->data = table[i].data;
      table[i].key = NULL;	
      table[i].data = NULL;
      return(rtn);
    }else
      return(&table[i]);
  }else
    goto D4;
  
 D6: 
  if(action == FIND_E || action == DELETE_E)	/* Insert if requested */
    return((ENTRY *) NULL);
  if(count == (length - 1)){	/* Table full? */
    //return((ENTRY *) 0);
    printf("hash table is full, double the size\n");
    resize();
    return hsearch(item,action);
  }
  count++;			/* Increment table occupancy count */
  table[i].key = item.key;		/* Save item */
  table[i].data = item.data;
  return(&table[i]);		/* Address of item is returned */
}


/*
    NOTE: The following algorithm only works on machines where
    the results of multiplying two integers is the least
    significant part of the double word integer required to hold
    the result.  It is adapted from Knuth, Volume 3, section 6.4.
*/

unsigned int Hash::hashm(char *key)	
{
    //static int first = 1;	/* TRUE on the first call only */

    if(first) {		/* Compute the number of bits in a byte */
	unsigned char c = ~0;	/* A byte full of 1's */
	bitsper = 0;
	while(c) {		/* Shift until no more 1's */
	    c >>= 1;
	    bitsper++;		/* Count number of shifts */
	}
	first = 0;
    }
    return((int) (((unsigned) (crunch(key) * FACTOR)) >> SHIFT));
}

/*
 * Secondary hashing, for use with multiplicitive hashing scheme.
 * Adapted from Knuth, Volume 3, section 6.4.
 */

unsigned int Hash::hash2m(char *key)	
{
    return((int) (((unsigned) ((crunch(key) * FACTOR) << m) >> SHIFT) | 1));
}

unsigned int Hash::crunch(char *key)
{
    unsigned int sum = 0;	/* Results */
    int s;			/* Length of the key */

    for(s = 0; *key; s++)	/* Simply add up the bytes */
	sum += *key++;

    return(sum + s);
}

int Hash::occupy(){
  return count;
}
void Hash::hdump()
{
    unsigned int i;	/* Counts table slots */
    unsigned int sum = 0;	/* Counts probes */
    for(i = 0; i < length; i++)
	if(table[i].key == NULL)
	    printf("%o.\t-,\t-,\t(NULL)\n", i);
	else {
	    unsigned int oldpr = prcnt; /* Save current probe count */
	    hsearch(table[i], FIND_E);
	    sum += prcnt;
	    printf("%o.\t%d,\t%d,\t%s\n", i,
		*table[i].data, prcnt, table[i].key);
	    prcnt = oldpr;
	}
    printf("Total probes = %d\n", sum);

}
