//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

// This section of code is orginiated from hsearch.c in BSD distribution
// Since I need to have the following
//    1. multiple hash tables in the same process
//    2. to be able to delete items in the hash table
//    3. to be able to grow the size of the table when the table is full
//    4. I need it is in C++ 
// I have to modify the hsearch.c code
//
//                              Ko-Haw Nieh


#ifndef HASH_H
#define HASH_H

#define SHIFT ((bitsper * sizeof(int)) - m) /* Shift factor */
#define FACTOR 035761254233	/* Magic multiplication factor */

typedef enum {
    FIND_E,		/* Find, if present */
    ENTER_E,		/* Find; enter if not present */
    DELETE_E,
} ACTION_E;

class ENTRY {	/* Hash table entry */
public:
    char *key;
    char *data;
};

typedef ENTRY TABELEM;	/* What the table contains (TABle ELEMents) */

class Hash {
protected:
 unsigned int bitsper;	/* Bits per byte */
 unsigned int hashm(char *key);
 unsigned int hash2m(char *key);
 unsigned int crunch(char *key);
 TABELEM  *table;	/* The address of the hash table */
 unsigned int count;	/* Number of entries in hash table */
 unsigned int length;	/* Size of the hash table */
 unsigned int m;		/* Log base 2 of length */
 unsigned int prcnt;	/* Number of probes this item */
 int          first;
public:
  Hash(int size);
  ~Hash();
  void   initialize(int size);
  ENTRY *hsearch(ENTRY item, ACTION_E action);
  int    occupy();
  void   resize();
  void   resize(int);
  void   hdump();
  
};

#endif
