//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "patchlevel.h"
#include "xet.h"
#include "config.h"
#include "util.h"
#include <libc.h>
#include <osfcn.h>

// This project started on April 16, 1992.
// The objective is to create an environment/tool to create motion pictures.
// The original name of the project was xmotion.
// May 1992 -- experiment/study images and their motion behavior under X11R5
// July   2, 1992 major restructure, rename it to Xet, upgrade to Wcl-2.21
// Sept. 18, 1992 most of the major functionalities were in place
// Sept. 25, 1992 started writing documentation
// Oct.   1, 1992 started wrapping up
// Nov.  10, 1992 started making demo sessions
// Dec.   2, 1992 upgrade to Wcl-2.3

static char *resourceName = "XET";
Xet   *XET;

static void parseArgs(int argc, char **argv);

extern int initialize();

main (int argc, char* argv[] ) 
{
  parseArgs(argc, argv);

  if(initialize() < 0){
    printf("initialization failed.\n");
    exit(-1);
  }
  // let WCL pick up whatever it needs from argc,argv
  XET = new Xet(resourceName, argc, argv);
  // let Xet process rest of the command line arguments
  XET->processCommandLine(argc, argv);
  // enter infinite loop
  XET->run();
}


void usage ()
{
  printf("Usage: xet [-rf file] [-v] [-u] [-h] [-s obj] [-S] [[xet scripts]...]\n");
  printf("-rf pathname -- specify the X resource file used by WCL\n");
  printf("-v           -- prints the xet version\n");
  printf("-h           -- prints this usage\n");
  printf("-u           -- prints this usage\n");
  printf("-n           -- don't run animation objects in the Xet logo\n");
  printf("-s  obj_list -- list of animation objects to be scheduled after initialization,\n");
  printf("                all object names are separated by ',',\n");
  printf("                no white space is allowed in the obj_list\n");
  printf("-S           -- schedule all available animation objects after initialization\n");
  printf("xet scripts  -- one or more Xet scripts separated by white spaces\n");
  printf("Example:  xet -S /u2/nieh/xet/demo/dog/dog_run.xet \n");
  printf("          xet -s cat1,cat2,cat3 /u2/nieh/xet/demo/running/cat_run.xet \n");
  printf("Remark: All unknown options are ignored with no warning message.\n");
}
 
static void parseArgs(int argc, char **argv)
{
  int i;
  
  argv++; //Skip over program name
  argc--;
  
  for (i= 0; argc > 0; argc--, i++)
    {
      switch(argv[i][0])
	{
	case '-':
	  switch(argv[i][1])
	    {
	    case 'v':
	      printf("xet: Version %d.%d (patch level %d)\n",
		     MAJORVERSION,MINORVERSION,PATCHLEVEL);
	      exit(0);
	      break;
	    case 'u':
	    case 'h':
	      usage();
	      exit(0);
	    default:
	      //printf("Illegal option\n");
	      //usage();
	      //exit(1);
	      break;
	    }/* switch */
	  break;
	  
	default:
	  //printf("Illegal option character\n");
	  //usage();
	  //exit(1);
	  break;
	}/* switch */
      
    }/* for */
  
}/* parse_args() */


