//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "outputWindow.h"

OutputWindow::OutputWindow(Widget da)
:da_width(0),da_height(0),da_area(da),Dpy(0),backgroundImage(NULL)
{
  if(da_area)
    Dpy = XtDisplay(da_area);
  updateGeometryAndImage();
}
OutputWindow::OutputWindow(Widget da,XImage *img)
:da_width(0),da_height(0),da_area(da),Dpy(0),backgroundImage(NULL)
{
  if(da_area)
    Dpy = XtDisplay(da_area);
  int           da_x,da_y;
  unsigned int  da_border_width,da_depth;
  Window        da_root;
  XGetGeometry(Dpy,XtWindow(da_area),&da_root,&da_x,&da_y,
	       &da_width,&da_height,&da_border_width,&da_depth);
  window = XtWindow(da_area);
  Pixmap pix = XCreatePixmap(Dpy,window,da_width,da_height,8);
  GC            loc_gc;
  XGCValues     gvals;
  loc_gc = XCreateGC(Dpy,pix, 0, &gvals);
  XPutImage(Dpy,pix,loc_gc,img,0,0,0,0,da_width,da_height);
  setImage(pix);
  XFreePixmap(Dpy,pix);
  XFreeGC(Dpy,loc_gc);
}
OutputWindow::OutputWindow(Display *d)
:da_width(0),da_height(0),da_area(0),Dpy(d),backgroundImage(NULL)
{
  // it must be root window, so capture whatever on it
  da_width  = DisplayWidth(Dpy,DefaultScreen(Dpy));
  da_height = DisplayHeight(Dpy,DefaultScreen(Dpy));
  window    = RootWindow(Dpy,DefaultScreen(Dpy));
  reCaptureRootWindow();
}

OutputWindow::~OutputWindow(){

}

void OutputWindow::updateGeometryAndImage(){
  if(da_area){
    int           da_x,da_y;
    unsigned int  da_border_width,da_depth;
    Window        da_root;
    XGetGeometry(Dpy,XtWindow(da_area),&da_root,&da_x,&da_y,
		 &da_width,&da_height,&da_border_width,&da_depth);
    window = XtWindow(da_area);
  }else{
    // it is root window
    da_width  = DisplayWidth(Dpy,DefaultScreen(Dpy));
    da_height = DisplayHeight(Dpy,DefaultScreen(Dpy));
    window    = RootWindow(Dpy,DefaultScreen(Dpy));
  }
  clearDrawingArea();
}

void OutputWindow::destroyBackground(){
  if(backgroundImage)
    XDestroyImage(backgroundImage);
  clearDrawingArea();
}

void OutputWindow::setImage(Pixmap pix){
  if(backgroundImage != NULL)
    XDestroyImage(backgroundImage);
  backgroundImage = XGetImage(Dpy,pix,0,0,da_width,da_height,0xff,ZPixmap);
  XSetWindowBackgroundPixmap(Dpy,window,pix);
}

void OutputWindow::putImage(GC gc){
  XPutImage(Dpy, window, gc, backgroundImage,0,0,0,0,da_width,da_height); 
}

void OutputWindow::reCaptureRootWindow(){
  if(backgroundImage != NULL)
    XDestroyImage(backgroundImage);
  backgroundImage = XGetImage(Dpy,window,0,0,da_width,da_height,0xff,ZPixmap);
}

void OutputWindow::clearDrawingArea(){
  Pixmap pix = XCreatePixmap(Dpy,window,da_width,da_height,8);
  GC            loc_gc;
  XGCValues     gvals;
  loc_gc = XCreateGC(Dpy,pix, 0, &gvals);
  XSetForeground(Dpy,loc_gc,WhitePixel(Dpy,DefaultScreen(Dpy)));
  XFillRectangle(Dpy,pix,loc_gc,0,0,da_width,da_height);
  backgroundImage = XGetImage(Dpy,pix,0,0,da_width,da_height,0xff,ZPixmap);
  XSetWindowBackgroundPixmap(Dpy,window,pix);
  XFreePixmap(Dpy,pix);
  XFreeGC(Dpy,loc_gc);
}


