//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "run_base.h"
#include "solidImage.h"
#include "util.h"
#include <libc.h>
#include "cutbgImage.h"
#include "xet.h"

extern Xet           *XET;

RunBase::RunBase(Display *d,OutputWindow *da,ClipList *cl, RunMode rm)
:terminationCriteria(NOT_APPLICATBLE_E),tc_max_moves(0),
nextMoveCounter(0),intervalIndicator(0),tc_max_interval(0),intervalXid(0),
interval(0),xid(0),status(0),name(NULL),next(NULL),prev(NULL),
tc_x(0),tc_y(0),tc_width(0),tc_height(0),clipList(NULL),
current_x(0),current_y(0),drawingArea(da)
{
  Dpy          = d;
  runMode      = rm;
  clipList     = cl;
  if(clipList){
    Clip *im     = cl->nextClip();
    clip_width   = im->Width();
    clip_height  = im->Height();
    gc           = im->GetGC(); 
    cl->reset();
  }
}

RunBase::~RunBase(){
  if(intervalXid != 0){
    XtRemoveTimeOut(intervalXid);
    intervalXid = 0;
  }
}
void  RunBase::setOutputWindow(OutputWindow *w){
  drawingArea = w;
}
OutputWindow *RunBase::getOutputWindow(){
  return drawingArea;
}
void  RunBase::setNextClip(){
  clip = clipList->nextClip();
  clip_width   = clip->Width();
  clip_height  = clip->Height();
  gc           = clip->GetGC(); 

}
RunMode RunBase::getRunMode(){
  return runMode;
}

void RunBase::setclipList(ClipList *cl){
  if(clipList)
    delete clipList;
  clipList = cl;
}

void RunBase::incrRefcounter(){
  if(clipList)
    clipList->incrRefCounter();
}

void RunBase::decrRefCounter(){
  if(clipList)
    clipList->decrRefCounter();
}

void  RunBase::writeOutTermCond(FILE *fp){
  switch(terminationCriteria){
  case NOT_APPLICATBLE_E : 
    fprintf(fp,":not_applicatble::");
    break;
  case MAX_MOVES_E:        
    fprintf(fp,":max_moves:%d:",tc_max_moves);
    break;
  case POSITION_E :        
    fprintf(fp,":position:%d %d %d %d:",tc_x,tc_y,tc_width,tc_height); 
    break;
  case WALL_CLOCK_E:       
    fprintf(fp,":wall_clock::"); 
    break;
  case TIME_INTERVAL_E:    
    fprintf(fp,":time_interval:%d:",tc_max_interval); 
    break;
  }
}

char *RunBase::getTermCondInString(){
  return getTermCondInString(terminationCriteria);
}

char *RunBase::getTermCondInString(TermCriteria tc){
  switch(tc){
  case NOT_APPLICATBLE_E : return "not_applicatble";
  case MAX_MOVES_E:        return "max_moves";
  case POSITION_E :        return "position";
  case WALL_CLOCK_E:       return "wall_clock";
  case TIME_INTERVAL_E:    return "time_interval";
  }
}

char *RunBase::getTermCondValueInString(){
  return getTermCondValueInString(terminationCriteria);
}

char *RunBase::getTermCondValueInString(TermCriteria tc){
  char buffer[200];
  switch(tc){
  case NOT_APPLICATBLE_E :
    sprintf(buffer, "not_applicatble"); break;
  case MAX_MOVES_E:        
    sprintf(buffer, "max_moves = %d",tc_max_moves); 
    break;
  case POSITION_E :        
    sprintf(buffer, "position = (%d %d %d %d)",tc_x,tc_y,tc_width,tc_height); 
    break;
  case WALL_CLOCK_E:       
    sprintf(buffer, "wall_clock = (not implemented yet)"); 
    break;
  case TIME_INTERVAL_E:    
    sprintf(buffer, "time_interval = %d",tc_max_interval);
    break;
  }
  char *rtn = new char[strlen(buffer+1)];
  strcpy(rtn,buffer);
  return rtn;
}

char *RunBase::getRunModeInString(){
  switch(runMode){
  case Fixed_E:  return "Fixed";
  case Glide_E: return "Glide";
  case Random_E: return "Random";
  case Fly_E:    return "Fly";
  case Blink_E:  return "Blink";
  case Locus_E:  return "Locus";
  case Movie_E:  return "Movie";		 
  }
}

TrailingMode RunBase::getTrailingMode(){
  return trailingMode;
}

BoundaryMode RunBase::getBoundaryMode(){
  return  Wrap_E;  //just to keep compiler happy and quiet
}

void RunBase::setTrailingMode(TrailingMode opt){
  trailingMode = opt;
}

Clip *RunBase::getClip(){
  return clip;
}

void RunBase::setName(char *s){
  ALLOC_STRCPY(name,s);
}

char *RunBase::getName(){
  return name;
}

void RunBase::reverseClip(){
  if(clipList)
    clipList->reverse();
}

void RunBase::reverse(){
}

char *RunBase::trailingModeInString(TrailingMode tm){
  switch(tm){
  case Preserve_E: return "Preserve";
  case Wipeout_E:  return "Wipeout";
  case Leave_E:    return "Leave";
  case Rainbow_E:  return "Rainbow";
  }
}

char *RunBase::boundaryModeInString(BoundaryMode bm){
  switch(bm){
  case Wrap_E:       return "Wrap";
  case BounceBack_E: return "BounceBack";
  case Disappear_E:  return "Disappear";
  }
}

void RunBase::resetTerminationCond(){
  switch(terminationCriteria){
  case NOT_APPLICATBLE_E : 
    break;
  case MAX_MOVES_E:
    nextMoveCounter = 0;
    break;
  case POSITION_E :
    break;
  case WALL_CLOCK_E:
    break;
  case TIME_INTERVAL_E:
    if(intervalXid != 0){
      XtRemoveTimeOut(intervalXid);
    }
    intervalIndicator = 1;
    intervalXid = XtAppAddTimeOut(XET->app, 
				  tc_max_interval,
				  (XtTimerCallbackProc) RunBase::work, 
				  (XtPointer *) this);
    break;
  }
}

void RunBase::work(XtPointer xptr, XtIntervalId *){
  RunBase *rb = (RunBase *) xptr;
  rb->intervalIndicator = 0;
  rb->intervalXid = 0;
}

int RunBase::shouldTerminate(){
  switch(terminationCriteria){
  case NOT_APPLICATBLE_E : 
    break;
  case MAX_MOVES_E:
    nextMoveCounter++;
    if(nextMoveCounter >= tc_max_moves)
      return 1;
    break;
  case POSITION_E :
    if(current_x >= tc_x && current_x <= tc_x+tc_width &&
       current_y >= tc_y && current_y <= tc_y+tc_height)
      return 1;
    break;
  case WALL_CLOCK_E:
    break;
  case TIME_INTERVAL_E:
    if(intervalIndicator == 0)
      return 1;
    break;
  }
  return 0;
}

TermCriteria RunBase::getTermCriteria(){
  return terminationCriteria;
}

void RunBase::setTermCriteria(char *t, char *s){
  if(t == NULL || s == NULL) 
    return;
  TermCriteria tc = NOT_APPLICATBLE_E;
  if(strcmp(t,"not_applicatble") == 0)  tc = NOT_APPLICATBLE_E; 
  if(strcmp(t,"max_moves") == 0)        tc = MAX_MOVES_E; 
  if(strcmp(t,"position") == 0)         tc = POSITION_E;
  if(strcmp(t,"wall_clock") == 0)       tc = WALL_CLOCK_E; 
  if(strcmp(t,"time_interval") == 0)    tc = TIME_INTERVAL_E;
  setTermCriteria(tc,s);
}

void RunBase::setTermCriteria(TermCriteria tc, char *s){
  terminationCriteria = tc;
  switch(tc){
  case NOT_APPLICATBLE_E : break;
  case MAX_MOVES_E:
    if(s)
      sscanf(s,"%d",&tc_max_moves);
    break;
  case POSITION_E :
    if(s)
      sscanf(s,"%d %d %d %d",&tc_x,&tc_y,&tc_width,&tc_height);
    break;
  case WALL_CLOCK_E:
    break;
  case TIME_INTERVAL_E:
    if(s)
      sscanf(s,"%d",&tc_max_interval);
    break;
  }
}

// virtual functions
int RunBase::nextMove(){
  nextMoveCounter++;
  return 0;
}

void RunBase::writeToDisk(FILE *fp){
  fprintf(fp,"*****************************************************\n");
  fprintf(fp,"******* This line should never be printed out *******\n");
  fprintf(fp,"*****************************************************\n");
}

void RunBase::reset(){
  nextMoveCounter= 0;
}

void RunBase::start(int , int , int , int ){
  if(clipList)
    clipList->reset();
}

void RunBase::setBoundingBox(int,int,int,int){}
void RunBase::setBoundaryMode(BoundaryMode){}
void RunBase::setLocus(Locus *){}
int  RunBase::setFlyEquation(char *,char *,char *){ return 1;}
int  RunBase::setFlyEquationI(char *,char *,int,int,int,int){ return 1;}
void RunBase::setTranslation(int , int ){}
RunBase *RunBase::Copy(){
  // This function should never be called
  return new RunBase(*this);
}

void RunBase::placeToNewLocation(int rx, int ry) {
  switch(trailingMode){
  case Leave_E : 
    x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    break;
  case Wipeout_E : 
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,clip_height,0,0,0,0);
    x_trailing_image->DisplayImage(drawingArea,current_x,
				   current_y,clip_width,clip_height);
    break;
  case Rainbow_E : 
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,clip_height, rand() % 255 ,0,0,0);
    x_trailing_image->DisplayImage(drawingArea,current_x,
				   current_y,clip_width,clip_height);
    break;
  case Preserve_E:
    if(drawingArea->backgroundImage != NULL){
      x_trailing_image = new CutbgImage(Dpy,drawingArea->da_area,drawingArea->backgroundImage);
      x_trailing_image->SetGC(gc);
      x_trailing_image->MakeImage(clip_width,clip_height,current_x,current_y,0,0);
      x_trailing_image->DisplayImage(drawingArea,current_x,
				     current_y,clip_width,clip_height);
    }else
       x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    break;
  }
  current_x = rx;
  current_y = ry;
  clip->DisplayImage(drawingArea, rx, ry);

  if(x_trailing_image)
    delete x_trailing_image;
}

// it is used by blinking mode and glide mode when dx > clip_width and
// d_y > clip_height
void RunBase::jump(int ) {
  switch(trailingMode){
  case Leave_E : 
    x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    break;
  case Wipeout_E : 
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,clip_height,0,0,0,0);
    x_trailing_image->DisplayImage(drawingArea,current_x,
				   current_y,clip_width,clip_height);
    break;
  case Rainbow_E : 
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,clip_height,rand() % 255 ,0,0,0);
    x_trailing_image->DisplayImage(drawingArea,current_x,
				   current_y,clip_width,clip_height);
    break;
  case Preserve_E:
    if(drawingArea->backgroundImage != NULL){
      x_trailing_image = new CutbgImage(Dpy,drawingArea->da_area,drawingArea->backgroundImage);
      x_trailing_image->SetGC(gc);
      x_trailing_image->MakeImage(clip_width,clip_height,current_x, current_y,0,0);
    }else
      x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    break;
  }
  
  x_trailing_image->DisplayImage(drawingArea,current_x,
				 current_y,clip_width,clip_height);
  current_x += d_x;
  current_y += d_y;
  // determinate the next direction
  if(current_x + clip_width  > ((int)drawingArea->da_width)) d_x = - d_x;
  if(current_y + clip_height > ((int)drawingArea->da_height)) d_y = - d_y;
  if(current_x < 0)  d_x = - d_x;
  if(current_y < 0)  d_y = - d_y;
  
  if(current_x > ((int)drawingArea->da_width))  current_x = drawingArea->da_width;
  if(current_y > ((int)drawingArea->da_height)) current_y = drawingArea->da_height;
  if(current_x < 0) current_x = 0;
  if(current_y < 0) current_y = 0;

  if(x_trailing_image)
    delete x_trailing_image;
}

// rx and ry are old location
// current_x and current_y were updated by the caller
void RunBase::takeCareOfOldLocation(int rx, int ry) {
  switch(trailingMode){
  case Leave_E : 
    x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    break;
  case Wipeout_E : 
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,clip_height,0,0,0,0);
    x_trailing_image->DisplayImage(drawingArea,rx,
				   ry,clip_width,clip_height);
    break;
  case Rainbow_E : 
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,clip_height,rand() % 255 ,0,0,0);
    x_trailing_image->DisplayImage(drawingArea,rx,
				   ry,clip_width,clip_height);
    break;
  case Preserve_E:
    if(drawingArea->backgroundImage != NULL){
      x_trailing_image = new CutbgImage(Dpy,drawingArea->da_area,drawingArea->backgroundImage);
      x_trailing_image->SetGC(gc);
      x_trailing_image->MakeImage(clip_width,clip_height,rx,ry,0,0);
      x_trailing_image->DisplayImage(drawingArea,rx,ry,clip_width,clip_height);
    }else
      x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    break;
  }
  if(x_trailing_image)
    delete x_trailing_image;
}

void RunBase::takeCareOfTrailingImage()
{
  switch(trailingMode){
  case Leave_E : 
    x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    y_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    break;
  case Rainbow_E :
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,abs(d_y),rand() % 255,0,0,0);
    y_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    y_trailing_image->SetGC(gc);
    y_trailing_image->MakeImage(abs(d_x), clip_height,rand() % 255,0,0,0);
    break;
  case Wipeout_E :
    x_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    x_trailing_image->SetGC(gc);
    x_trailing_image->MakeImage(clip_width,abs(d_y),0,0,0,0);
    y_trailing_image = new SolidImage(Dpy,drawingArea->da_area);
    y_trailing_image->SetGC(gc);
    y_trailing_image->MakeImage(abs(d_x), clip_height,0,0,0,0);
    break;
  case Preserve_E:
    if(drawingArea->backgroundImage != NULL){
      x_trailing_image = new CutbgImage(Dpy,drawingArea->da_area,drawingArea->backgroundImage);
      x_trailing_image->SetGC(gc);
      y_trailing_image = new CutbgImage(Dpy,drawingArea->da_area,drawingArea->backgroundImage);
      y_trailing_image->SetGC(gc);
    }else{
      x_trailing_image  = new Clip(Dpy,drawingArea->da_area);
      y_trailing_image  = new Clip(Dpy,drawingArea->da_area);
    }
    break;
  }
  
  // wipe out trailing pixels
  if(d_x > 0 && d_y == 0) {
    if(trailingMode == Preserve_E){
      y_trailing_image->MakeImageAndDisplay(drawingArea,abs(d_x), clip_height,
					    current_x - d_x,
					    current_y ,0,0);
    }else
      y_trailing_image->DisplayImage(drawingArea,current_x - d_x,
				     current_y,abs(d_x), clip_height);
  }
  if(d_x < 0 && d_y == 0){
    if(trailingMode == Preserve_E){
      y_trailing_image->MakeImageAndDisplay(drawingArea,abs(d_x), 
					    clip_height,current_x + clip_width,
					    current_y ,0,0);
    }else
      y_trailing_image->DisplayImage(drawingArea,current_x + clip_width,
				     current_y,abs(d_x), clip_height);
  }
  if(d_x == 0 && d_y > 0){
    if(trailingMode == Preserve_E){
      x_trailing_image->MakeImageAndDisplay(drawingArea,clip_width,
					    abs(d_y),current_x,
					    current_y - d_y ,0,0);
    }else
      x_trailing_image->DisplayImage(drawingArea,current_x,
				     current_y - d_y,clip_width,abs(d_y));
  }
  if(d_x == 0 && d_y < 0){
    if(trailingMode == Preserve_E){
      x_trailing_image->MakeImageAndDisplay(drawingArea,clip_width,abs(d_y),
					    current_x,
					    current_y + clip_height ,0,0);
    }else
      x_trailing_image->DisplayImage(drawingArea,current_x,
				     current_y + clip_height ,clip_width,abs(d_y));
  }
  
  if(d_x > 0 && d_y > 0) {
    if(trailingMode == Preserve_E){
      x_trailing_image->MakeImageAndDisplay(drawingArea,clip_width,abs(d_y),
					    current_x - d_x,
					    current_y - d_y ,0,0);
      y_trailing_image->MakeImageAndDisplay(drawingArea,abs(d_x), clip_height,
					    current_x - d_x,
					    current_y - d_y ,0,0);
    }else{
      x_trailing_image->DisplayImage(drawingArea,current_x - d_x,
				     current_y - d_y,clip_width,d_y);
      y_trailing_image->DisplayImage(drawingArea,current_x - d_x,
				     current_y - d_y,d_x,clip_height);
    }
  }
  if(d_x < 0 && d_y < 0){
    if(trailingMode == Preserve_E){
      x_trailing_image->MakeImageAndDisplay(drawingArea,clip_width,abs(d_y),
					    current_x - d_x,
					    current_y + clip_height ,0,0);
      y_trailing_image->MakeImageAndDisplay(drawingArea,abs(d_x), clip_height,
					    current_x + clip_width,
					    current_y - d_y ,0,0);
    }else{
      y_trailing_image->DisplayImage(drawingArea,current_x + clip_width,
				     current_y - d_y,abs(d_x),clip_height);
      x_trailing_image->DisplayImage(drawingArea,current_x - d_x,
				     current_y + clip_height,clip_width,abs(d_y));
    }
  }
  if(d_x < 0 && d_y > 0){
    if(trailingMode == Preserve_E){
      x_trailing_image->MakeImageAndDisplay(drawingArea,clip_width,abs(d_y),
					    current_x - d_x,
					    current_y - d_y ,0,0);
      y_trailing_image->MakeImageAndDisplay(drawingArea,abs(d_x), clip_height,
					    current_x + clip_width,
					    current_y - d_y ,0,0);
    }else{
      x_trailing_image->DisplayImage(drawingArea,current_x - d_x,
				     current_y - d_y,clip_width,d_y);
      y_trailing_image->DisplayImage(drawingArea,current_x + clip_width,
				     current_y - d_y,abs(d_x),clip_height);
    }
  }
  if(d_x > 0 && d_y < 0){
    if(trailingMode == Preserve_E){
      x_trailing_image->MakeImageAndDisplay(drawingArea,clip_width,abs(d_y),
					    current_x - d_x,
					    current_y + clip_height ,0,0);
      y_trailing_image->MakeImageAndDisplay(drawingArea,abs(d_x), clip_height,
					    current_x - d_x,
					    current_y - d_y ,0,0);
    }else{
      x_trailing_image->DisplayImage(drawingArea,current_x - d_x,
				     current_y + clip_height,clip_width,
				     abs(d_y));
      y_trailing_image->DisplayImage(drawingArea,current_x - d_x,
				     current_y - d_y,d_x,clip_height);
    }
  }
  if(x_trailing_image)
    delete x_trailing_image;
  if(y_trailing_image)
    delete y_trailing_image;
}    

// This function is not used by Xet for this release
// In the future, I meay need to have this function
// what this function does is when two glide objects collide
// they bounce back.
void RunBase::isCollide(RunBase *c){
  int ddx,ddy;
  ddx = (abs(d_x) + abs(c->d_x)) ;
  ddy = (abs(d_y) + abs(c->d_y)) ;

  // satisfy the following means two objects collides
  if((((current_x < c->current_x) && ((current_x + clip_width) > c->current_x)) ||
     ((current_x >= c->current_x) && ((c->current_x + c->clip_width) > current_x))) 
     && 
     (((current_y < c->current_y) && ((current_y + clip_height) > c->current_y)) ||
     ((current_y >= c->current_y) && ((c->current_y + c->clip_height) > current_y))))
    {
      // satisfy the following means collide between horizonal interface
      if(((current_x < c->current_x) && 
	  ((current_x + clip_width - ddx) > c->current_x))  
	 ||
	 ((current_x >= c->current_x) && 
	  ((c->current_x + c->clip_width - ddx) > current_x))){

	    d_y = - d_y;
	    c->d_y = - c->d_y;
	  }
      // satisfy the following means collide between vertival interface
      if(((current_y < c->current_y) &&
	  ((current_y + clip_height - ddy) > c->current_y))
	 ||
	 ((current_y >= c->current_y) &&
	  ((c->current_y + c->clip_height - ddy) > current_y))) { 
	    d_x = - d_x;
	    c->d_x = - c->d_x;
	  }
    }
  
}

