//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "xet.h"
#include "run_fly.h"
#include "util.h"
#include <libc.h>
#include "scheduler.h"
#include "config.h"

extern Xet       *XET;

RunFly::RunFly(Display *d,OutputWindow *w,ClipList *im)
     :RunBase(d,w,im,Fly_E),Xequation(NULL),Yequation(NULL),dir(FORWARD_E)
{
}

RunBase *RunFly::Copy(){
  RunFly *rf =  new RunFly(*this);
  rf->Xequation = new Math_Expr(Xequation->getExprInString());
  rf->Yequation = new Math_Expr(Yequation->getExprInString());
  rf->clipList = new ClipList(*clipList);
  STRCPY(rf->name,name);
  return rf;
}

RunFly::~RunFly(){
  if(Xequation)
    delete Xequation;
  if(Yequation)
    delete Yequation;
  if(clipList) delete clipList;
  if(name) delete name;
  if(intervalXid != 0){
    XtRemoveTimeOut(intervalXid);
    intervalXid = 0;
  } 
}

void RunFly::setBoundaryMode(BoundaryMode b){
  boundaryMode = b;
}

BoundaryMode RunFly::getBoundaryMode(){
  return boundaryMode;
}

void RunFly::writeToDisk(FILE *fp){
  if(fp){
    fprintf(fp,"fly:");
    clipList->writeOut(fp);
    fprintf(fp,":%s:%s:%s:%s:%d:%d:%d:%d:%d",
	    trailingModeInString(trailingMode),
	    boundaryModeInString(boundaryMode),
	    Xequation->getExprInString(),
	    Yequation->getExprInString(),
	    fly_start_save,
	    fly_end_save,
	    fly_incr_save,
	    fly_current_save,
	    interval);
    writeOutTermCond(fp);
    fprintf(fp,"\n");
  }
}

int RunFly::setFlyEquation(char *Field1,char *Field2,char *Field3){
  int a,b,c,d;
  sscanf(Field3,"%d,%d,%d,%d",&a,&b,&c,&d);
  return(setFlyEquationI(Field1,Field2,a,b,c,d));
}

int RunFly::setFlyEquationI(char *Field1,char *Field2,
			 int a, int b, int c, int d){
  fly_start   = a;
  fly_end     = b;
  fly_incr    = c;
  fly_current = d;
  Math_Expr *oldXequation = Xequation;
  Math_Expr *oldYequation = Yequation;
  Xequation = new Math_Expr(Field1);
  Yequation = new Math_Expr(Field2);
  Xequation->setCurrentValue("left",   0.0);
  Xequation->setCurrentValue("right",  (double) drawingArea->da_width);
  Xequation->setCurrentValue("top",    0.0);
  Xequation->setCurrentValue("bottom", (double) drawingArea->da_height);
  Yequation->setCurrentValue("left",   0.0);
  Yequation->setCurrentValue("right",  (double) drawingArea->da_width);
  Yequation->setCurrentValue("top",    0.0);
  Yequation->setCurrentValue("bottom", (double) drawingArea->da_height);

  Xequation->setCurrentValue("z", (double) fly_current);
  Yequation->setCurrentValue("z", (double) fly_current);
  if(Xequation->evaluate() == failed_E){
    delete Xequation;
    Xequation = oldXequation;
    return -1;
  }
  if(Yequation->evaluate() == failed_E){
    delete Yequation;
    Yequation = oldYequation;
    return -1;
  }
  current_x = (int) Xequation->GetResult();
  current_y = (int) Yequation->GetResult();
  fly_start_save     = fly_start;
  fly_end_save       = fly_end;
  fly_incr_save      = fly_incr;
  fly_current_save   = fly_current;  
  delete oldXequation;
  delete oldYequation;
  return 1;
}
void RunFly::reset(){
  resetTerminationCond();
  if(dir == FORWARD_E){
    fly_incr      = fly_incr_save;
    fly_start     = fly_start_save;
    fly_end       = fly_end_save;
    fly_current   = fly_start_save;
  }
  else{
    fly_incr    = - fly_incr_save;
    fly_start     = fly_end_save;
    fly_end       = fly_start_save;
    fly_current   = fly_end_save;
  }
 
  Xequation->setCurrentValue("z", (double) fly_current);
  Yequation->setCurrentValue("z", (double) fly_current);
  if(Xequation->evaluate() == failed_E){
    return;
  }
  if(Yequation->evaluate() == failed_E){
    return;
  }
  current_x = (int) Xequation->GetResult();
  current_y = (int) Yequation->GetResult();
  clipList->reset();
}

void RunFly::start(int,int,int,int){
  clipList->reset();
}

void RunFly::reverse(){
  if(dir == FORWARD_E)
    dir = BACKWARD_E;
  else
    dir = FORWARD_E;
  fly_incr  = - fly_incr;
}

int RunFly::nextMove(){
  if(shouldTerminate() == 1){
    takeCareOfOldLocation(current_x,current_y);
    return 1;
  }
  setNextClip();
  if(fly_start > fly_end){
    if(fly_current < fly_end){
      switch(boundaryMode){
      case Wrap_E:
	fly_current = fly_start;
	break;
      case BounceBack_E: 
	fly_current = fly_end;
	fly_incr    = - fly_incr;
	break;
      case Disappear_E: 
	XET->scheduler->Delete(name);
	// may be a problem here since the scheduler deleted this object
        // at this point.
        // I hope there are still uncorrupted memory for finishing up this 
        // return;
        return 0;
      }
    }
    if(fly_current > fly_start){
      switch(boundaryMode){
      case Wrap_E: 
	fly_current = fly_end;
	break;
      case BounceBack_E:
	fly_current = fly_start;
	fly_incr    = - fly_incr;
	break;
      case Disappear_E: 
	XET->scheduler->Delete(name);
	// may be a problem here since the scheduler deleted this object
        // at this point.
        // I hope there are still uncorrupted memory for finishing up this 
        // return;
	return 0;
      }
    }
  }
  if(fly_start < fly_end){
    if(fly_current > fly_end){
      switch(boundaryMode){
      case Wrap_E:
	fly_current = fly_start;
	break;
      case BounceBack_E: 
	fly_current = fly_end;
	fly_incr    = - fly_incr;
	break;
      case Disappear_E: 
	XET->scheduler->Delete(name);
	// may be a problem here since the scheduler deleted this object
        // at this point.
        // I hope there are still uncorrupted memory for finishing up this 
        // return;
        return 0;
      }
    }
    if(fly_current < fly_start){
      switch(boundaryMode){
      case Wrap_E: 
	fly_current = fly_end;
	break;
      case BounceBack_E:
	fly_current = fly_start;
	fly_incr    = - fly_incr;
	break;
      case Disappear_E: 
	XET->scheduler->Delete(name);
	// may be a problem here since the scheduler deleted this object
        // at this point.
        // I hope there are still uncorrupted memory for finishing up this 
        // return;
        return 0;
      }
    }
  }

  Xequation->setCurrentValue("z", (double) fly_current);
  Yequation->setCurrentValue("z", (double) fly_current);
  if(Xequation->evaluate() == failed_E)
    return 0;
  if(Yequation->evaluate() == failed_E)
    return 0;
  int prev_x = current_x;
  int prev_y = current_y;
  current_x = (int) Xequation->GetResult();
  current_y = (int) Yequation->GetResult();
  d_x = current_x - prev_x;
  d_y = current_y - prev_y;
  if(abs(d_x) >= clip_width || abs(d_y) >= clip_height){
    takeCareOfOldLocation(prev_x,prev_y);
    clip->DisplayImage(drawingArea, current_x, current_y);
  }else{
    takeCareOfTrailingImage();
    clip->DisplayImage(drawingArea, current_x, current_y);
  }
  fly_current += fly_incr;
  return 0;
}


