//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "xet.h"
#include "run_locus.h"
#include "util.h"
#include <libc.h>
#include "config.h"

extern Xet       *XET;

RunLocus::RunLocus(Display *d,OutputWindow *w,ClipList *im)
     :RunBase(d,w,im,Locus_E),locus(NULL),trans_x(0),trans_y(0),
     current(0),incr(1),dir(FORWARD_E)
{
}

RunLocus::~RunLocus(){
  if(intervalXid != 0){
    XtRemoveTimeOut(intervalXid);
    intervalXid = 0;
  }
  if(clipList) delete clipList;
  if(name) delete name;
}

RunBase *RunLocus::Copy(){
  RunLocus *rl =  new RunLocus(*this);
  rl->clipList = new ClipList(*clipList);
  STRCPY(rl->name,name);
  return rl;
}

void RunLocus::setLocus(Locus *l){
  locus = l;
}

void RunLocus::setTranslation(int x, int y){
  trans_x = x;
  trans_y = y;
}

void RunLocus::setBoundaryMode(BoundaryMode b){
  boundaryMode = b;
}

BoundaryMode RunLocus::getBoundaryMode(){
  return boundaryMode;
}

void RunLocus::writeToDisk(FILE *fp){
  if(fp){
    fprintf(fp,"locus:");
    clipList->writeOut(fp);
    fprintf(fp,":%s:%s:%s:%d:%d:%d",
	    trailingModeInString(trailingMode),
	    boundaryModeInString(boundaryMode),
	    locus->getName(),trans_x,trans_y,
	    interval);
    writeOutTermCond(fp);
    fprintf(fp,"\n");
  }
}

void RunLocus::reset(){
  resetTerminationCond();
  if(dir == FORWARD_E)
    current = 0;
  else
    current = locus->getSize() - 1;
  LocusPoint *lp = locus->getPoint(current);
  current_x = lp->x + trans_x;
  current_y = lp->y + trans_y;
  d_x = d_y = 0;
  clipList->reset();
}

void RunLocus::reverse(){
  if(dir == FORWARD_E)
    dir = BACKWARD_E;
  else
    dir = FORWARD_E;
}

void RunLocus::start(int,int,int,int){
  reset();
}

int RunLocus::nextMove(){
  if(shouldTerminate() == 1){
    takeCareOfOldLocation(current_x,current_y);
    return 1;
  }
  if(dir == FORWARD_E)
    current = current + incr;
  else
    current = current - incr;
  setNextClip();
  int prev_x = current_x;
  int prev_y = current_y;
  LocusPoint *lp;
  lp = locus->getPoint(current);
  if(lp){
    current_x = lp->x + trans_x;
    current_y = lp->y + trans_y;
  }else{
    switch(boundaryMode){
    case Wrap_E:
      reset();
      break;
    case BounceBack_E:
      reverse();
      reset();
      break;
    case Disappear_E: 
      XET->scheduler->Delete(name);
      return 0;
    }
  }
  d_x = current_x - prev_x;
  d_y = current_y - prev_y;

  if(abs(d_x) >= clip_width || abs(d_y) >= clip_height){
    takeCareOfOldLocation(prev_x,prev_y);
    clip->DisplayImage(drawingArea, current_x, current_y);
  }else{
    takeCareOfTrailingImage();
    clip->DisplayImage(drawingArea, current_x, current_y);
  }
  return 0;
}



