//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "run_movie.h"
#include "util.h"
#include <libc.h>
#include "config.h"

RunMovie::RunMovie(Display *d,OutputWindow *w)
     :RunBase(d,w,NULL,Movie_E),movie(NULL),current(NULL),dir(FORWARD_E)
{
}

RunMovie::~RunMovie(){
  RunBase *rb;
  RunBase *tmp = movie;
  while(tmp->next != movie){
    rb = tmp;
    tmp = tmp->next;
    delete rb;
  }
  delete tmp;
  if(clipList) delete clipList;
  if(name) delete name;
}
int RunMovie::howmanyObjects(){
  RunBase *tmp = movie;
  int rtn = 0;
  while(tmp){
    rtn++;
    tmp = tmp->next;
    if(tmp == movie)
      break;
  }
  return rtn;
}
void RunMovie::BuildMovieXmList(Widget w){
  RunBase *tmp = movie;
  char buffer[200];
  deleteAllItemsOfXmList(w);
  while(tmp){
    char *tc = tmp->getTermCondValueInString();
    sprintf(buffer,MOVIE_FORMAT,tmp->name,tmp->getRunModeInString(),
	    tmp->interval,tc);
    PutTextInListWidget(w,buffer);
    tmp = tmp->next;
    if(tmp == movie)
      break;
  }
}

// add an object to the last
void RunMovie::addRunObject(RunBase *obj){
  RunBase *rb = obj->Copy();
  RunBase *tmp;
  if(movie){
    tmp = movie;
    while(tmp->next != movie){
      tmp = tmp->next;
    }
    tmp->next = rb;
    rb->prev = tmp;
    rb->next = movie;
    movie->prev = rb;
    //tail = rb;
  }else{
    movie = rb;
    rb->next = movie;
    rb->prev = movie;
    //tail = rb;
  }
  rb->reset();
}

// exahnge position of this one and the next one
// the first item is counted from 0
void RunMovie::exchange(int which){
  if(which < 0 || movie == NULL)
    return;
  int idx;
  RunBase *t1,*t2;
  t1 = movie;
  if(t1)
    t2 = t1->next;
  if(t2 == movie) // only one object in the list
    return;
  idx = 0;
  while(t1->next != movie){
    if(idx == which){ // need to exchange this one and the next one
      t2 = t1->next;
      t2->next->prev = t1;
      t1->next = t2->next;
      t1->prev->next = t2;
      t2->prev = t1->prev;
      t1->prev = t2;
      t2->next = t1;
      if(idx == 0)  // move the first one down so the head is now t2
	movie = t2;
      break;
    }
    t1 = t1->next;
    idx++;
  }
}

void RunMovie::moveUp(int which){
  exchange(which - 1);
}

void RunMovie::moveDown(int which){
  exchange(which);
}

void RunMovie::moveToFirst(int which){
  if(which <= 0 || movie == NULL)
    return;
  int idx;
  RunBase *t1,*t2;
  t1 = movie;
  if(t1)
    t2 = t1->next;
  if(t2 == movie) // only one object in the list
    return;
  idx = 0;
  while(t1->next != movie){
    if(idx == which){ // mv t1 to the first
      // get off t1 from the list
      t1->prev->next = t1->next;
      t1->next->prev = t1->prev;
      // insert t1 to the front
      t1->next = movie;
      t1->prev = movie->prev;
      movie->prev->next = t1;
      movie->prev = t1;
      movie = t1;
      return;
    }
    t1 = t1->next;
    idx++;
  }
  // research to the end
  if(idx == which)
    movie = movie->prev;
}

void RunMovie::moveToLast(int which){
  if(which < 0 || movie == NULL)
    return;
  int idx;
  RunBase *t1,*t2;
  t1 = movie;
  if(t1)
    t2 = t1->next;
  if(t2 == movie) // only one object in the list
    return;
  idx = 0;
  if(which == 0){
    movie = movie->next;
    return;
  }
  while(t1->next != movie){
    if(idx == which){ // mv t1 to the last
      // get off t1 from the list
      t1->prev->next = t1->next;
      t1->next->prev = t1->prev;
      // insert t1 to the end
      t1->next = movie;
      movie->prev->next = t1;
      t1->prev = movie->prev;
      movie->prev = t1;
      break;
    }
    t1 = t1->next;
    idx++;
  }
}

void RunMovie::Delete(int which){
  if(which < 0 || movie == NULL)
    return;
  int idx;
  RunBase *t1,*t2;
  t1 = movie;
  if(t1)
    t2 = t1->next;
  if(t2 == movie) // only one object in the list
    return;
  idx = 0;
  while(t1->next != movie){
    if(idx == which){ 
      // get off t1 from the list
      t1->prev->next = t1->next;
      t1->next->prev = t1->prev;
      if(which == 0)
	movie = t1->next;
      delete t1;
      break;
    }
    t1 = t1->next;
    idx++;
  }
}

RunBase *RunMovie::getAnObject(int which){
  RunBase *tmp = movie;
  int i = 0;
  while(tmp){
    if(i == which)
      return tmp;
    tmp = tmp->next;
    if(tmp == movie)
      break;
    i++;
  }
  return NULL;
}

RunBase *RunMovie::Copy(){
  RunMovie *rtn = new RunMovie(Dpy,drawingArea);
  if(!movie) // empty movie just return
    return rtn;
  RunBase *tmp = movie;
  while(tmp->next != movie){
    rtn->addRunObject(tmp);
    tmp = tmp->next;
  }
  rtn->addRunObject(tmp); // copy the last one
  STRCPY(rtn->name,"NoName");
  rtn->reset();
  return rtn;
}
void RunMovie::setOutputWindow(OutputWindow *ow){
  RunBase *tmp = movie;
  while(tmp){
    tmp->setOutputWindow(ow);
    tmp = tmp->next;
    if(tmp == movie)
      break;
  }
}
void RunMovie::writeToDisk(FILE *fp){
  RunBase *tmp = movie;
  fprintf(fp,"movie:\n");
  while(tmp){
    fprintf(fp,"movie:%s:%s:",tmp->name,
	    (tmp->getOutputWindow()->da_area) ? "" : "root");
    tmp->writeToDisk(fp);
    tmp = tmp->next;
    if(tmp == movie)
      break;
  }
  fprintf(fp,"end-of-movie\n");
}

void RunMovie::reset(){
  if(movie){
    if(dir == FORWARD_E)
      current = movie;
    else
      current = movie->prev;
    current->reset();
    interval = current->interval;
  }
}

void RunMovie::reverse(){
  reverseDeep();
}

void RunMovie::reverseDeep(){
  if(dir == FORWARD_E)
    dir = BACKWARD_E;
  else
    dir = FORWARD_E;
  RunBase *rb;
  RunBase *tmp = movie;
  while(tmp->next != movie){
    rb = tmp;
    //rb->reset();
    rb->reverse();
    tmp = tmp->next;
  }
  tmp->reverse();
}

unsigned long RunMovie::getCurrentInterval(){
  return current->interval;
}

int RunMovie::nextMove(){
  if(movie){
    if(current->nextMove() == 1){
      if(dir == FORWARD_E)
	current = current->next;
      else
	current = current->prev;
      interval = current->interval;
      current->reset();
      current->nextMove();
    }
  }
  return 0;
}

