//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "run_random.h"
#include "util.h"
#include <libc.h>
#include "config.h"

RunRandom::RunRandom(Display *d,OutputWindow *w,ClipList *im)
     :RunBase(d,w,im,Random_E)
{
}

RunRandom::~RunRandom(){
  if(intervalXid != 0){
    XtRemoveTimeOut(intervalXid);
    intervalXid = 0;
  }
  if(clipList) delete clipList;
  if(name) delete name;
}

RunBase *RunRandom::Copy(){
  RunRandom *rr = new RunRandom(*this);
  rr->clipList = new ClipList(*clipList);
  STRCPY(rr->name,name);
  return rr;
}

void RunRandom::writeToDisk(FILE *fp){
  if(fp){
    fprintf(fp,"random:");
    clipList->writeOut(fp);
    fprintf(fp,":%s:%d:%d:%d:%d:%d",
	    trailingModeInString(trailingMode),
	    bounding_x,bounding_y,bounding_width,bounding_height,
	    interval);
    writeOutTermCond(fp);
    fprintf(fp,"\n");
  }
}

void RunRandom::start(int x, int y, int w, int h){
  current_x = x;
  current_y = y;
  d_x = 0;
  d_y = 0;
  bounding_x = (x <= 0) ? 0 : x;
  bounding_y = (y <= 0) ? 0 : y;
  bounding_width = (w <= 0 || (x+w) >= (int) drawingArea->da_width) ? 
                    (int) drawingArea->da_width - x : w;
  bounding_height = (h <= 0 || (y+h) >= (int) drawingArea->da_height) ? 
                    (int) drawingArea->da_height - y : h;
  clipList->reset();
}

void RunRandom::reset(){
  resetTerminationCond();
}

int RunRandom::nextMove(){
  if(shouldTerminate() == 1){
    takeCareOfOldLocation(current_x,current_y);
    return 1;
  }
  setNextClip();
  placeToNewLocation(rand() % bounding_width  + bounding_x, 
		     rand() % bounding_height + bounding_y);
  return 0;
}

