//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include "util.h"
#include <libc.h>
#include <unistd.h>
#include <sys/file.h>
#include "color.h"
#include "config.h"
#include <X11/cursorfont.h>

extern Color      *currentCmap;
extern int         asyncResult;

/* defines for access(2) */
#define READ_OK     04
#define WRITE_OK    02
#define EXECUTE_OK  01
#define SEARCH_OK   01
//#define F_OK        00

static Widget GlobalInfoWidget;

char *calcEffectiveDirectory(char *dir){
  char buffer[256];
  char *home;
  if(dir == NULL)
    return NULL;
  switch(dir[0]){
  case '/':
    sprintf(buffer,"%s",dir);
    break;
  case '~':
    home = getenv("HOME");
    if(home){
      if(dir[1] == '/')
	sprintf(buffer,"%s%s",home,&dir[1]);
      else
	sprintf(buffer,"%s/%s",home,&dir[1]);
    }else
      return NULL;
    break;
  default:
    sprintf(buffer,"%s/%s",CURRENT_DIRECTORY,dir);
  }
  ALLOC_STRCPY(EFFECTIVE_DIRECTORY,buffer);
  return EFFECTIVE_DIRECTORY;
}


int FileReadable(char *filePath){
  return( access( filePath, READ_OK ) == 0 );
}
int DirectoryExist(char *path){
  struct stat buf;
  int rtn = stat(path, &buf);
  if(rtn == 0){
    if(S_ISDIR(buf.st_mode))
      return 1;
    else
      return 0;
  }
  return -1;
}
int DirectoryExistOrCreate(char *path){
  struct stat buf;
  int rtn = stat(path, &buf);
  if(rtn == 0){
    if(S_ISDIR(buf.st_mode))
      return 1;
    else
      return -1;
  }
  if(mkdir(path,0777) == 0)
    return 1;
  else
    return -1;
}
char *skipDirectoryPath(char *path){
  int i,len = strlen(path);
  for(i=len-1;i>=0;i--){
    if(path[i] == '/')
      return &path[i+1];
  }
  return path;
}
int  isBlankString(char *s){
  if(s == NULL)
    return 1;
  int len = strlen(s);
  for(int i = 0;i < len;i++){
    if(s[i] == ' ' || s[i] == '\t' || s[i] == '\n')
      continue;
    else
      return(0);
  }
  return 1;
}
char *SkipToNextDigit(char *ptr)
{
  if(ptr == NULL) 
    return NULL;
  // ptr points to a digit or not, 
  // but we don\'t want this one, we want the next one
  while(isdigit(*ptr++));
  while(!isdigit(*ptr)){
    if(*ptr == '\0' || *ptr == '\n')
      return NULL;
    (void)*ptr++;
  }
  return ptr;
}

void registerGlobalInfoWidget(Widget w){
  GlobalInfoWidget = w;
}
void insertStringtoInfo(char *s){
  insertTXTText(GlobalInfoWidget,s);
}
void insertStringtoInfo(char *sf,char *s, char *s1){
  char buf[255];
  sprintf(buf,sf,s,s1);
  insertTXTText(GlobalInfoWidget,buf);
}

void insertStringtoInfo(char *sf,char *s){
  char buf[255];
  sprintf(buf,sf,s);
  insertTXTText(GlobalInfoWidget,buf);
}

void insertStringtoInfo(char *sf,int s){
  char buf[255];
  sprintf(buf,sf,s);
  insertTXTText(GlobalInfoWidget,buf);
}

void insertTXTText(Widget infoTXT, char *s){
  if(infoTXT){
    XmTextPosition endOfText = XmTextGetLastPosition(infoTXT);
    XmTextInsert(infoTXT,endOfText,s);
    endOfText = XmTextGetLastPosition(infoTXT);
    XmTextSetCursorPosition(infoTXT,endOfText);
    XmTextShowPosition(infoTXT,endOfText);
  }
}

int dontBlockXEvents(XtAppContext app){
  XEvent event;
  char buf[24];
  while(XtAppPending(app)){
    XtAppNextEvent(app,&event);
    switch(event.type){
    case KeyPress:
      if((XLookupString(&event.xkey, buf, 24, NULL, NULL) == 1) &&
	 (buf[0] == '\033')){
	   XtDispatchEvent(&event);
	   return(1);         // user hit escape key
	 }
      break;
    default:
      XtDispatchEvent(&event);
      break;
    }
    XtDispatchEvent(&event);
  }
}

// the following doesn't grab the pointer as I expected 8/22/1992
void GrabPointer(Widget w){
  Window win = XtWindow(w);
  XtGrabPointer(w,True, 
		(unsigned int) (PointerMotionMask | ButtonPressMask | ButtonReleaseMask),
		GrabModeAsync,GrabModeAsync,win,None,CurrentTime);
}

int performAsyncInteraction(Display *Dpy,Widget w){
  XEvent event;
  XBell(Dpy,50);
  while(XtIsManaged(w)){
    XtNextEvent(&event);
    XtDispatchEvent(&event);
  }
  return asyncResult;
}

// **************************************************
void PutTextInListWidget1(Widget w,char *s){
  int   tItemCount;
  Arg   args[10];
  char  buff[120];
  int   n;
  n = 0;
  XtSetArg(args[n], XmNitemCount, &tItemCount); n++;
  XtGetValues(w, args, n);
  sprintf(buff,"%2d: %s",tItemCount+1,s);
  XmString line = XmStringCreateSimple(buff);
  XmListAddItem(w, line, tItemCount+1);   
  XmStringFree(line);
  //XmListSetBottomPos(w,tItemCount+1); // was XmListSetPos
  //XmListSetPos(w,tItemCount+1,0);
  //XmListSetPos(w,tItemCount+1);
  XFlush(XtDisplay(w));
}
void PutTextInListWidget(Widget w,char *s){
  int   tItemCount;
  Arg   args[10];
  char  buff[120];
  int   n;
  n = 0;
  XtSetArg(args[n], XmNitemCount, &tItemCount); n++;
  XtGetValues(w, args, n);
  sprintf(buff,"%2d: %s",tItemCount+1,s);
  XmString line = XmStringCreateSimple(buff);
  XmListAddItem(w, line, tItemCount+1);   
  XmStringFree(line);
  XmListSetBottomPos(w,tItemCount+1); // was XmListSetPos
  //XmListSetPos(w,tItemCount+1,0);
  //XmListSetPos(w,tItemCount+1);
  XFlush(XtDisplay(w));
}
// **************************************************
//Since the XmListDeleteAllItems() causes Motif to crash, 
// So I have to use the following procedure to do a work around
// Ko-Haw Nieh 12/27/1990
// under X11R5 the problem ran away so XmListDeleteAllItems(w) works
// Ko-Haw Nieh 10/01/1992
void deleteAllItemsOfXmList(Widget w){
  int tItemCount;
  Arg   args[10];
  int   n;
  n = 0;
  XtSetArg(args[n], XmNitemCount, &tItemCount); n++;
  XtGetValues(w, args, n);
  if(tItemCount == 0) // nothing to delete
    return;
  XmListDeleteAllItems(w);
//  for(int khn=tItemCount;khn>0;khn--)
//    XmListDeletePos(w,khn);
  //XmListDeleteItemsPos(w,tItemCount,1);
  //XmListDeleteItemsPos(w,1,1);
  //XmListDeleteAllItems(w);
}
void FromXmListToTextField(Widget list, Widget Field)
{ 
  // see which one is selected
  int size,*ary[2];
  Boolean rtn;
  rtn = XmListGetSelectedPos(list,ary,&size);

  // find out the real string from the XmList widget
  char *str;
  Arg             arg[2];
  XmString        *currentList;
  int             numItems;
  if(rtn){
    XtSetArg(arg[0], XmNitems, &currentList);
    XtSetArg(arg[1], XmNitemCount, &numItems);
    XtGetValues(list, arg, 2);
    
    str = get_normal_string_from_XmString(currentList[*ary[0]-1]);
    
    // stick the selected item to the field
    // strip the first number
    char item1[10],item2[80],item3[80];
    sscanf(str,"%s %s %s",item1,item2,item3);
    XmTextFieldSetString(Field,item2);
  }
}
int  whichLineInXmListIsSelected(Widget list)
{ 
  // see which one is selected
  int size,*ary[2];
  Boolean rtn;
  rtn = XmListGetSelectedPos(list,ary,&size);
  if(rtn)
    return *ary[0]-1;
  else
    return -1;
}
char *whichItemInXmListIsSelected(Widget list)
{ 
  // see which one is selected
  int size,*ary[2];
  Boolean rtn;
  rtn = XmListGetSelectedPos(list,ary,&size);
  if(!rtn)
    return NULL;
  // find out the real string from the XmList widget
  char *str;
  Arg             arg[2];
  XmString        *currentList;
  int             numItems;

  XtSetArg(arg[0], XmNitems, &currentList);
  XtSetArg(arg[1], XmNitemCount, &numItems);
  XtGetValues(list, arg, 2);

  str = get_normal_string_from_XmString(currentList[*ary[0]-1]);
  return str;
}
char *get_normal_string_from_XmString(XmString cs)
{
  XmStringContext context;
  XmStringCharSet charset;
  XmStringDirection direction;
  Boolean separator;
  char *primitive_string;
  XmStringInitContext(&context,cs);
  XmStringGetNextSegment(context,&primitive_string,
	   &charset,&direction,&separator);
  XmStringFreeContext(context);
  return(primitive_string);
}
char *DeleteSelectedItemFromXmList(Widget list)
{ 
  // see which one is selected
  int size,*ary[2];
  Boolean rtn;
  rtn = XmListGetSelectedPos(list,ary,&size);
  
  if(rtn){
    // find out the real string from the XmList widget
    char *str;
    Arg             arg[2];
    XmString        *currentList;
    int             numItems;
    
    XtSetArg(arg[0], XmNitems, &currentList);
    XtSetArg(arg[1], XmNitemCount, &numItems);
    XtGetValues(list, arg, 2);
    
    str = get_normal_string_from_XmString(currentList[*ary[0]-1]);
    
    // delete the selected item from XmList
    
    XmListDeleteItemsPos(list,1,*ary[0]);
    
    return str;
  }else
    return NULL;
}
void PutTextInLabelWidget(Widget w,char *s)
{
  XmString xs = XmStringCreateSimple(s);
  Arg  args[2];
  int  n = 0;
  XtSetArg(args[n], XmNlabelType, XmSTRING); n++;
  XtSetArg(args[n], XmNlabelString, xs); n++;
  XtSetValues(w, args, n);
  XmStringFree(xs);

}
void PutTextInLabelWidget(Widget w,char *s1, char *s2)
{
  XmString xs1 = XmStringCreateSimple(s1);
  XmString xs2 = XmStringCreateSimple(s2);
  XmString xs  = XmStringSeparatorCreate();
  XmString xc  = XmStringConcat(xs1,xs);
  XmString xf  = XmStringConcat(xc,xs2);
  Arg  args[2];
  int  n = 0;
  XtSetArg(args[n], XmNlabelType, XmSTRING); n++;
  XtSetArg(args[n], XmNlabelString, xf); n++;
  XtSetValues(w, args, n);
  XmStringFree(xs1);
  XmStringFree(xs2);
  XmStringFree(xs);
  XmStringFree(xc);
  XmStringFree(xf);
}
void PutTextInLabelWidget(Widget w,char *s1, char *s2, char *s3)
{
  XmString xs1 = XmStringCreateSimple(s1);
  XmString xs2 = XmStringCreateSimple(s2);
  XmString xs3 = XmStringCreateSimple(s3);
  XmString xs  = XmStringSeparatorCreate();
  XmString xc  = XmStringConcat(xs1,xs);
  XmString xg  = XmStringConcat(xc,xs2);
  XmString xh  = XmStringConcat(xg,xs);
  XmString xf  = XmStringConcat(xh,xs3);
  Arg  args[2];
  int  n = 0;
  XtSetArg(args[n], XmNlabelType, XmSTRING); n++;
  XtSetArg(args[n], XmNlabelString, xf); n++;
  XtSetValues(w, args, n);
  XmStringFree(xs1);
  XmStringFree(xs2);
  XmStringFree(xs);
  XmStringFree(xc);
  XmStringFree(xg);
  XmStringFree(xh);
  XmStringFree(xf);
}

void deleteItemInXmList(Widget list,char *key){
  char *str;
  Arg             arg[2];
  XmString        *currentList;
  int             numItems;

  XtSetArg(arg[0], XmNitems, &currentList);
  XtSetArg(arg[1], XmNitemCount, &numItems);
  XtGetValues(list, arg, 2);
  char key2[256],itemno[10];
  for(int i =0;i < numItems;i++){
    str = get_normal_string_from_XmString(currentList[i]);
    sscanf(str,"%s %s",itemno,key2);
    if(strcmp(key,key2) == 0)
      break;
  }
  if(i != numItems)
    XmListDeleteItemsPos(list,1,i+1);
  XmListSelectPos(list,i+1,False);

}
void placeToXmListInRightOrder(Widget list,char *s){
  char *str;
  Arg             arg[2];
  XmString        *currentList;
  int             numItems;
  int             i,htitem = 1;
  XtSetArg(arg[0], XmNitems, &currentList);
  XtSetArg(arg[1], XmNitemCount, &numItems);
  XtGetValues(list, arg, 2);
  XmString info,info2;
  char tmpbuf[256];
  char key[256],key2[256],itemno[10];

  if(numItems == 0){
    sprintf(tmpbuf,"%2d: %s",1,s);
    info = XmStringCreateSimple(tmpbuf);
    XmListAddItem(list,info,1);
    XmStringFree(info);
    XmListSelectPos(list,1,False);
    XmListSetPos(list,1);
    return;
  }

  sscanf(s,"%s",key);
  for(i =0;i < numItems;i++){
    str = get_normal_string_from_XmString(currentList[i]);
    sscanf(str,"%s %s",itemno,key2);
    if(strcmp(key,key2) == 0){
      sprintf(tmpbuf,"%2d: %s",i+1,s);
      info = XmStringCreateSimple(tmpbuf);
      XmListReplaceItemsPos(list,&info,1,i+1);
      XmStringFree(info);
      htitem = i + 1;
      XmListSelectPos(list,htitem,False);
      XmListSetPos(list,htitem);
      return;
    }
    if(strcmp(key,key2) < 0){
      sprintf(tmpbuf,"%2d: %s",i+1,s);
      info = XmStringCreateSimple(tmpbuf);
      XmListAddItem(list,info,i+1);
      XmStringFree(info);
      htitem = i + 1;
      break;
    }
  }
  if(i==numItems){  // must insert it to the last
    sprintf(tmpbuf,"%2d: %s",i+1,s);
    info = XmStringCreateSimple(tmpbuf);
    XmListAddItem(list,info,i+1);
    XmStringFree(info);
    htitem = i + 1;
    XmListSelectPos(list,htitem,False);
    XmListSetPos(list,htitem);
    return;
  }
  XtSetArg(arg[0], XmNitems, &currentList);
  XtSetArg(arg[1], XmNitemCount, &numItems);
  XtGetValues(list, arg, 2);
  for(i=htitem;i < numItems;i++){
    str = get_normal_string_from_XmString(currentList[i]);
    sprintf(tmpbuf,"%2d: %s",i+1,&str[4]);
    info2 = XmStringCreateSimple(tmpbuf);
    XmListReplaceItemsPos(list,&info2,1,i+1);
    XmStringFree(info2);
  }
  XmListSelectPos(list,htitem,False);
  XmListSetPos(list,htitem);
}

void replaceXmListContent(Widget list,char *from, char *to){

  // find out the real string from the XmList widget
  char *str;
  Arg             arg[2];
  XmString        *currentList;
  int             numItems;

  XtSetArg(arg[0], XmNitems, &currentList);
  XtSetArg(arg[1], XmNitemCount, &numItems);
  XtGetValues(list, arg, 2);
  char *here;
  XmString info;
  char *buffer,*ptr;
  for(int i =0;i < numItems;i++){
    str = get_normal_string_from_XmString(currentList[i]);
    buffer = alloca(strlen(str) + strlen(to) + 1);
    here = strstr(str,from);
    if(here){
      ptr = &here[strlen(from)];
      here[0] = '\0';
      sprintf(buffer,"%s%s%s",str,to,ptr);
      info = XmStringCreateSimple(buffer);
      XmListReplaceItemsPos(list,&info,1,i+1);
      //XmListSelectPos(list,i+1,False);
      XmStringFree(info);	
    }
  }
  XmListSelectPos(list,1,False);
}
void replaceXmListHighlightContent(Widget list,char *from, char *to){
  int size,*ary[2];
  Boolean rtn;
  rtn = XmListGetSelectedPos(list,ary,&size);
  if(!rtn) 
    return ;
  char *str = whichItemInXmListIsSelected(list);
  char *buffer, *here, *ptr;
  XmString info;
  if(str){
    buffer = alloca(strlen(str) + strlen(to) + 1);
    here = strstr(str,from);
    if(here){
      ptr = &here[strlen(from)];
      here[0] = '\0';
      sprintf(buffer,"%s%s%s",str,to,ptr);
      info = XmStringCreateSimple(buffer);
      XmListReplaceItemsPos(list,&info,1,*ary[0]);
      XmListSelectPos(list,*ary[0],False);
      XmStringFree(info);	
    }
  }
}
void writeXimageInPPM(Display *Dpy,char *fileName,XImage *image){
  FILE *fp = fopen(fileName,"w+");
  if(fp == NULL){
    insertStringtoInfo("can not open %s\n",fileName);
    return;
  }
  int ncolors = 256;
  XColor p[256];
  int i;
  for(i=0;i<ncolors;i++){
    p[i].pixel = i;
    p[i].flags = DoRed | DoGreen | DoBlue;
  }
  XQueryColors(Dpy,currentCmap->getcmap(),p,ncolors);

  fprintf(fp,"P6\n");
  fprintf(fp,"%d %d\n",image->width,image->height);
  fprintf(fp,"255\n");
  int x,y;
  unsigned long pix;
  unsigned char r,g,b;
  for(y = 0;y < image->height ;y++){
    for(x = 0;x < image->width;x++){
      pix = XGetPixel(image,x,y);
      r = p[pix].red >> 8;
      g = p[pix].green >> 8;
      b = p[pix].blue >> 8;
      fprintf(fp,"%c%c%c",r,g,b);
    }
  }
  fclose(fp);
  insertStringtoInfo("wrote <%s>\n",fileName);

}

void SortString(char **da, int n){
  if(da == NULL) return;
  int gap,i,j;
  char *a,*b;
  for(gap=n/2;gap>0;gap/=2)
    for(i=gap;i<n;i++)
      for(j=i-gap;j>=0;j-=gap){
	a =  da[j];
	b =  da[j+gap];
	if(strcmp(a,b)<=0)
	  break;
	da[j] = b;
	da[j+gap] = a;
      }
}


int getFileLen (int fd)
{
  int retval;
  lseek (fd, 0, L_SET);
  retval = (int) lseek (fd, 0, L_XTND);
  lseek (fd, 0, L_SET);
  return (retval);
}

// actually load the help text
char *getSource (char *filename)
{
  int fd, flen;
  char *helpTextBuffer;
  helpTextBuffer = NULL;
  if ((fd = open (filename, O_RDONLY)) < 0) {
    ALLOC_STRCPY(helpTextBuffer,"Can not locate Xet help file.");
    return (helpTextBuffer);
  }
  flen = getFileLen(fd);
  helpTextBuffer = new char[flen + 1];
  if (read (fd, helpTextBuffer, flen) <= 0) {
    ALLOC_STRCPY(helpTextBuffer,"Can not read help file");
    return (helpTextBuffer);
  }
  close (fd);
  return (helpTextBuffer);
}
void setHand2Cursor(Widget w){
  Cursor hand2Cursor = XCreateFontCursor(XtDisplay(w),XC_hand2);
  XDefineCursor(XtDisplay(w),XtWindow(w),hand2Cursor);
}
void setCrossReverseCursor(Widget w){
  Cursor cursor = XCreateFontCursor(XtDisplay(w),XC_cross_reverse);
  XDefineCursor(XtDisplay(w),XtWindow(w),cursor);
}

void normalCursor(Widget w){
  Cursor normalCursor = XCreateFontCursor(XtDisplay(w),XC_left_ptr);
  XDefineCursor(XtDisplay(w),XtWindow(w),normalCursor);
}

