//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "xet.h"
#include <libc.h>
#include <osfcn.h>
#include <string.h>
#include <unistd.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/FileSB.h>
#include <X11/cursorfont.h>
#include <sys/param.h>
#include "importImage.h"
#include "color.h"
#include "ppmImage.h"
#include "wrapper.h"
#include "clipsTable.h"
#include "util.h"
#include "textImage.h"
#include "run_fly.h"
#include "run_glide.h"
#include "run_random.h"
#include "run_locus.h"
#include "run_fixed.h"
#include "run_blink.h"
#include "procSpooler.h"
#include "cutbgImage.h"
#include "colorImage.h"
#include "clipList.h"
#include <X11/Xlibint.h>

Color          *currentCmap;

extern int      asyncResult;


// The intent of this fucntion is to trap all X protocol error
// for some reason, some of the X protocol error can not be trapped.
static int supressXProtocolError(Display *,xError *,XExtCodes *, int *){
  printf("An X protocol error occurred, If you see this message you may be ok.\n");
  printf("If you know how to eliminate this kind of problem, please let me know by sending me e-mail at nieh@netcom.com.\n");
  return 1;
}

Xet::Xet(char *resName, Cardinal argc, char* argv[])
: WCLInterpreter(resName, argc, argv),flipOrientation("none"),
  activeRectON(0),drawingStatus(Ready_E),activeLocus(NULL),overrideFlag(1),
  paste_x(0),paste_y(0),arx1(0),arx2(0),ary1(0),ary2(0),
  logoArea(NULL),drawingArea(NULL),rootArea(NULL)
{
  
  // create the entire top level user interface
  CreateAll();
}

void Xet::processCommandLine(Cardinal argc, char* argv[]){
  mainWindow         = nameToWidget("*main");
  infoTXT            = nameToWidget("*infoTXT");
  xetLogo            = nameToWidget("*xetLogo"); 
  Widget da          = nameToWidget("*drawingArea"); 

  registerGlobalInfoWidget(infoTXT);

  Dpy = XtDisplay(da);

  // install extension routine to supress X Protocol Error
  XExtCodes *xetcodes = XInitExtension(Dpy,"MIT-SUNDRY-NONSTANDARD");
  if(xetcodes){
    XESetError(Dpy,xetcodes->extension,supressXProtocolError);
  }else
    printf("extension is not available\n");

  drawingArea = new OutputWindow(da);
  rootArea    = new OutputWindow(Dpy);

  // set up several GC
  vals.foreground = BlackPixel(Dpy,DefaultScreen(Dpy));
  vals.background = WhitePixel(Dpy,DefaultScreen(Dpy));
  vals.graphics_exposures = False;
  gc = XCreateGC(Dpy,drawingArea->window , 
		 GCForeground | GCBackground | GCGraphicsExposures,
		 &vals);
  locus_gc = XCreateGC(Dpy,drawingArea->window , 
		       GCForeground | GCBackground | GCGraphicsExposures,
		       &vals);
  font_gc = XCreateGC(Dpy,drawingArea->window , 
		      GCForeground | GCBackground | GCGraphicsExposures,
		      &vals);

  currentCmap = new Color(Dpy,appShell);

  GlobalFont    = new GFont(Dpy);
  clipsPool     = new ClipsTable(MAX_SLOTS_IN_CLIP_TABLE);
  locusPool     = new LocusTable(MAX_SLOTS_IN_LOCUS_TABLE);
  procSpooler   = new ProcSpooler();
  scheduler     = new Scheduler(app,MAX_SLOTS_IN_SCHEDULER);
  logoClips     = new ClipsTable(10);
  logoScheduler = new Scheduler(app,10);
  
  // build a color pallet which contains all colors
  colorPallet = new ColorImage(Dpy,drawingArea->da_area);
  colorPallet->SetGC(gc);
  colorPallet->MakeImage(16,16,12,12,2,1);

  MakeItOval("*Help");
  MakeItOval("*File");
  MakeItOval("*Build");
  MakeItOval("*Color");
  MakeItOval("*Play");
  MakeItOval("*Misc");
  
  if(PRIVATE_COLORMAP && 
     (PRIVATE_COLORMAP[0] == 'n' || PRIVATE_COLORMAP[0] == 'N'))
    currentCmap->setCmapStatus(DEFAULT_E);
  else{
    currentCmap->setCmapStatus(PRIVATE_E);
    currentCmap->setWindowColormap(appShell);
  }
  
  if(MESSAGE_WINDOW &&
     (MESSAGE_WINDOW[0] == 'n' || MESSAGE_WINDOW[0] == 'N'))
    XtUnmanageChild(XtParent(XtParent(infoTXT)));
  else
    XtManageChild(XtParent(XtParent(infoTXT)));

  overrideFlag = XET_OVERRIDE_FLAG;

  XtAddCallback(drawingArea->da_area ,XmNexposeCallback,
		(XtCallbackProc) PaintBackground,
		(caddr_t) this);

  XtAddCallback(drawingArea->da_area ,XmNresizeCallback,
		(XtCallbackProc) resizeDrawingArea,
		(caddr_t) this);

  XtAddEventHandler(drawingArea->da_area ,Button1MotionMask | ButtonPressMask,
		    False,
		    (XtEventHandler) updateXYLocation,
		    NULL);


  getCurrentDrawingAreaGeometry();
  createSeveralPipmapCursors();

  int unfreezeAll = 0;
  int runAnimatedLogo = 1;
  char *theseObj = NULL;
  char tmpbuf[200];
  for(int iii = 1; iii < argc;iii++){
    if(argv[iii] == NULL)
      break;
    if(strcmp("-S",argv[iii]) == 0){
      unfreezeAll = 1;
      continue;
    }
    if(strcmp("-s",argv[iii]) == 0){
      iii++;
      theseObj = argv[iii];
      continue;
    }
    if(strcmp("-n",argv[iii]) == 0){
      iii++;
      runAnimatedLogo = 0;
      continue;
    }
    // rest of the arguments must be Xet scripts file
    if(argv[iii][0] == '/')
      loadxetfile(argv[iii]);
    else{
      sprintf(tmpbuf,"%s/%s",CURRENT_DIRECTORY,argv[iii]);
      loadxetfile(tmpbuf);
    }
  }
  if(unfreezeAll == 1)
    scheduler->unfreeze();
  else{
    if(theseObj){
      scheduler->unfreeze(theseObj);
    }
  }
  buildXetLogo();
  XtAddCallback(xetLogo ,XmNexposeCallback,(XtCallbackProc) PaintXetLogo,
		(caddr_t) this);
  if(runAnimatedLogo == 1)
    logoScheduler->unfreeze();
}

Xet::~Xet(){
  //should never be called
}

void Xet::actionDispatcher(Widget , char *s){
  XtCallActionProc(mainWindow,s,NULL,NULL,NULL);
}

void Xet::FSBUpdateImportFilename(){
  Widget importclipNameTF    = nameToWidget("importTBL*importclipNameTF");
  Widget importImgimageTF  = nameToWidget("importTBL*importImgimageTF");
  Widget filer             = nameToWidget("FSB*fsb");
  Widget selectedText      = XmFileSelectionBoxGetChild(filer,XmDIALOG_TEXT);
  char *fileName = XmTextGetString(selectedText);
  char *clipName = skipDirectoryPath(fileName);
  XmTextFieldSetString(importclipNameTF,clipName);
  XmTextFieldSetString(importImgimageTF,fileName);
  XtFree(fileName);
  importfetchinfo();
}

void Xet::importfetchinfo(){
  Widget importImgimageTF  = nameToWidget("importTBL*importImgimageTF");
  Widget importImgTypeTF   = nameToWidget("importTBL*importImgTypeTF");
  Widget importImgSizeTF   = nameToWidget("importTBL*importImgSizeTF");
  char  *fileName          = XmTextFieldGetString(importImgimageTF);

  ImportImage  *importImage = new ImportImage(fileName);
  char *imgType  = importImage->getImageType();
  int   width    = importImage->getWidth();
  int   height   = importImage->getHeight();
  int   fs       = importImage->getFileSize();
  char khn[100];
  XmTextFieldSetString(importImgTypeTF,imgType);
  sprintf(khn,"(%dx%d) bytes=%d",width,height,fs);
  XmTextFieldSetString(importImgSizeTF,khn);
  XtFree(fileName);
  delete importImage;
}

void Xet::setFlipOrientation(char *s){
  flipOrientation = s;
}

// very important function, all import iamge process are initiated here

void Xet::importDoit(){
  Widget importDoitw          = nameToWidget("importTBL*importDoit");
  Widget importclipNameTF       = nameToWidget("importTBL*importclipNameTF");
  Widget importImgimageTF     = nameToWidget("importTBL*importImgimageTF");
  char *importclipName          = XmTextFieldGetString(importclipNameTF);

  if(strcmp(importclipName,"") == 0){
    popupModalInfoBox("Must specify a clip name.");
    XtFree(importclipName);
    XtSetSensitive(importDoitw,True);
    return;
  }
  Clip *clp = clipsPool->findClip(importclipName);
  if(clp != NULL){
    popupModalInfoBox("Clip <%s> already exist, use another name",importclipName);
    XtFree(importclipName);
    XtSetSensitive(importDoitw,True);
    return;
  }

  // ask spooler to find out is there a duplicated clip name
  if(procSpooler->haveYouSeenThisName(importclipName) == 1){
    popupModalInfoBox("Clip <%s> already exist in spooler queue, use another name",importclipName);
    XtFree(importclipName);
    XtSetSensitive(importDoitw,True);
    return;
  }

  char *importImgimage   = XmTextFieldGetString(importImgimageTF);
  if(isBlankString(importImgimage) == 1){ // no image file
    popupModalInfoBox("Must specify an image file name\n");
    XtFree(importImgimage);
    XtFree(importclipName);
    XtSetSensitive(importDoitw,True);
    return;
  }

  // image info

  Widget importImgTypeTF   = nameToWidget("importTBL*importImgTypeTF");
  Widget importImgSizeTF   = nameToWidget("importTBL*importImgSizeTF");

  char *importImgType    = XmTextFieldGetString(importImgTypeTF);
  char *importImgSize    = XmTextFieldGetString(importImgSizeTF);
  
  // filtering specification
  Widget importActiveRegionTF = nameToWidget("importTBL*importActiveRegionTF");
  Widget importcolorTF        = nameToWidget("importTBL*importcolorTF");
  Widget importXscaleTF       = nameToWidget("importTBL*importXscaleTF");
  Widget importYscaleTF       = nameToWidget("importTBL*importYscaleTF");
  Widget importrotateTF       = nameToWidget("importTBL*importrotateTF");
  Widget importshearTF        = nameToWidget("importTBL*importshearTF");
  char *importActiveRegion    = XmTextFieldGetString(importActiveRegionTF);
  char *importcolor           = XmTextFieldGetString(importcolorTF);
  char *importXscale          = XmTextFieldGetString(importXscaleTF);
  char *importYscale          = XmTextFieldGetString(importYscaleTF);
  char *importrotate          = XmTextFieldGetString(importrotateTF);
  char *importshear           = XmTextFieldGetString(importshearTF);

  Widget ScaleSize   = nameToWidget("importTBL*importFilterradiobox.lt");
  Widget ScaleFactor = nameToWidget("importTBL*importFilterradiobox.mt");
  Widget ScaleProsp  = nameToWidget("importTBL*importFilterradiobox.rt");
  Widget ScaleFit    = nameToWidget("importTBL*importFilterradiobox.rm");

  Widget quantNumber = nameToWidget("importTBL*importFilterquant.lt");
  Widget quantppm    = nameToWidget("importTBL*importFilterquant.mt");

  Widget deleteppm   = nameToWidget("importTBL*importFilterdelete.lt");
  Widget nodeleteppm = nameToWidget("importTBL*importFilterdelete.mt");

  Widget smooth_yes  = nameToWidget("importTBL*importFiltersmooth.lt");
  Widget smooth_no   = nameToWidget("importTBL* importFiltersmooth.mt");

  int deletionFlag = 0;
  if(XmToggleButtonGetState(deleteppm))   deletionFlag = 1;
  if(XmToggleButtonGetState(nodeleteppm)) deletionFlag = 0;

  // convert image file to ppm format if necessary
  char cmd_buf[512];
  cmd_buf[0] = '\0';
  if(strcmp(importImgType,"PPM (ASCII)") == 0 || 
     strcmp(importImgType,"PPM (Raw)") == 0 ||
     strcmp(importImgType,"PBM") == 0){  
    sprintf(cmd_buf,"(cat %s ",importImgimage);
  }
  if(strcmp(importImgType,"GIF87a") == 0 || 
     strcmp(importImgType,"GIF89a") == 0){
    sprintf(cmd_buf,"(giftoppm %s ",importImgimage);
  }
  if(strcmp(importImgType,"JFIF") == 0){
    sprintf(cmd_buf,"(djpeg %s ",importImgimage);
  }
  if(cmd_buf[0] == '\0'){
    popupModalInfoBox("Unsupported image type, try another one\n");
    XtSetSensitive(importDoitw,True);
    XtFree(importImgimage); 
    XtFree(importImgType);  
    XtFree(importImgSize);  
    XtFree(importActiveRegion);
    XtFree(importcolor );      
    XtFree(importXscale);      
    XtFree(importYscale);      
    XtFree(importrotate);      
    XtFree(importclipName);
    return;
  }

  // cut an active region from the original image
  // should verify the contents of the field 
  // for the time being assume it is always correct
  if(strcmp(importActiveRegion,"") != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmcut %s ",importActiveRegion);
  }

  if(strcmp(importshear,"") != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmshear %s ",importshear);
  }

  // need flip ?
  if(strcmp(flipOrientation,"none") != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmflip -%s ",flipOrientation);
  }

  // need rotation ?
  if(strcmp(importrotate,"") != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmrotate %s ",importrotate);
  }
  
  // need scale ? what kind ?
  if(XmToggleButtonGetState(ScaleSize)){
    int sxx,syy;
    if(strcmp("",importXscale) != 0)
      sxx = atoi(importXscale);
    else
      sxx = drawingArea->da_width;
    if(strcmp("",importYscale) != 0)
      syy = atoi(importYscale);
    else
      syy = drawingArea->da_height;
    sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmscale -xsize %d -ysize %d ",
	    sxx,syy);
  }
  if(XmToggleButtonGetState(ScaleFactor)){
    int sx,sy;
    if(strcmp("",importXscale) != 0)
      sx = atoi(importXscale);
    else
      sx = 1;
    if(strcmp("",importYscale) != 0)
      sy = atoi(importYscale);
    else
      sy = 1;
    if(sy != 1 || sx != 1)
      sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmscale -xscale %d -yscale %d ",
	      sx,sy);
  }
  if(XmToggleButtonGetState(ScaleProsp)){
    if(strcmp("",importXscale) != 0)
      sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmscale %s ",importXscale);
  }
  if(XmToggleButtonGetState(ScaleFit)){
    int fitx,fity;
    if(strcmp("",importXscale) != 0)
      fitx = atoi(importXscale);
    else
      fitx = drawingArea->da_width;
    if(strcmp("",importYscale) != 0)
      fity = atoi(importYscale);
    else
      fity = drawingArea->da_height;
    sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmscale -xysize %d %d ",fitx,fity);
  }

  if(XmToggleButtonGetState(smooth_yes)){
    sprintf(&cmd_buf[strlen(cmd_buf)]," | pnmsmooth ");
  }

  // need quantize colors ?
  if(XmToggleButtonGetState(quantNumber)){
    if(strcmp(importcolor,"") != 0)
      sprintf(&cmd_buf[strlen(cmd_buf)]," | ppmquant %s ",importcolor);
    else
      sprintf(&cmd_buf[strlen(cmd_buf)]," | ppmquant 255 ");
  }
  if(XmToggleButtonGetState(quantppm)){
    if(strcmp(importcolor,"") != 0){
      if(importcolor[0] == '/')
	sprintf(&cmd_buf[strlen(cmd_buf)]," | ppmquant -map %s ",importcolor);
      else
	sprintf(&cmd_buf[strlen(cmd_buf)]," | ppmquant -map %s/%s ",CURRENT_DIRECTORY,importcolor);
    }else{
      if(XET_CURRENT_COLOR_MAP_FILE)
	sprintf(&cmd_buf[strlen(cmd_buf)]," | ppmquant -map %s ",XET_CURRENT_COLOR_MAP_FILE);
      else
	sprintf(&cmd_buf[strlen(cmd_buf)]," | ppmquant -map %s ",XET_DEFAULT_COLOR_MAP_FILE);
    }
  }
  // build an image file Name
  char imgFileName[200];
  sprintf(imgFileName,"%s/%s.ppm",XET_WORKING_DIRECTORY,importclipName);

  // build the final command line

  sprintf(&cmd_buf[strlen(cmd_buf)],"  > %s)", imgFileName);
  
  procSpooler->registerAProcEntry(cmd_buf, "r", imgFileName, importclipName,
				  deletionFlag
				);
  insertStringtoInfo("Registere an import image job !! \n");
  
  XtFree(importImgimage); 
  XtFree(importImgType);  
  XtFree(importImgSize);  
  XtFree(importActiveRegion);
  XtFree(importcolor );      
  XtFree(importXscale);      
  XtFree(importYscale);      
  XtFree(importrotate);      
  XtFree(importclipName);
}
// read the stdout of a subprocess and place it into the info window
void Xet::readForkResult(XtPointer,int fd, XtInputId *){
  char buffer[BUFSIZ+1];
  int n;
  n = read(fd,buffer,BUFSIZ);
  if(n < 0)
    return;
  buffer[n] = '\0';
  insertStringtoInfo(buffer);
  flush();

}
void Xet::readMaskfile(char *s, char *clipName,ClipsTable *cp){
  unsigned int width, height;
  int          x_hot, y_hot;
  Pixmap       bitmap;
  if(cp == NULL)
    cp = clipsPool;
  Clip *clp;
  if(BitmapOpenFailed != XReadBitmapFile(Dpy,drawingArea->window,s,
					 &width,&height,&bitmap,&x_hot,&y_hot))
    {
      clp = cp->findClip(clipName);
      if(clp != NULL){
	clp->setShapeMask(bitmap);
	clp->setShapeFileName(s);
	return;
      }else
	insertStringtoInfo("It is impossible, clip <%s> disappeared\n",clipName);
    }else{
      insertStringtoInfo("Bitmap file <%s> not found\n",s);
    }
}

// flag == 1 ; delete intermeida file
// flag == 0 ; don\'t delete intermeida file
// flag == 2 ; should never happen
// flag == 3 ; put froge to the background
// flag == 4 ; put froge into a clip;
void Xet::readImagefile(char *filename, char *clipName, int flag,
			ClipsTable *cp){
  if(filename == NULL    || strcmp("",filename) == 0)
    return;
  if(cp == NULL)
    cp = clipsPool;
  PPMImage *ppmimage;
  setEffectiveCursor(importCursor);
  currentCmap->Initialize();
  ppmimage = new PPMImage(Dpy,drawingArea->da_area,filename);
  ppmimage->SetGC(gc);
  if(ppmimage->loadPPM() == -1){
    insertStringtoInfo("Bad ppm file <%s>",filename);
  }else{
    insertStringtoInfo("Fitting %d colors into effective color map\n",
		       currentCmap->howMany());
    currentCmap->fitIntoDefaultColormap();
    scheduler->suspend();          // suspend all running objects
    ppmimage->buildXimage();
    ppmimage->setDiskFileName(filename);
    ppmimage->setClipName(clipName);
    ppmimage->setNcolors(currentCmap->howMany());
    cp->addClip(clipName,ppmimage);
    if(flag == 1 || flag == 4 || flag == 3){
      unlink(filename);
    }
    if(flag == 3){ // put the clip to the background and then delete it 
      Pixmap pix = XCreatePixmap(Dpy,drawingArea->window,
				 drawingArea->da_width,
				 drawingArea->da_height,8);
      int xloc = ((drawingArea->da_width > ppmimage->Width()) ? 
		  (drawingArea->da_width - ppmimage->Width()) / 2 : 0);
      int yloc = ((drawingArea->da_height > ppmimage->Height()) ? 
		  (drawingArea->da_height - ppmimage->Height()) / 2  : 0);
      XSetForeground(Dpy,gc,WhitePixel(Dpy,DefaultScreen(Dpy)));
      XFillRectangle(Dpy,pix,gc,0,0,drawingArea->da_width,drawingArea->da_height);

      ppmimage->DisplayImage(drawingArea,pix,xloc,yloc,-1);
      drawingArea->setImage(pix);
      rePaintBackground();
      XFreePixmap(Dpy,pix);
      cp->deleteClip(clipName);
    }
    scheduler->resume();   // resume all running objects
  }
  setSystemDefinedCursor(XC_left_ptr);
  insertStringtoInfo("Loading <%s> image file done.\n",clipName);
}

void Xet::bgroundDoit(){
  Widget bgroundbgroundTF  = nameToWidget("bgroundTBL*bgroundbgroundTF");
  Widget bgroundclipNameTF   = nameToWidget("bgroundTBL*bgroundclipNameTF");
  Widget bgroundJustificationXLocTF = nameToWidget("bgroundTBL*bgroundJustificationXLocTF");
  Widget bgroundJustificationYLocTF = nameToWidget("bgroundTBL*bgroundJustificationYLocTF");
  char *bgroundJustificationXLoc =  XmTextFieldGetString(bgroundJustificationXLocTF);
  char *bgroundJustificationYLoc =  XmTextFieldGetString(bgroundJustificationYLocTF);
  char *bgroundbground =  XmTextFieldGetString(bgroundbgroundTF);
  char *bgroundclipName  =  XmTextFieldGetString(bgroundclipNameTF);

  // find where the output goes
  int destination;
  Widget drawarea = nameToWidget("bgroundTBL*bgroundDSTADIOBOX.lt");
  Widget rootarea = nameToWidget("bgroundTBL*bgroundDSTADIOBOX.mt");
  if(XmToggleButtonGetState(rootarea))    destination = 1;
  if(XmToggleButtonGetState(drawarea))    destination = 2;

  // find placement mode
  int placementMode;
  Widget replace    = nameToWidget("bgroundTBL*bgroundPMRADIOBOX.lt");
  Widget overwrite  = nameToWidget("bgroundTBL*bgroundPMRADIOBOX.mt");
  if(XmToggleButtonGetState(replace))       placementMode = 1;
  if(XmToggleButtonGetState(overwrite))     placementMode = 2;
  
  // find tiling policy
  int tileState;
  Widget tiled   = nameToWidget("bgroundTBL*bgroundModeRadioBox.lt");
  Widget tiledV  = nameToWidget("bgroundTBL*bgroundModeRadioBox.vt");
  Widget tiledH  = nameToWidget("bgroundTBL*bgroundModeRadioBox.ht");
  Widget untiled = nameToWidget("bgroundTBL*bgroundModeRadioBox.mt");
  if(XmToggleButtonGetState(tiled))       tileState = 1;
  if(XmToggleButtonGetState(untiled))     tileState = 2;
  if(XmToggleButtonGetState(tiledV))      tileState = 3;
  if(XmToggleButtonGetState(tiledH))      tileState = 4;

  // find tiling type
  int tileType;
  Widget left_top       = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.lt");   
  Widget center_top     = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.mt");   
  Widget right_top      = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.rt");   
  Widget left_center    = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.lm");   
  Widget center_center  = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.mm");   
  Widget right_center   = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.rm");   
  Widget left_bottom    = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.lb");   
  Widget center_bottom  = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.mb");   
  Widget right_bottom   = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.rb");   
  Widget fixed_location = nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.fixed");
  Widget random_location= nameToWidget("bgroundTBL*bgroundSpecRADIOBOX.random");
  if(XmToggleButtonGetState(left_top))           tileType = 1;
  if(XmToggleButtonGetState(center_top))         tileType = 2;
  if(XmToggleButtonGetState(right_top))          tileType = 3;
  if(XmToggleButtonGetState(left_center))        tileType = 4;
  if(XmToggleButtonGetState(center_center))      tileType = 5;
  if(XmToggleButtonGetState(right_center))       tileType = 6;
  if(XmToggleButtonGetState(left_bottom))        tileType = 7;
  if(XmToggleButtonGetState(center_bottom))      tileType = 8;
  if(XmToggleButtonGetState(right_bottom))       tileType = 9;
  if(XmToggleButtonGetState(fixed_location))     tileType = 10;
  if(XmToggleButtonGetState(random_location))    tileType = 11;
  int pixval;
  if(strcmp(bgroundbground,"") != 0)
    pixval = atoi(bgroundbground);
  else
    pixval = -1;
  int xloc = atoi(bgroundJustificationXLoc);
  int yloc = atoi(bgroundJustificationYLoc);
  if(destination == 1)
    paintDrawingArea(bgroundclipName,pixval,placementMode,tileState,
		     tileType,xloc,yloc,rootArea);
  else
    paintDrawingArea(bgroundclipName,pixval,placementMode,tileState,
		     tileType,xloc,yloc,drawingArea);
  XtFree(bgroundbground);
  XtFree(bgroundclipName);
  XtFree(bgroundJustificationXLoc);
  XtFree(bgroundJustificationYLoc);
}
void Xet::paintDrawingArea(char *bgroundclipName, int pixval, 
			   int placementMode, 
			   int tileState,int tileType,
			   int adjxloc, int adjyloc,
			   OutputWindow *outputwin)
{
  
  // must create a pixmap to hold the image
  Pixmap pix = XCreatePixmap(Dpy,outputwin->window,
			     outputwin->da_width,
			     outputwin->da_height,8);
  // if it is in replacement mode then clear the pix first
  // set the shapemaskbg color if necessary otherwise set it to -1 (ignore)
  int shapemaskbg = -1;
  if(placementMode == 1){
    // paint the background to a user specified pixval
 
    if(pixval != -1){
      XSetForeground(Dpy,gc,pixval);
      shapemaskbg = pixval;
    }else
      XSetForeground(Dpy,gc,WhitePixel(Dpy,DefaultScreen(Dpy)));
    XFillRectangle(Dpy,pix,gc,0,0,outputwin->da_width,outputwin->da_height);
  }else{ // put the current backgroundImage to the pix
    XPutImage(Dpy,pix,gc,outputwin->backgroundImage,0,0,0,0,
	      outputwin->da_width,outputwin->da_height);
  }
  Clip *clp = clipsPool->findClip(bgroundclipName);
  if(clp == NULL){
    if(strcmp("",bgroundclipName) == 0){
      scheduler->suspend();
      outputwin->setImage(pix);
      outputwin->putImage(gc);
      scheduler->resume();
      insertStringtoInfo("Paint background to pixval = %d \n",pixval);
    }else
      insertStringtoInfo("Clip <%s> doesn't exist\n",bgroundclipName);
    XFreePixmap(Dpy,pix);
    return;
  }
  // now place clips onto the background
  int clip_width  = clp->Width();
  int clip_height = clp->Height();
  int xloc, yloc;
  xloc = yloc = 0;

  switch(tileType){
  case 1:  // left top
    xloc = yloc = 0;
    break; 
  case 2: // center top
    xloc = ((outputwin->da_width > clip_width) ? (outputwin->da_width - clip_width) / 2 : 0);
    yloc = 0;
    break;
  case 3:  // right top
    xloc = ((outputwin->da_width > clip_width) ? (outputwin->da_width - clip_width)  : 0);
    yloc = 0;
    break;
  case 4:  // left center
    xloc = 0;
    yloc = ((outputwin->da_height > clip_height) ? (outputwin->da_height - clip_height) / 2  : 0);
    break;
  case 5:  // center center
    xloc = ((outputwin->da_width > clip_width) ? (outputwin->da_width - clip_width) / 2 : 0);
    yloc = ((outputwin->da_height > clip_height) ? (outputwin->da_height - clip_height) / 2  : 0);
    break;
  case 6:   // right center
    xloc = ((outputwin->da_width > clip_width) ? (outputwin->da_width - clip_width)  : 0);
    yloc = ((outputwin->da_height > clip_height) ? (outputwin->da_height - clip_height) / 2  : 0);
    break;
  case 7:   // left bottom
    xloc = 0;
    yloc = ((outputwin->da_height > clip_height) ? (outputwin->da_height - clip_height)  : 0);
    break;
  case 8:   // center bottom
    xloc = ((outputwin->da_width > clip_width) ? (outputwin->da_width - clip_width) / 2 : 0);
    yloc = ((outputwin->da_height > clip_height) ? (outputwin->da_height - clip_height)  : 0);
    break;
  case 9:  // right bottom
    xloc = ((outputwin->da_width > clip_width) ? (outputwin->da_width - clip_width)  : 0);
    yloc = ((outputwin->da_height > clip_height) ? (outputwin->da_height - clip_height)  : 0);
    break;
  case 10:  // fixed location
      xloc = adjxloc;
      yloc = adjyloc;
    break;
  case 11: // random location
    xloc = rand() % ((outputwin->da_width > clip_width) ? (outputwin->da_width - clip_width) : 0);
    yloc = rand() % ((outputwin->da_height > clip_height) ? (outputwin->da_height - clip_height) : 0);
    break;
  }
  int x_start,y_start,n,x,y;
  int x_max, y_max;
  x_max = (int) outputwin->da_width;
  y_max = (int) outputwin->da_height;
  x_start = y_start = 0;
  scheduler->suspend();
  switch(tileState){
  case 1 :  // tiled
    if(xloc <= 0)
      x_start = xloc;
    else{
      if(outputwin->da_width - xloc > 0){
	n = xloc / clip_width;
	x_start = xloc - (n+1) * clip_width;
      }else
	x_start = 0;
    }
    if(yloc <= 0)
      y_start = yloc;
    else{
      if(outputwin->da_height - yloc > 0){
	n = yloc / clip_height;
	y_start = yloc - (n+1) * clip_height;
      }else
	y_start = 0;
    }
    for(x = x_start ; x < x_max ; x = x + (int)clip_width){
      for(y = y_start ; y < y_max ; y = y + (int)clip_height){
	clp->DisplayImage(outputwin,pix,x,y,shapemaskbg);
      }
    }
    break;
  case 2:   // untiled
    clp->DisplayImage(outputwin,pix,xloc,yloc,shapemaskbg);
    break;
  case 3:  // vertical tiled
    if(yloc <= 0)
      y_start = yloc;
    else{
      if(outputwin->da_height - yloc > 0){
	n = yloc / clip_height;
	y_start = yloc - (n+1) * clip_height;
      }else
	y_start = 0;
    }
    for(y = y_start ; y < y_max ; y = y + (int)clip_height){
      clp->DisplayImage(outputwin,pix,xloc,y,shapemaskbg);
    }
    break;
  case 4:  //horizontal tiled
    if(xloc <= 0)
      x_start = xloc;
    else{
      if(outputwin->da_width - xloc > 0){
	n = xloc / clip_width;
	x_start = xloc - (n+1) * clip_width;
      }else
	x_start = 0;
    }
    for(x = x_start ; x < x_max ; x = x + (int)clip_width){
      clp->DisplayImage(outputwin,pix,x,yloc,shapemaskbg);
    }
    break;
  }
  outputwin->setImage(pix);
  outputwin->putImage(gc);
  
  scheduler->resume();
  XFreePixmap(Dpy,pix);
}

void Xet::configClipsInfo(char *s){
  Widget configList = nameToWidget("configClipsTBL*configList");
  if(configList){
    if(s)
      placeToXmListInRightOrder(configList,s); 
    else
      buildClipList(configList);
  }
  updateXetStatistic();
}

void Xet::buildClipList(){
  Widget selectList = nameToWidget("selectClipsTBL*selectList");
  buildClipList(selectList);
}

void Xet::buildClipList(Widget List){
  if(List)
    clipsPool->buildClipsXmList(List);
  updateXetStatistic();
}
void Xet::popupselectClipsTBL(){
  Widget selectClipsTBL = nameToWidget("selectClipsTBL");
  if(selectClipsTBL == NULL)
    WcCreateRoot(Dpy,"selectClipsTBL");
  buildClipList();
}
void Xet::selectthisclip(){
  Widget selectList          = nameToWidget("selectClipsTBL*selectList");
  Widget bgroundclipNameTF   = nameToWidget("bgroundTBL*bgroundclipNameTF");
  Widget clipclipNameTF      = nameToWidget("clipSetupTBL*clipclipNameTF");
  Widget textclipNameTF      = nameToWidget("textTBL*textbackgroundimageTF");
  Widget shapebackgroundimageTF = nameToWidget("shapeTBL*shapebackgroundimageTF");

  if(textclipNameTF)
    FromXmListToTextField(selectList,textclipNameTF);

  if(bgroundclipNameTF)
    FromXmListToTextField(selectList,bgroundclipNameTF);
  
  if(shapebackgroundimageTF)
    FromXmListToTextField(selectList,shapebackgroundimageTF);
  
  if(clipclipNameTF){
    char  *clipclipName = XmTextFieldGetString(clipclipNameTF);
    char buffer[256];
    char *buf = whichItemInXmListIsSelected(selectList);
    if(buf){
      char itemno[10],clipName[200];
      sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
      if(strcmp(clipclipName,"") != 0)
	sprintf(buffer,"%s,%s",clipclipName,clipName);
      else
	sprintf(buffer,"%s",clipName);
      XmTextFieldSetString(clipclipNameTF,buffer);
      XtFree(clipclipName);
    }
  }
}

void Xet::textDoit() {
  Widget textclipNameTF          = nameToWidget("textTBL*textclipNameTF");         
  Widget textbgroundTF         = nameToWidget("textTBL*textbgroundTF");        
  Widget textfgcolorTF         = nameToWidget("textTBL*textfgcolorTF");        
  Widget textbackgroundimageTF = nameToWidget("textTBL*textbackgroundimageTF");
  Widget textfontTF            = nameToWidget("textTBL*textfontTF");          
  Widget textFontSizeTF        = nameToWidget("textTBL*textFontSizeTF");       
  Widget textStringTF          = nameToWidget("textTBL*textStringTF");         
  Widget textClipSizeTF        = nameToWidget("textTBL*textClipSizeTF");

  char *textclipName          =  XmTextFieldGetString(textclipNameTF);
  if(isBlankString(textclipName) == 1){ // no image file
    popupModalInfoBox("Must specify a clip name");
    XtFree(textclipName);
    return;
  }         
  Clip *clp = clipsPool->findClip(textclipName);
  if(clp != NULL){
    popupModalInfoBox("Clip <%s> already exist, use another name",textclipName);
    XtFree(textclipName);
    return;
  }

  char *textbground         =  XmTextFieldGetString(textbgroundTF);         
  char *textfgcolor         =  XmTextFieldGetString(textfgcolorTF);         
  char *textbackgroundimage =  XmTextFieldGetString(textbackgroundimageTF); 
  char *textfont            =  XmTextFieldGetString(textfontTF);            
  char *textFontSize        =  XmTextFieldGetString(textFontSizeTF);       
  char *textString          =  XmTextFieldGetString(textStringTF);          
  char *textClipSize        =  XmTextFieldGetString(textClipSizeTF);

  int textAdj = 5;

  Widget textJustificationXLocTF = nameToWidget("textTBL*textJustificationXLocTF");
  Widget textJustificationYLocTF = nameToWidget("textTBL*textJustificationYLocTF");
  
  char *textJustificationXLoc =  XmTextFieldGetString(textJustificationXLocTF);
  char *textJustificationYLoc =  XmTextFieldGetString(textJustificationYLocTF);
  int xloc = atoi(textJustificationXLoc);
  int yloc = atoi(textJustificationYLoc);
  Widget left_top       = nameToWidget("textTBL*textSpecRADIOBOX.lt");     
  Widget center_top     = nameToWidget("textTBL*textSpecRADIOBOX.mt");     
  Widget right_top      = nameToWidget("textTBL*textSpecRADIOBOX.rt");     
  Widget left_center    = nameToWidget("textTBL*textSpecRADIOBOX.lm");     
  Widget center_center  = nameToWidget("textTBL*textSpecRADIOBOX.mm");     
  Widget right_center   = nameToWidget("textTBL*textSpecRADIOBOX.rm");     
  Widget left_bottom    = nameToWidget("textTBL*textSpecRADIOBOX.lb");     
  Widget center_bottom  = nameToWidget("textTBL*textSpecRADIOBOX.mb");     
  Widget right_bottom   = nameToWidget("textTBL*textSpecRADIOBOX.rb");     
  Widget fixed_location = nameToWidget("textTBL*textSpecRADIOBOX.fixed");  
  Widget random_location= nameToWidget("textTBL*textSpecRADIOBOX.random"); 
  if(XmToggleButtonGetState(left_top))           textAdj = 1;
  if(XmToggleButtonGetState(center_top))         textAdj = 2;
  if(XmToggleButtonGetState(right_top))          textAdj = 3;
  if(XmToggleButtonGetState(left_center))        textAdj = 4;
  if(XmToggleButtonGetState(center_center))      textAdj = 5;
  if(XmToggleButtonGetState(right_center))       textAdj = 6;
  if(XmToggleButtonGetState(left_bottom))        textAdj = 7;
  if(XmToggleButtonGetState(center_bottom))      textAdj = 8;
  if(XmToggleButtonGetState(right_bottom))       textAdj = 9;
  if(XmToggleButtonGetState(fixed_location))     textAdj = 10;
  if(XmToggleButtonGetState(random_location))    textAdj = 11;

  int fsz = atoi(textFontSize);

  TextImage *textImage;
  int font_width,font_height;
  int flag;
  if(strcmp("",textfont) == 0)
    flag = GlobalFont->SetFont("-bitstream-courier-bold-*-*-*-0-0-*-*-*-0-*-*");
  else
    flag = GlobalFont->SetFont(textfont);
  if(flag == 0){
    popupModalInfoBox("<%s> is not a valid font\n",textfont);
    goto D99;
  }
  if(strcmp("",textClipSize) == 0){
    XFontStruct *xfs = GlobalFont->LoadQueryScalableFont(fsz);
    if(xfs){
      font_width = GlobalFont->Width(textString);
      font_height = GlobalFont->Height(textString);
    }else{
      popupModalInfoBox("<%s> is not a valid font\n",textfont);
      goto D99;
    }
  }else{
    sscanf(textClipSize,"%d x %d",&font_width,&font_height);
  }

  unsigned long fg,bg;
  if(strcmp("",textbground) != 0)
    bg = atoi(textbground);
  else
    bg = WhitePixel(Dpy,DefaultScreen(Dpy));
  if(strcmp("",textfgcolor) != 0)
    fg = atoi(textfgcolor);
  else
    fg = BlackPixel(Dpy,DefaultScreen(Dpy));
  XImage *img;
  if(strcmp("",textbackgroundimage) == 0)
    img = NULL;
  else{
    Clip *clp = clipsPool->findClip(textbackgroundimage);
    if(clp == NULL){
      popupModalInfoBox("Clip <%s> doesn't exist. \n",textbackgroundimage);
      img = NULL;
    }else
      img = clp->GetImage();
  }
  if(GlobalFont->IsScalableFont(textfont) == 0 || fsz != 0){
    textImage = new TextImage(Dpy,drawingArea->da_area,GlobalFont);
    textImage->SetGC(font_gc);
    textImage->setClipName(textclipName);
    textImage->DrawText(textString,drawingArea->da_area,font_width,
			font_height,xloc,yloc,
			textAdj,fsz,fg,bg,img);
    clipsPool->addClip(textclipName,textImage);
    insertStringtoInfo("Build text clip <%s> done\n",textclipName);
  }else
    popupModalInfoBox("Must specify a font size greater then 0 for scaleable font\n");

  D99:
    XtFree(textclipName);
    XtFree(textbground);
    XtFree(textfgcolor);
    XtFree(textbackgroundimage);
    XtFree(textfont);
    XtFree(textFontSize);
    XtFree(textString);

  updateXetStatistic();
}

void Xet::cliprehearse(int flag){
  Widget clipclipNameTF      = nameToWidget("clipSetupTBL*clipclipNameTF");
  Widget clipRunObjectTF   = nameToWidget("clipSetupTBL*clipRunObjectTF");
  Widget clipDspIntervalTF = nameToWidget("clipSetupTBL*clipDspIntervalTF");
  Widget clipField1TF      = nameToWidget("clipSetupTBL*clipField1TF");
  Widget clipField2TF      = nameToWidget("clipSetupTBL*clipField2TF");
  Widget clipField3TF      = nameToWidget("clipSetupTBL*clipField3TF");
  Widget clipField4TF      = nameToWidget("clipSetupTBL*clipField4TF");
  Widget clipField5TF      = nameToWidget("clipSetupTBL*clipField5TF");
  Widget  preserve      = nameToWidget("clipSetupTBL*clipTrailRADIOBOX.lt");
  Widget  wipeout       = nameToWidget("clipSetupTBL*clipTrailRADIOBOX.mt");
  Widget  leaving       = nameToWidget("clipSetupTBL*clipTrailRADIOBOX.rt");
  Widget  rainbow       = nameToWidget("clipSetupTBL*clipTrailRADIOBOX.lm");
  Widget  fixed_run     = nameToWidget("clipSetupTBL*clipSpecRADIOBOX.lt");
  Widget  glide_run     = nameToWidget("clipSetupTBL*clipSpecRADIOBOX.mt");
  Widget  random_run    = nameToWidget("clipSetupTBL*clipSpecRADIOBOX.lm");
  Widget  fly_run       = nameToWidget("clipSetupTBL*clipSpecRADIOBOX.rt");
  Widget  blink_run     = nameToWidget("clipSetupTBL*clipSpecRADIOBOX.lr");
  Widget  locus_run     = nameToWidget("clipSetupTBL*clipSpecRADIOBOX.mr");
  Widget  wrap_bnd      = nameToWidget("clipSetupTBL*clipBoundaryRADIOBOX.lt");
  Widget  bounce_bnd    = nameToWidget("clipSetupTBL*clipBoundaryRADIOBOX.mt");
  Widget  disappear_bnd = nameToWidget("clipSetupTBL*clipBoundaryRADIOBOX.rt");

  int destination = 2;
  Widget drawarea = nameToWidget("clipSetupTBL*clipDstRADIOBOX.lt");
  Widget rootarea = nameToWidget("clipSetupTBL*clipDstRADIOBOX.mt");
  if(XmToggleButtonGetState(rootarea))    destination = 1;
  if(XmToggleButtonGetState(drawarea))    destination = 2;
  
  TrailingMode trailingMode;
  if(XmToggleButtonGetState(preserve))  trailingMode = Preserve_E;
  if(XmToggleButtonGetState(wipeout))   trailingMode =  Wipeout_E;
  if(XmToggleButtonGetState(leaving))   trailingMode =  Leave_E;
  if(XmToggleButtonGetState(rainbow))   trailingMode =  Rainbow_E;

  RunMode runMode;
  if(XmToggleButtonGetState(fixed_run))  runMode = Fixed_E;
  if(XmToggleButtonGetState(glide_run))  runMode = Glide_E;
  if(XmToggleButtonGetState(random_run)) runMode = Random_E;
  if(XmToggleButtonGetState(fly_run))    runMode = Fly_E;
  if(XmToggleButtonGetState(blink_run))  runMode = Blink_E;
  if(XmToggleButtonGetState(locus_run))  runMode = Locus_E;

  BoundaryMode boundaryMode;
  if(XmToggleButtonGetState(wrap_bnd))        boundaryMode = Wrap_E;
  if(XmToggleButtonGetState(bounce_bnd))      boundaryMode = BounceBack_E;
  if(XmToggleButtonGetState(disappear_bnd))   boundaryMode = Disappear_E;

  char *clipclipName       =  XmTextFieldGetString(clipclipNameTF);
  char *clipRunObject    =  XmTextFieldGetString(clipRunObjectTF);
  char *clipDspInterval  =  XmTextFieldGetString(clipDspIntervalTF);
  char *clipField1       =  XmTextFieldGetString(clipField1TF);
  char *clipField2       =  XmTextFieldGetString(clipField2TF);
  char *clipField3       =  XmTextFieldGetString(clipField3TF);
  char *clipField4       =  XmTextFieldGetString(clipField4TF);
  
  ClipList *clp = new ClipList(clipclipName,clipsPool);
  RunBase *cat = scheduler->findRunObject(clipRunObject);
  if(clp->Size() == 0){
    popupModalInfoBox("Cliplist <%s> is empty or bad clip names. \n",clipclipName);
    delete clp;
    goto Label_99;
  }
  if(strcmp("",clipRunObject) == 0){
    popupModalInfoBox("Must specify a name for run object");
    delete clp;
    goto Label_99;
  }
  if(cat){
    popupModalInfoBox("Run object <%s> already exist",clipRunObject);
    delete clp;
    goto Label_99;
  }
  // set up interval time in msec
  int interval;
  if(strcmp(clipDspInterval,"") == 0)
    interval  = 30;
  else
    interval  = atoi(clipDspInterval);

  int bx, by, bwidth, bheight;
  int start_x, start_y, d_x, d_y;
  start_x  = atoi(clipField1); 
  start_y  = atoi(clipField2);
  d_x      = atoi(clipField3);
  d_y      = atoi(clipField4);


  // if user select to animate on root then, the following must set to NULL
  OutputWindow *outputWin;
  if(destination == 1)
    outputWin = rootArea;
  else
    outputWin = drawingArea;

  // repeat the following section can generate multiple instances
  // this feeature will be provided in the future
  RunBase      *glide;
  Locus        *locus;
  switch(runMode){
  case Movie_E: // should never get this case here
    break;
  case Fixed_E:
    glide = new RunFixed(Dpy,outputWin,clp);
    glide->start(start_x, start_y, d_x, d_y);
    break;
  case Blink_E:
    glide = new RunBlink(Dpy,outputWin,clp);
    glide->start(start_x, start_y, d_x, d_y);
    break;
  case Random_E: 
    glide = new RunRandom(Dpy,outputWin,clp);
    if(start_x <= 0) start_x = 0;
    if(start_y <= 0) start_y = 0;
    sscanf(clipField3,"%d %d",&d_x, &d_y);
    glide->start(start_x, start_y, d_x, d_y);
    break;
  case Glide_E:
    glide = new RunGlide(Dpy,outputWin,clp);
    sscanf(clipField3,"%d %d",&d_x, &d_y);
    if(strcmp(clipField4,"") == 0){
      bx = by = bwidth = bheight = 0;
    }else{
      sscanf(clipField4,"%d %d %d %d",&bx, &by,&bwidth,&bheight);
    }
    glide->start(start_x, start_y, d_x, d_y);
    glide->setBoundingBox(bx,by,bwidth,bheight);
    break;
  case Fly_E: // it is in fly mode
    glide = new RunFly(Dpy,outputWin,clp);
    glide->start(0,0,0,0);
    glide->setBoundaryMode(boundaryMode);
    if(glide->setFlyEquation(clipField1,clipField2,clipField3) != 1){
      delete glide;
      goto Label_99;
    }
    break;
  case Locus_E:
    locus = locusPool->findLocus(clipField1);
    if(locus != NULL){
      glide = new RunLocus(Dpy,outputWin,clp);
      glide->setLocus(locus);
      locus->incrRefCounter();
      glide->start(0,0,0,0);
      glide->setTranslation(start_y,d_x);
      glide->setBoundaryMode(boundaryMode);
    }else{
      popupModalInfoBox("%s ia not a valid Locus name\n",clipField1);
      goto Label_99;
    }
    break;
  }

  glide->setTrailingMode(trailingMode);
  glide->reset();
  scheduler->addRunObject(clipRunObject,glide,interval, flag);
  glide->setName(clipRunObject);
  insertStringtoInfo("Run object <%s> is added to the scheduler\n",clipRunObject);
  Label_99:
  XtFree(clipclipName);
  XtFree(clipDspInterval);
  XtFree(clipField1);
  XtFree(clipField2);
  XtFree(clipField3);
  XtFree(clipField4);

}
void Xet::clipstoprehearse(){
  Widget clipRunObjectTF = nameToWidget("clipSetupTBL*clipRunObjectTF");
  char *clipRunObject=  XmTextFieldGetString(clipRunObjectTF);
  scheduler->Delete(clipRunObject);
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  if(schedList){
    deleteItemInXmList(schedList,clipRunObject);
  }
  XtFree(clipRunObject);
}

void Xet::updatedrawingWhereField(XEvent *evt){
  int x,y;
  XButtonEvent *but_event;
  Widget drawingWhere;
  char buf[100],buf1[100];
  x = ((XMotionEvent *)evt)->x;
  y = ((XMotionEvent *)evt)->y;
  drawingWhere  = nameToWidget("*drawingWhere");
  sprintf(buf,"(%4d x %4d)",drawingArea->da_width,drawingArea->da_height);
  sprintf(buf1,"(%4d,%4d)",x,y);
  PutTextInLabelWidget(drawingWhere,buf,buf1);

  int          rx,ry,ox,oy;
  unsigned int mask;
  Window       rW,cW;

  switch (evt->type) {
  case ButtonPress:
    but_event = (XButtonEvent *) evt;
    switch (but_event->button) {
    case Button3:
      sprintf(buf,"(%4d x %4d)",drawingArea->da_width,drawingArea->da_height);
      sprintf(buf1,"(%4d,%4d)",paste_x,paste_y);
      PutTextInLabelWidget(drawingWhere,buf,buf1);

      break;
    case Button1: 
      paste_x = x;
      paste_y = y;
      break;
    case Button2:
      paste_x = x;
      paste_y = y;
      if (activeRectON == 1) {     /* turn off old cropping rectangle */
	XSetFunction(Dpy,gc,GXinvert);
	DrawRect(arx1,ary1,arx2,ary2);
	XSetFunction(Dpy,gc,GXcopy);
      }
      activeRectON = 0;
      if(drawingStatus == DefineLocus_E)
	activeLocus->initialize();
      arx1 = ox = x;  ary1 = oy = y;         /* nail down one corner */
      while (1) {
	if (XQueryPointer(Dpy,drawingArea->window,&rW,&cW,&rx,&ry,&x,&y,&mask)) {
	  if (!(mask & Button2Mask)) break;    /* button released */
	  if(drawingStatus == DefineLocus_E){
	    XDrawLine(Dpy,drawingArea->window,locus_gc,x,y,x,y);
	    activeLocus->addPoints(x,y);
	  }

	  if(drawingStatus == Ready_E){ 
	    if (x!=ox || y!=oy) {                /* moved.  erase and redraw */
	      arx2 = x;  ary2 = y;
	      XSetFunction(Dpy,gc,GXinvert);
	      DrawRect(arx1,ary1,ox,oy);
	      activeRectON = DrawRect(arx1,ary1,arx2,ary2);
	      XSetFunction(Dpy,gc,GXcopy);
	      XFlush(Dpy);
	      ox=arx2;  oy=ary2;
	    }
	  }
	}
      }
      if(drawingStatus == DefineLocus_E){
	activeLocus->endOfAddingPoints();
	drawingStatus = Ready_E;
      }
      break;
    }
  }
}
int Xet::DrawRect(int x,int y,int x1,int y1)
{
  int w,h;

  /* returns 0 if it didn't draw anything (rect is too small), 1 if it did */
  w = abs(x-x1);  h = abs(y-y1);
  if (x > x1) x = x1;
  if (y > y1) y = y1;

  /* keep rectangle inside window */  
  if (x < 0) { w += x; x=0; }
  if (y < 0) { h += y; y=0; }
  if (x+w >= drawingArea->da_width)  w=drawingArea->da_width - x - 1;  
  if (y+h >= drawingArea->da_height) h=drawingArea->da_height - y - 1;

  if (w < 4 || h < 4) return 0;   /* too small */

  XSetPlaneMask(Dpy, gc, AllPlanes);
  XDrawRectangle(Dpy, drawingArea->window, gc, x, y, w, h);
  XSetPlaneMask(Dpy, gc, 1L);
  XDrawRectangle(Dpy, drawingArea->window , gc, x+1, y+1, w-2, h-2);
  XSetPlaneMask(Dpy, gc, AllPlanes);
  return 1;
}
void Xet::cropTheArea()
{
  Widget snapshotRegionTF = nameToWidget("snapshotTBL*snapshotRegionTF");
  char *snapshotRegion = XmTextFieldGetString(snapshotRegionTF);
  if(strcmp(snapshotRegion,"") == 0)
    cutTheArea();
  else{
    int x,y,width,height;
    sscanf(snapshotRegion,"%d %d %d %d",&x,&y,&width,&height);
    arx1 = x;
    ary1 = y;
    arx2 = x + width;
    ary2 = y + height;
    activeRectON = 1;
    cutTheArea();
    activeRectON = 0;
  }
  updateXetStatistic();
}
void Xet::cutTheArea()
{
  Widget snapshotclipNameTF = nameToWidget("snapshotTBL*snapshotclipNameTF");
  Widget snapshotfileTF   = nameToWidget("snapshotTBL*snapshotfileTF");
  char *snapshotclipName    = XmTextFieldGetString(snapshotclipNameTF);
  char *snapshotfile      = XmTextFieldGetString(snapshotfileTF);

  if(isBlankString(snapshotclipName) == 1){
    popupModalInfoBox("Must enter a clip name\n");
    return;
  }
  Clip *clp = clipsPool->findClip(snapshotclipName);
  if(clp != NULL){
    popupModalInfoBox("Clip <%s> already exist, use another name\n",snapshotclipName);
    XtFree(snapshotclipName);
    XtFree(snapshotfile);
    return;
  }
  if(activeRectON == 1){
    // turn off the crop boundary
    XSetFunction(Dpy,gc,GXinvert);
    DrawRect(arx1,ary1,arx2,ary2);
    XSetFunction(Dpy,gc,GXcopy);
    if(drawingArea->backgroundImage){
      clp = new CutbgImage(Dpy,drawingArea->da_area,
			   drawingArea->backgroundImage);
      clp->MakeImage(abs(arx1-arx2),abs(ary1-ary2),
		     (arx1<arx2)?arx1:arx2,(ary1<ary2)?ary1:ary2,0,0);
      clp->SetGC(gc);
      clp->setClipName(snapshotclipName);
      clipsPool->addClip(snapshotclipName,clp);
      // now blink it several times
      for(int i =0;i < SNAPSHOT_BLINKING_COUNT; i++){
	XSetForeground(Dpy,gc,rand() % 253 + 2);
	XFillRectangle(Dpy,drawingArea->window,gc,(arx1<arx2)?arx1:arx2,
		       (ary1<ary2)?ary1:ary2,abs(arx1-arx2),abs(ary1-ary2));
	clp->DisplayImage(drawingArea,(arx1<arx2)?arx1:arx2,
			  (ary1<ary2)?ary1:ary2,abs(arx1-arx2),abs(ary1-ary2));
      }
      // redraw the crop boundary
      XSetFunction(Dpy,gc,GXinvert);
      DrawRect(arx1,ary1,arx2,ary2);
      XSetFunction(Dpy,gc,GXcopy);
      insertStringtoInfo("Snap shot an active region done\n");
    }else
      popupModalInfoBox("It is impossible, there is no background image");
  }else{
    popupModalInfoBox("No active window is available\n");
  }
  XtFree(snapshotclipName);
  XtFree(snapshotfile);
}
void Xet::cropEntireBackground(){
  activeRectON = 1;
  arx1 = ary1 = 0;
  arx2 = drawingArea->da_width;
  ary2 = drawingArea->da_height;
  cropTheArea();
  activeRectON = 0;
}

