//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "patchlevel.h"
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include "config.h"
#include "xet.h"
#include "ppmImage.h"
#include <X11/Xmu/Converters.h>
#include <libc.h>
#include <osfcn.h>
#include <string.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
extern "C" {
#include <X11/Xmp/Table.h>
}
#include "util.h"
#include "colorImage.h"
#include "color.h"
#include "procSpooler.h"
#include "run_fixed.h"
#include "wrapper.h"
#include "run_glide.h"
#include "run_random.h"

extern  Color   *currentCmap;
extern  int      asyncResult;

ProcSpooler *Xet::getProcSpooler(){
  return procSpooler;
}
Scheduler *Xet::getScheduler(){
  return scheduler;
}
// flag == 1 paste to bg otherwise paste to fg
void Xet::colorSelPaste(int flag){
  Widget colorPanelTF = nameToWidget("colorSelTBL*colorPanelTF"); 
  char *colorPanel    = XmTextFieldGetString(colorPanelTF);
  char *pixval;
  pixval = &colorPanel[strlen("Pixval = ")];
  pixval[3] = '\0';
  // background set up panel
  Widget bgroundbgroundTF  = nameToWidget("bgroundTBL*bgroundbgroundTF");
  if(bgroundbgroundTF)
    XmTextFieldSetString(bgroundbgroundTF,pixval);
  // build text clips panel
  Widget textbgroundTF         = nameToWidget("textTBL*textbgroundTF");        
  Widget textfgcolorTF         = nameToWidget("textTBL*textfgcolorTF"); 
  if(textbgroundTF && flag == 1)
    XmTextFieldSetString(textbgroundTF,pixval);
  if(textfgcolorTF && flag == 2)
    XmTextFieldSetString(textfgcolorTF,pixval); 
  // build locus panel
  Widget locuspointColorTF = nameToWidget("locusTBL*locuspointColorTF");
  if(locuspointColorTF)
    XmTextFieldSetString(locuspointColorTF,pixval);
  
  XtFree(colorPanel);
}

void Xet::pastePoints(){
  char buf[100];
  Widget clipField1TF =  nameToWidget("clipSetupTBL*clipField1TF");
  Widget clipField2TF =  nameToWidget("clipSetupTBL*clipField2TF");
  if(clipField1TF && clipField2TF){
    sprintf(buf,"%d",paste_x);
    XmTextFieldSetString(clipField1TF,buf);
    sprintf(buf,"%d",paste_y);
    XmTextFieldSetString(clipField2TF,buf);
  }
  Widget textXTF =  nameToWidget("textTBL*textJustificationXLocTF");
  Widget textYTF =  nameToWidget("textTBL*textJustificationYLocTF");
  if(textXTF && textYTF){
    sprintf(buf,"%d",paste_x);
    XmTextFieldSetString(textXTF,buf);
    sprintf(buf,"%d",paste_y);
    XmTextFieldSetString(textYTF,buf);
  }
  Widget bgroundXTF =  nameToWidget("bgroundTBL*bgroundJustificationXLocTF");
  Widget bgroundYTF =  nameToWidget("bgroundTBL*bgroundJustificationYLocTF");
  if(bgroundXTF && bgroundYTF){
    sprintf(buf,"%d",paste_x);
    XmTextFieldSetString(bgroundXTF,buf);
    sprintf(buf,"%d",paste_y);
    XmTextFieldSetString(bgroundYTF,buf);
  }
}

void Xet::destroyBackground(){
  Widget areYouSureTBL = createAreYouSureDialog("Destroy background, Are you sure ?");
  int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
  if(rtn){
    drawingArea->destroyBackground();
    rePaintBackground();
  }
  destroyAreYouSureDialog();
}

void Xet::clearDrawingArea(){
  drawingArea->clearDrawingArea();
  rePaintBackground();
}

void Xet::MakeItOval(char *s){
  Widget w = nameToWidget(s);
  XmuReshapeWidget(w,XmuShapeOval,0,0);
}

void Xet::ReInitialize(){
  Widget areYouSureTBL = createAreYouSureDialog("Re-initialize everything, Are you sure ?");
  int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
  if(rtn){
    scheduler->suspend();
    scheduler->clearEverything();
    drawingArea->destroyBackground();
    clipsPool->clearEverything();
    locusPool->clearEverything();
    procSpooler->DeleteAll();
    currentCmap->freeCmap();
    // in case scheduler panel is open and something still in it
    Widget movieList     =  nameToWidget("schedulerTBL*movieList");
    if(movieList)
      deleteAllItemsOfXmList(movieList);
    updateXetStatistic();
    rePaintBackground();
  }else
    destroyAreYouSureDialog();
}

void Xet::getCurrentDrawingAreaGeometry(){
  drawingArea->updateGeometryAndImage();
  Widget drawingWhere  = nameToWidget("*drawingWhere");
  char buf[100],buf1[100];
  sprintf(buf,"(%4d x %4d)",drawingArea->da_width,drawingArea->da_height);
  sprintf(buf1,"(%4d,%4d)",0,0);
  PutTextInLabelWidget(drawingWhere,buf,buf1);
  //is it necessary to have the following statement ? I guess so
  scheduler->reset();
}
void Xet::rePaintRootArea(){
  rootArea->putImage(gc);
}
void Xet::rePaintBackground(){
  drawingArea->putImage(gc);
  if (activeRectON == 1) {     /* turn off old cropping rectangle */
	XSetFunction(Dpy,gc,GXinvert);
	DrawRect(arx1,ary1,arx2,ary2);
	XSetFunction(Dpy,gc,GXcopy);
      }
}

Widget Xet::createAreYouSureDialog(char *s){
  Widget areYouSureTBL = nameToWidget("*areYouSureTBL");
  if(areYouSureTBL){
    scheduler->suspend();
    XtManageChild(areYouSureTBL);
    char buf[200];
    sprintf(buf,"*areYouSureTBL*areYouSureAreYouSuretitle.labelString: %s",s);
    WcSetValue(areYouSureTBL,buf);
    return areYouSureTBL;
  }else
    return NULL;
} 

void Xet::destroyAreYouSureDialog(){
  scheduler->resume();
}

void Xet::popupModalInfoBox(char *s,char *p){
  char buffer[200];
  sprintf(buffer,s,p);
  popupModalInfoBox(buffer);
}

void Xet::popupModalInfoBox(char *s, int p){
  char buffer[200];
  sprintf(buffer,s,p);
  popupModalInfoBox(buffer);
}

void Xet::popupModalInfoBox(char *s){
  Widget infoModalTBL = nameToWidget("*infoModalTBL");
  XtManageChild(infoModalTBL);
  char buf[200];
  sprintf(buf,"*infoModalTBL*infoModalInfoModaltitle.labelString: %s",s);
  WcSetValue(infoModalTBL,buf);
  performAsyncInteraction(Dpy,infoModalTBL);
}

void Xet::updateColorPanelFields(XEvent *evt){
  int x,y;
  Widget colorPanelTF  = nameToWidget("colorSelTBL*colorPanelTF"); 
  Widget colorPanelTF1 = nameToWidget("colorSelTBL*colorPanelTF1"); 
  Widget colorPanelDA  = nameToWidget("colorSelTBL*colorPanelDA");
  Window       da_root;          // useless variables
  int          da_x,da_y;        // useless variables
  unsigned int da_width,da_height,da_border_width,da_depth;  // useless variables 
  XGetGeometry(XtDisplay(colorPanelDA),XtWindow(colorPanelDA),&da_root,&da_x,&da_y,
	       &da_width,&da_height,&da_border_width,&da_depth);
  int clip_width = colorPallet->Width();
  int clip_height = colorPallet->Height();
  int xloc = ((da_width > clip_width) ? (da_width - clip_width) / 2 : 0);
  int yloc = ((da_height > clip_height) ? (da_height - clip_height) / 2  : 0);
  char buf[100];
  x = ((XMotionEvent *)evt)->x;
  y = ((XMotionEvent *)evt)->y;
  XColor pval = ((ColorImage *)colorPallet)->getPixval(x - xloc, y - yloc);
  sprintf(buf,"Pixval = %3d RGB = (%3d,%3d,%3d)",pval.pixel,
	  pval.red >> 8,pval.green >> 8,pval.blue >> 8);
  XmTextFieldSetString(colorPanelTF,buf);
  Arg  args[2];
  int  n = 0;
  XtSetArg(args[n], XmNbackground, pval.pixel); n++;
  XtSetValues(colorPanelTF1,args, n);
}

void Xet::configENV(){
  Widget  configENVTBL    = nameToWidget("configENVTBL");
  if(configENVTBL == NULL)
    WcCreateRoot(Dpy,"configENVTBL");

  Widget configENVcwdTF            = nameToWidget("configENVTBL*configENVcwdTF");             
  Widget configENVpwdTF            = nameToWidget("configENVTBL*configENVpwdTF");             
  Widget configENVcanvasSizeTF     = nameToWidget("configENVTBL*configENVcanvasSizeTF");      
  Widget configENVrootDirTF        = nameToWidget("configENVTBL*configENVrootDirTF");          
  Widget configENVhasInfoWinTF     = nameToWidget("configENVTBL*configENVhasInfoWinTF");      
  Widget configENVcolormapTF       = nameToWidget("configENVTBL*configENVcolormapTF");        
  Widget configENVstdcmapTF        = nameToWidget("configENVTBL*configENVstdcmapTF");        
  Widget configENVoverrideFlagTF     = nameToWidget("configENVTBL*configENVoverrideFlagTF");    
  Widget configENVlocusPointsTF    = nameToWidget("configENVTBL*configENVlocusPointsTF");

  char buffer[100];
  sprintf(buffer,"%s",XET_ROOT_DIRECTORY);
  XmTextFieldSetString(configENVrootDirTF,buffer);
  sprintf(buffer,"%s",CURRENT_DIRECTORY);
  XmTextFieldSetString(configENVcwdTF,buffer);
  sprintf(buffer,"%s",XET_WORKING_DIRECTORY);
  XmTextFieldSetString(configENVpwdTF,buffer);
  sprintf(buffer,"%s",XET_DEFAULT_COLOR_MAP_FILE);
  XmTextFieldSetString(configENVstdcmapTF,buffer);
  sprintf(buffer,"%d",DEFAULT_MAX_LOCUS_POINTS);
  XmTextFieldSetString(configENVlocusPointsTF,buffer);
  if(overrideFlag == 1)
    XmTextFieldSetString(configENVoverrideFlagTF,"yes"); 
  else
    XmTextFieldSetString(configENVoverrideFlagTF,"no"); 
  sprintf(buffer,"%d x %d",drawingArea->da_width, drawingArea->da_height);
  XmTextFieldSetString(configENVcanvasSizeTF,buffer);
  if(currentCmap->getCmapStatus() == DEFAULT_E)
    XmTextFieldSetString(configENVcolormapTF,"no");
  else
    XmTextFieldSetString(configENVcolormapTF,"yes");
  if(XtIsManaged(XtParent(XtParent(infoTXT))))
    XmTextFieldSetString(configENVhasInfoWinTF,"yes");
  else
    XmTextFieldSetString(configENVhasInfoWinTF,"no");
}
void Xet::configENVApply(){
  Widget configENVcwdTF            = nameToWidget("configENVTBL*configENVcwdTF");             
  Widget configENVpwdTF            = nameToWidget("configENVTBL*configENVpwdTF");             
  Widget configENVcanvasSizeTF     = nameToWidget("configENVTBL*configENVcanvasSizeTF");      
  Widget configENVrootDirTF        = nameToWidget("configENVTBL*configENVrootDirTF");          
  Widget configENVhasInfoWinTF     = nameToWidget("configENVTBL*configENVhasInfoWinTF");      
  Widget configENVcolormapTF       = nameToWidget("configENVTBL*configENVcolormapTF");        
  Widget configENVstdcmapTF        = nameToWidget("configENVTBL*configENVstdcmapTF");        
  Widget configENVoverrideFlagTF   = nameToWidget("configENVTBL*configENVoverrideFlagTF");    
  Widget configENVlocusPointsTF    = nameToWidget("configENVTBL*configENVlocusPointsTF");
  
  char *configENVcwd            = XmTextFieldGetString(configENVcwdTF);
  char *configENVpwd            = XmTextFieldGetString(configENVpwdTF);
  char *configENVcanvasSize     = XmTextFieldGetString(configENVcanvasSizeTF);
  char *configENVrootDir        = XmTextFieldGetString(configENVrootDirTF);
  char *configENVhasInfoWin     = XmTextFieldGetString(configENVhasInfoWinTF);
  char *configENVcolormap       = XmTextFieldGetString(configENVcolormapTF);
  char *configENVstdcmap        = XmTextFieldGetString(configENVstdcmapTF);
  char *configENVoverrideFlag   = XmTextFieldGetString(configENVoverrideFlagTF);
  char *configENVlocusPoints    = XmTextFieldGetString(configENVlocusPointsTF);
  
  ALLOC_STRCPY(XET_ROOT_DIRECTORY,configENVrootDir);
  ALLOC_STRCPY(CURRENT_DIRECTORY,configENVcwd);
  ALLOC_STRCPY(XET_WORKING_DIRECTORY,configENVpwd);
  ALLOC_STRCPY(XET_DEFAULT_COLOR_MAP_FILE,configENVstdcmap);
  
  DEFAULT_MAX_LOCUS_POINTS = atoi(configENVlocusPoints);
  
  if(configENVhasInfoWin[0] == 'n' || configENVhasInfoWin[0] == 'N')
    XtUnmanageChild(XtParent(XtParent(infoTXT)));
  else
    XtManageChild(XtParent(XtParent(infoTXT)));
  
  if(configENVcolormap[0] == 'n' || configENVcolormap[0] == 'N')
    currentCmap->setCmapStatus(DEFAULT_E);
  else
    currentCmap->setCmapStatus(PRIVATE_E);
  if(configENVoverrideFlag[0] == 'y' || configENVoverrideFlag[0] == 'Y')
    overrideFlag = 1;
  else
    overrideFlag = 0;

  Widget colorSelTBL = nameToWidget("colorSelTBL");
  if(colorSelTBL)
    currentCmap->setWindowColormap(colorSelTBL);
  currentCmap->setWindowColormap(appShell);
  unsigned int h,w;
  XmpTableOpts xto;
  Widget drawingAreaTBL = nameToWidget("*drawingAreaTBL");

  if(strcmp("",configENVcanvasSize) != 0){
    sscanf(configENVcanvasSize,"%d x %d",&w,&h);
    if(h != drawingArea->da_height || 
       w != drawingArea->da_width){ // user wants to fix the canvas size

      // make the drawing area as big as possible
      xto = XmpTableOptsParse("");
      XmpTableChildOptions(drawingAreaTBL,xto);

      // find out the max width and height
      getCurrentDrawingAreaGeometry();

      if(w > drawingArea->da_width){
	w = drawingArea->da_width;
	insertStringtoInfo("drawingArea width can be greater then %d\n",drawingArea->da_width);
      }
      if(h > drawingArea->da_height){
	h = drawingArea->da_height;
	insertStringtoInfo("drawingArea height can be greater then %d\n",drawingArea->da_height);
      }
      XtVaSetValues(drawingArea->da_area, XmNwidth, w, XmNheight, h, NULL);
      // now fix the drawingArea, it must be smaller then the max
      //  otherwise X protocol error will occur
      xto = XmpTableOptsParse("WH");
      XmpTableChildOptions(drawingAreaTBL,xto);

    }
  }
  else{
    xto = XmpTableOptsParse("");
    XmpTableChildOptions(drawingAreaTBL,xto);
  }
  XtFree(configENVcwd);
  XtFree(configENVpwd);
  XtFree(configENVcanvasSize);
  XtFree(configENVrootDir);
  XtFree(configENVhasInfoWin);
  XtFree(configENVcolormap);
  XtFree(configENVoverrideFlag);
  XtFree(configENVlocusPoints);

}

void Xet::genericCancel(char *s){
  Widget w = nameToWidget(s);
  if(w)
    XtSetSensitive(w,True);
}

void Xet::updateXetStatistic(){
  char buf1[100],buf2[100],buf3[100];
  Widget xetStatistic  = nameToWidget("*xetStatistic");
  sprintf(buf1,"Spooler    : %d jobs",procSpooler->Howmany());
  sprintf(buf3,"Clip Pool  : %d clips",clipsPool->Howmany());
  sprintf(buf2,"Scheduler : %d/%d objects",
	  scheduler->howmanyRunning(),scheduler->howmanyObjects());
  PutTextInLabelWidget(xetStatistic,buf1,buf3,buf2);
}

void Xet::forgeApply(){

  Widget forgeStarTF        = nameToWidget("forgeTBL*forgeStarTF");       
  Widget forgeSeedTF        = nameToWidget("forgeTBL*forgeSeedTF");        
  Widget forgeSaturationTF  = nameToWidget("forgeTBL*forgeSaturationTF");  
  Widget forgeIceTF         = nameToWidget("forgeTBL*forgeIceTF");         
  Widget forgeGlaciersTF    = nameToWidget("forgeTBL*forgeGlaciersTF");    
  Widget forgePowerTF       = nameToWidget("forgeTBL*forgePowerTF");       
  Widget forgeDimensionTF   = nameToWidget("forgeTBL*forgeDimensionTF");   
  Widget forgeHourTF        = nameToWidget("forgeTBL*forgeHourTF");        
  Widget ForgeTiltAngleTF   = nameToWidget("forgeTBL*ForgeTiltAngleTF");   
  Widget forgeMeshTF        = nameToWidget("forgeTBL*forgeMeshTF");        
  Widget ForgeXsizeTF       = nameToWidget("forgeTBL*ForgeXsizeTF");       
  Widget ForgeYsizeTF       = nameToWidget("forgeTBL*ForgeYsizeTF");       
  Widget ForgeclipNameTF      = nameToWidget("forgeTBL*ForgeclipNameTF");
  Widget loc_background     = nameToWidget("forgeTBL*forgeTypeoutpout.lt");
  Widget loc_clip           = nameToWidget("forgeTBL*forgeTypeoutpout.mt");
  Widget planet             = nameToWidget("forgeTBL*forgeTypepcn.lt"); 
  Widget cloud              = nameToWidget("forgeTBL*forgeTypepcn.mt");   
  Widget night              = nameToWidget("forgeTBL*forgeTypepcn.rt"); 

  char *forgeStar        = XmTextFieldGetString(forgeStarTF);        
  char *forgeSeed        = XmTextFieldGetString(forgeSeedTF);        
  char *forgeSaturation  = XmTextFieldGetString(forgeSaturationTF);  
  char *forgeIce         = XmTextFieldGetString(forgeIceTF);         
  char *forgeGlaciers    = XmTextFieldGetString(forgeGlaciersTF);    
  char *forgePower       = XmTextFieldGetString(forgePowerTF);       
  char *forgeDimension   = XmTextFieldGetString(forgeDimensionTF);   
  char *forgeHour        = XmTextFieldGetString(forgeHourTF);        
  char *ForgeTiltAngle   = XmTextFieldGetString(ForgeTiltAngleTF);   
  char *forgeMesh        = XmTextFieldGetString(forgeMeshTF);        
  char *ForgeXsize       = XmTextFieldGetString(ForgeXsizeTF);       
  char *ForgeYsize       = XmTextFieldGetString(ForgeYsizeTF);       
  char *ForgeclipName      = XmTextFieldGetString(ForgeclipNameTF);

  char cmd_buf[512];
  cmd_buf[0] = '\0';
  
  if(XmToggleButtonGetState(planet)){
    sprintf(&cmd_buf[strlen(cmd_buf)]," (ppmforge  ");
  }
  if(XmToggleButtonGetState(cloud)){
    sprintf(&cmd_buf[strlen(cmd_buf)]," (ppmforge -cloud ");
  }
  if(XmToggleButtonGetState(night)){
    sprintf(&cmd_buf[strlen(cmd_buf)]," (ppmforge -night ");
  }
  
  if(strcmp("",forgeStar) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -stars %s ",forgeStar); 
  }       
  if(strcmp("",forgeSeed) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -seed %s ",forgeSeed);
  }        
  if(strcmp("",forgeSaturation) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -saturation %s",forgeSaturation); 
  } 
  if(strcmp("",forgeIce) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -ice %s",forgeIce);
  }         
  if(strcmp("",forgeGlaciers) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -glaciers %s",forgeGlaciers);
  }    
  if(strcmp("",forgePower) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -power %s",forgePower);
  }       
  if(strcmp("",forgeDimension) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -dimension %s",forgeDimension);
  }   
  if(strcmp("",forgeHour) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -hour %s",forgeHour);
  }        
  if(strcmp("",ForgeTiltAngle) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -inclination %s",ForgeTiltAngle);
  }   
  if(strcmp("",forgeMesh) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -mesh %s",forgeMesh);
  }        
  if(strcmp("",ForgeXsize) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -xsize %s",ForgeXsize);
  }else
    sprintf(&cmd_buf[strlen(cmd_buf)]," -xsize %d",drawingArea->da_width);
  if(strcmp("",ForgeYsize) != 0){
    sprintf(&cmd_buf[strlen(cmd_buf)]," -ysize %s",ForgeYsize);
  }else
    sprintf(&cmd_buf[strlen(cmd_buf)]," -ysize %d",drawingArea->da_height);
  
  char imgFileName[200];

  sprintf(imgFileName, "%s/forge%d",XET_WORKING_DIRECTORY,getpid());

  sprintf(&cmd_buf[strlen(cmd_buf)]," | ppmquant 240 > %s) ",imgFileName);
  
  int outputFlag;
  if(XmToggleButtonGetState(loc_background)){
    outputFlag = 3;
  }
  if(XmToggleButtonGetState(loc_clip)){
    outputFlag = 4;
    if(strcmp("",ForgeclipName) == 0){
      popupModalInfoBox("!!! Must specify a clip name !!!");
      goto D99;
    }
  }
  procSpooler->registerAProcEntry(cmd_buf,
				  "r",
				  imgFileName,
				  ForgeclipName,
				  outputFlag
				);
  insertStringtoInfo("%s\n",cmd_buf);
  insertStringtoInfo("Registere a fractral image job !! \n");

 D99:
  XtFree(forgeStar);        
  XtFree(forgeSeed);        
  XtFree(forgeSaturation);  
  XtFree(forgeIce);         
  XtFree(forgeGlaciers);    
  XtFree(forgePower);       
  XtFree(forgeDimension);   
  XtFree(forgeHour);        
  XtFree(ForgeTiltAngle);   
  XtFree(forgeMesh);        
  XtFree(ForgeXsize);       
  XtFree(ForgeYsize);       
  XtFree(ForgeclipName);
}      
void Xet::drawGrid(int space){
  XSetLineAttributes(Dpy,gc,(unsigned int) 0,LineSolid,CapRound,
		     JoinRound);
  XSetForeground(Dpy,gc,BlackPixel(Dpy,DefaultScreen(Dpy)));
  int x1,y1,x2,y2;
  int i;
  for(i=1;;i++){
    y1 = 0;
    y2 = drawingArea->da_height;
    x1 = x2 = i * space;
    if(x1 > drawingArea->da_width)
      break;
    XDrawLine(Dpy,drawingArea->window,gc,x1,y1,x2,y2);
  }
  for(i=1;;i++){
    x1 = 0;
    x2 = drawingArea->da_width;
    y1 = y2 = i * space;
    if(y1 > drawingArea->da_height)
      break;
    XDrawLine(Dpy,drawingArea->window,gc,x1,y1,x2,y2);
  }
}

void Xet::createSeveralPipmapCursors(){
  unsigned int width, height;
  int          x_hot, y_hot;
  char filename[256];
  XColor fc,bc;
  fc.red = 0;
  fc.green = 0;
  fc.blue = 0;
  bc.red = 252;
  bc.green = 252;
  bc.blue = 252;
  Pixmap bitmap;
  sprintf(filename,"%s/data/icons/save.xbm",XET_ROOT_DIRECTORY);
  if(BitmapOpenFailed == XReadBitmapFile(Dpy,drawingArea->window,
					 filename,
					 &width,&height,&bitmap,&x_hot,&y_hot))
    {
      insertStringtoInfo("Bitmap file <%s> not found\n",filename);
      
    }else{
      saveCursor = XCreatePixmapCursor(Dpy,bitmap,bitmap,&fc,&bc,0,0);
      XFreePixmap(Dpy,bitmap);
    }
  sprintf(filename,"%s/data/icons/import.xbm",XET_ROOT_DIRECTORY);
  if(BitmapOpenFailed == XReadBitmapFile(Dpy,drawingArea->window,
					 filename,
					 &width,&height,&bitmap,&x_hot,&y_hot))
    {
      insertStringtoInfo("Bitmap file <%s> not found\n",filename);
      
    }else{
      importCursor = XCreatePixmapCursor(Dpy,bitmap,bitmap,&fc,&bc,0,0);
      XFreePixmap(Dpy,bitmap);
    }
  sprintf(filename,"%s/data/icons/load.xbm",XET_ROOT_DIRECTORY);
  if(BitmapOpenFailed == XReadBitmapFile(Dpy,drawingArea->window,
					 filename,
					 &width,&height,&bitmap,&x_hot,&y_hot))
    {
      insertStringtoInfo("Bitmap file <%s> not found\n",filename);
      
    }else{
      loadCursor = XCreatePixmapCursor(Dpy,bitmap,bitmap,&fc,&bc,0,0);
      XFreePixmap(Dpy,bitmap);
    }
  sprintf(filename,"%s/data/icons/write.xbm",XET_ROOT_DIRECTORY);
  if(BitmapOpenFailed == XReadBitmapFile(Dpy,drawingArea->window,
					 filename,
					 &width,&height,&bitmap,&x_hot,&y_hot))
    {
      insertStringtoInfo("Bitmap file <%s> not found\n",filename);
      
    }else{
      writeCursor = XCreatePixmapCursor(Dpy,bitmap,bitmap,&fc,&bc,0,0);
      XFreePixmap(Dpy,bitmap);
    }
}
			
static  char *panel[] = {"bgroundTBL","LoadFSB","FSB","LoadcmapFSB",
			 "clipAttribTBL","LoadlocusFSB",
			 "clipSetupTBL",
			 "cmapfilenameTBL",
			 "colorSelTBL",
			 "configClipsTBL",
			 "configENVTBL",
			 "editLocusTBL",
			 "editRunTBL",
			 "filenameTBL",
			 "forgeTBL",
			 "xetHelpTBL",
			 "importTBL",
			 "locusTBL",
			 "renameClipTBL",
			 "renameLocusTBL",
			 "renameSchedTBL",
			 "schedulerTBL",
			 "selectClipsTBL",
			 "shapeTBL",
			 "shapeImportMaskTBL",
			 "snapshotTBL",
			 "spoolerTBL",
			 "textTBL",
			 "writeClipImageTBL",
			 "writeClipMaskTBL",
			 "writeLocusTBL",
			 "writeSchedulerTBL",
			 NULL,
		       };
 
void    Xet::setEffectiveCursor(Cursor cursor){
  Widget w;
  XDefineCursor(Dpy,XtWindow(mainWindow),cursor);
  for(int i=0;;i++){
    if(panel[i] == NULL)
      break;
    w = nameToWidget(panel[i]);
    if(w) 
      XDefineCursor(Dpy,XtWindow(w),cursor);
  }
  XFlush(Dpy);
}
void    Xet::setSystemDefinedCursor(int c){
  Widget w;
  Cursor normalCursor = XCreateFontCursor(Dpy,c);
  XDefineCursor(Dpy,XtWindow(mainWindow),normalCursor);
  for(int i=0;;i++){
    if(panel[i] == NULL)
      break;
    w = nameToWidget(panel[i]);
    if(w) 
      XDefineCursor(Dpy,XtWindow(w),normalCursor);
  }
  XFlush(Dpy); 
}
void    Xet::buildXetLogo(){
  //see we can read the logo image 
  // if yes then load the logo Image
  // if not then generate one and write it out so next time we can read it.
  char buf[256];
  sprintf(buf,"%s/data/icons/xetlogo.ppm",XET_ROOT_DIRECTORY);
  if(FileReadable(buf)){
    insertStringtoInfo("Getting Xet logo from <%s>.\n",buf); 
    currentCmap->Initialize();
    int fg = currentCmap->allocateNamedColor(XET_FIREBRICK3);
    int bg = currentCmap->allocateNamedColor(XET_GOLDENROD);
    int ln = currentCmap->allocateNamedColor(XET_NAVY);
    PPMImage *ppmimage = new PPMImage(Dpy,drawingArea->da_area,buf);
    ppmimage->SetGC(gc);
    if(ppmimage->loadPPM() == -1){
      insertStringtoInfo("Bad Logo ppm file <%s>\n",buf);
      insertStringtoInfo("Building Xet logo.\n");
      buildXetLogo1();
      return;
    }
    currentCmap->fitIntoDefaultColormap();
    ppmimage->buildXimage();
    
    logoArea = new OutputWindow(xetLogo,ppmimage->GetImage());
    logoArea->putImage(gc);
    delete ppmimage;

  }else{
    insertStringtoInfo("Building Xet logo.\n");
    logoArea = new OutputWindow(xetLogo);
    buildXetLogo1();
  }
  BuildXetLogoMovingObjects();
}

void    Xet::buildXetLogo1(){
  GC            loc_gc;
  XGCValues     gvals; 
  unsigned int  logoImageWidth  = logoArea->da_width;
  unsigned int  logoImageHeight = logoArea->da_height;
  Pixmap logoPixmap = XCreatePixmap(Dpy,XtWindow(xetLogo),
				   logoImageWidth,logoImageWidth,8);
  loc_gc = XCreateGC(Dpy,logoPixmap, 0, &gvals);
  int fg = currentCmap->allocateNamedColor(XET_FIREBRICK3);
  int bg = currentCmap->allocateNamedColor(XET_GOLDENROD);
  int ln = currentCmap->allocateNamedColor(XET_NAVY);
  XSetForeground(Dpy,loc_gc,(bg == -1) ? WhitePixel(Dpy,DefaultScreen(Dpy)):bg);
  XFillRectangle(Dpy,logoPixmap,loc_gc,0,0,logoImageWidth,logoImageWidth);
  XSetForeground(Dpy,loc_gc,(fg == -1) ? BlackPixel(Dpy,DefaultScreen(Dpy)):fg);
  XSetBackground(Dpy,loc_gc,(bg == -1) ? WhitePixel(Dpy,DefaultScreen(Dpy)):bg);
  GlobalFont->SetFont("-bitstream-courier-bold-*-*-*-0-0-*-*-*-0-*-*");
  XFontStruct *xfs = GlobalFont->LoadQueryScalableFont(300);
  char *textString = "Xet";
  char textString1[100];
  sprintf(textString1,"    Xet-%d.%d(%d) 12/31/1992 Ko-Haw Nieh ",
	  MAJORVERSION,MINORVERSION,PATCHLEVEL);
//  sprintf(textString1,"Xet-%d.%d(%d) 12/31/1992 nieh@netcom.com ",
//	  MAJORVERSION,MINORVERSION,PATCHLEVEL);
  int font_width,font_height;
  if(xfs){
    font_width = GlobalFont->Width(textString);
    font_height = GlobalFont->Height(textString);
    XSetFont(Dpy,loc_gc,xfs->fid);
    int x_loc  = (logoImageWidth - font_width) / 2;
    int y_loc  = (logoImageHeight - font_height) / 2 + xfs->ascent;
    XDrawString(Dpy,logoPixmap,loc_gc,x_loc,y_loc-6,textString,strlen(textString));

    XSetForeground(Dpy,loc_gc,BlackPixel(Dpy,DefaultScreen(Dpy)));
    XFontStruct *xfs1 = XLoadQueryFont(Dpy,"lucidasanstypewriter-10");
    XSetForeground(Dpy,loc_gc,BlackPixel(Dpy,DefaultScreen(Dpy)));
    XSetFont(Dpy,loc_gc,xfs1->fid);
    XDrawString(Dpy,logoPixmap,loc_gc,70,logoImageHeight-1,textString1,strlen(textString1));   

    int i,j;    
    for(i=logoImageWidth-30;i<=logoImageWidth-15;i=i+10){
      for(j=15;j<=25;j=j+10){
	XFillRectangle(Dpy,logoPixmap,loc_gc,i,j-7,5,5);
      }
    }
    for(i=logoImageWidth-35;i<=logoImageWidth-10;i=i+10){
      for(j=10;j<=30;j=j+10){
	XFillRectangle(Dpy,logoPixmap,loc_gc,i,j-7,5,5);
      }
    }
    
    XSetLineAttributes(Dpy,loc_gc,(unsigned int) 2,LineSolid,CapRound, JoinRound);
    XSetForeground(Dpy,loc_gc,(ln == -1) ? BlackPixel(Dpy,DefaultScreen(Dpy)):ln);
    XDrawLine(Dpy,logoPixmap,loc_gc,5,logoImageHeight-10,logoImageWidth - 5,logoImageHeight-10);
    XSetForeground(Dpy,loc_gc,WhitePixel(Dpy,DefaultScreen(Dpy)));
    
    for(i=10;i<=30;i=i+10){
      for(j=15;j<=25;j=j+10){
	XFillRectangle(Dpy,logoPixmap,loc_gc,i,j-7,5,5);
      }
    }
    for(i=15;i<=25;i=i+10){
      for(j=10;j<=30;j=j+10){
	XFillRectangle(Dpy,logoPixmap,loc_gc,i,j-7,5,5);
      }
    }
    logoArea->setImage(logoPixmap);
    logoArea->putImage(gc);
    XImage *img = XGetImage(Dpy,logoPixmap,0,0,
			       logoImageWidth,logoImageHeight,0xff,ZPixmap);
    char buf[256];
    sprintf(buf,"%s/data/icons/xetlogo.ppm",XET_ROOT_DIRECTORY);
    writeXimageInPPM(Dpy,buf,img);
    
  }else{
    insertStringtoInfo("Xet logo font <%s> not found\n","-bitstream-courier-bold-*-*-*-0-0-*-*-*-0-*-*");
  }
  XFreeGC(Dpy,loc_gc); 
  XFreePixmap(Dpy,logoPixmap);
}
void Xet::rePaintXetLogo(){
  if(logoArea)
    logoArea->putImage(gc);
}

void Xet::BuildXetLogoMovingObjects(){
  insertStringtoInfo("Loading animated Xet logo... \n");
  char buf[256];
  sprintf(buf,"%s/data/icons/xetlogo.xet",XET_ROOT_DIRECTORY);
  if(FileReadable(buf)){
    loadxetfile(buf);
  }else
    insertStringtoInfo("unable to load animated Xet logo <%s>\n",buf);
}

// The following is an example about how to write program to build animation
// objects. This section of code is replaced by xet/data/icons/xetlogo.xet
//void Xet::BuildXetLogoMovingObjects(){
//  insertStringtoInfo("Building animated Xet logo... \n");
//  PPMImage *ppmimage;
//  RunBase *glide;
//  ClipList *ct1;
//  char buf[256];
//  sprintf(buf,"%s/data/icons/black.ppm",XET_ROOT_DIRECTORY);
//  if(FileReadable(buf)){
//    currentCmap->Initialize();
//    ppmimage = new PPMImage(Dpy,logoArea->da_area,buf);
//    if(ppmimage->loadPPM() != -1){
//      ppmimage->SetGC(gc);
//      ppmimage->buildXimage();
//      ppmimage->setDiskFileName(buf);
//      ppmimage->setClipName("black");
//      ppmimage->setNcolors(currentCmap->howMany());
//      logoClips->addClip("black",ppmimage);
//      ct1 = new ClipList("black",logoClips);
//      glide = new RunGlide(Dpy,logoArea,ct1);
//      glide->start(0,0, 1, 1);
//      glide->setBoundingBox(0,0,150,34);
//      glide->setTrailingMode(Preserve_E);
//      //glide->reset();
//      logoScheduler->addRunObject("black",glide,30, 1);
//      glide->setName("black");
//    }
//  }
//  sprintf(buf,"%s/data/icons/white.ppm",XET_ROOT_DIRECTORY);
//  if(FileReadable(buf)){
//    currentCmap->Initialize();
//    ppmimage = new PPMImage(Dpy,logoArea->da_area,buf);
//    if(ppmimage->loadPPM() != -1){
//      ppmimage->SetGC(gc);
//      ppmimage->buildXimage();
//      ppmimage->setClipName("white");
//      logoClips->addClip("white",ppmimage);
//      ct1 = new ClipList("white",logoClips);
//      glide = new RunGlide(Dpy,logoArea,ct1);
//      glide->start(294,0, -1, 1);
//      glide->setBoundingBox(150,0,294,34);
//      glide->setTrailingMode(Preserve_E);
//      //glide->reset();
//      logoScheduler->addRunObject("white",glide,30, 1);
//      glide->setName("white"); 
//      logoScheduler->unfreeze();
//    }
//  }
//}

void Xet::configClipsQuantize(){
  insertStringtoInfo("Quantize all clips is not implemented yet\n");
}

void Xet::configClipsShowAll(){
  char *allClipNames = clipsPool->concateAllNames();
  if(allClipNames[0] != '\0'){
    ClipList *clp = new ClipList(allClipNames,clipsPool);
    RunBase *glide = new RunFixed(Dpy,drawingArea,clp);
    glide->start(0, 0, 0, 0);
    glide->setTrailingMode(Leave_E);
    ((RunFixed *) glide)->setDspFlag(1);
    int d = scheduler->get_dup_index();
    char buff[200];
    sprintf(buff,"showAll_%d",d);
    scheduler->addRunObject(buff,glide,2000, 2);
    insertStringtoInfo("slide show <%s> is registered\n",buff);
  }else
    insertStringtoInfo("There is no clip to display\n");
  delete allClipNames;
}

void Xet::cleaninfotxt(){
  XmTextSetString(infoTXT,"");
}

void Xet::dumpinfotxt(){
  char *txt = XmTextGetString(infoTXT);
  char filename[200];
  sprintf(filename,"%s/xet_info.log",CURRENT_DIRECTORY);
  FILE *fp = fopen(filename,"a+");
  if(fp){
    time_t t = time(0);
    fprintf(fp,"# user : %s \n", getenv("USER"));
    fprintf(fp,"# date : %s",(char *)ctime(&t));
    fprintf(fp,"# \n");
    fprintf(fp,"%s",txt);
    fprintf(fp,"# \n");
    fclose(fp);
  }else
   insertStringtoInfo("Can not open Xet log file <%s>.\n",filename);
  XtFree(txt);
  insertStringtoInfo("Wrote <%s> done.\n",filename);
}
void Xet::colorFreeCells(){
  Widget areYouSureTBL = createAreYouSureDialog("Free all possible cells in current colormap, Are you sure ?");
  int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
  if(rtn){
    currentCmap->freeCmap();
  }
  destroyAreYouSureDialog();
}
void Xet::reCaptureRootWindow(){
  rootArea->reCaptureRootWindow();
}
void Xet::freezeLogo(){
  logoScheduler->freeze();
  logoArea->putImage(gc);
}
void Xet::unfreezeLogo(){
  logoScheduler->unfreeze();
}
