//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include "config.h"
#include "xet.h"
#include <libc.h>
#include <osfcn.h>
#include <string.h>
#include "util.h"
#include <Xm/ToggleB.h>
#include <Xm/FileSB.h>
#include <X11/cursorfont.h>
#include "scheduler.h"
#include "locusTable.h"
#include "clipsTable.h"
#include "colorImage.h"
#include "cloneImage.h"
#include "wrapper.h"
#include "color.h"
#include "run_fly.h"
#include "run_glide.h"
#include "run_random.h"
#include "run_locus.h"
#include "run_fixed.h"
#include "run_blink.h"
#include "procSpooler.h"

extern Color          *currentCmap;

int    asyncResult;

void Xet::buildSchedulerList(char *s){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  if(schedList){
    if(s)
      placeToXmListInRightOrder(schedList,s);
    else
      scheduler->buildSchedulerXmList(schedList);
  }
}

void Xet::freezeMotion(){
  scheduler->freeze();
  buildSchedulerList(NULL);
}
void Xet::unfreezeMotion(){
  scheduler->unfreeze();
  buildSchedulerList(NULL);
}
void Xet::schedulerFreeze(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  int size,*ary[2];
  Boolean rtn;
  rtn = XmListGetSelectedPos(schedList,ary,&size);
  if(!rtn)
    return ;
  int where = *ary[0];
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf == NULL)  return;
  char itemno[10],roName[256],runmode[10];
  sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
  scheduler->freeze(roName);
  char *here = strstr(buf,"Running");
  if(here)
    strcpy(here,"Frozen>\0");
  else
    return;
  XmString info = XmStringCreateSimple(buf);
  XmListReplaceItemsPos(schedList,&info,1,where);
  XmStringFree(info);	
  XmListSelectPos(schedList,where,0); 

}
void Xet::schedulerUnfreeze(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  int size,*ary[2];
  Boolean rtn;
  rtn = XmListGetSelectedPos(schedList,ary,&size);
  if(!rtn)
    return ;
  int where = *ary[0];
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf == NULL)  return;
  char itemno[10],roName[256],runmode[10];
  sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
  scheduler->unfreeze(roName);
  char *here = strstr(buf,"Frozen");
  if(here)
    strcpy(here,"Running>\0");
  else
    return;
  XmString info = XmStringCreateSimple(buf);
  XmListReplaceItemsPos(schedList,&info,1,where);
  XmStringFree(info);	
  XmListSelectPos(schedList,where,0); 

}
void Xet::schedulerResetAll(){
  rePaintBackground();
  scheduler->reset();
}
void Xet::schedulerReverseAll(){
  scheduler->reverse();
}
void Xet::schedulerReverseClipAll(){
  scheduler->reverseClip();
}
void Xet::schedOutputDrawingArea(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],roName[256],runmode[10];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
    scheduler->setOutputWindow(roName,drawingArea);
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}
void Xet::schedOutputRoot(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],roName[256],runmode[10];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
    scheduler->setOutputWindow(roName,rootArea);
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}
void Xet::schedulerToRootWinAll(){
  scheduler->setOutputWindow(rootArea);
}
void Xet::schedulerToDAALL(){
  scheduler->setOutputWindow(drawingArea);
}

void Xet::schedulerReverse(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],roName[256],runmode[10];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
    scheduler->reverse(roName);
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}
void Xet::schedulerReverseClip(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],roName[256],runmode[10];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
    scheduler->reverseClip(roName);
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}
void Xet::schedulerReset(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],roName[256],runmode[20];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
    rePaintBackground();
    scheduler->reset(roName);
  }else
    popupModalInfoBox("!!! Must select an item !!!");
}

void Xet::schedulerDuplicate(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],roName[20],runmode[20];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
    RunBase *ro = scheduler->dup(roName);
  }else
    popupModalInfoBox("!!! Must select an item !!!");
}

void Xet::schedulerDeleteAllItems(){
  Widget schedulerTBL  = nameToWidget("schedulerTBL");
  Widget areYouSureTBL = createAreYouSureDialog("Delete all scheduled items, Are you sure ?");
  int rtn;
  rtn = performAsyncInteraction(Dpy,areYouSureTBL);
  if(rtn){
    scheduler->schedulerDeleteAll();
    Widget schedList =  nameToWidget("schedulerTBL*schedList");
    deleteAllItemsOfXmList(schedList);
    Widget configList = nameToWidget("configClipsTBL*configList");
    if(configList)
      configClipsInfo(NULL);
    destroyAreYouSureDialog();
  }
}

///////////////////////////////////////////////////////////////
void Xet::schedulerDelete(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10];
    char buffer[100],roName[256],runmode[10];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
    sprintf(buffer,"Delete %s, Are you sure ?",roName);
    Widget areYouSureTBL = createAreYouSureDialog(buffer);
    int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
    if(rtn){
      scheduler->Delete(roName);
      DeleteSelectedItemFromXmList(schedList);
      buildSchedulerList(NULL);
      Widget configList = nameToWidget("configClipsTBL*configList");
      if(configList)
	configClipsInfo(NULL);
      updateXetStatistic();
    }
    destroyAreYouSureDialog();
  }else
    popupModalInfoBox("!!! Must select an item !!!");
}
void Xet::schedulerEdit(int flag){
  if(flag == 0){ // it is an animation object editing
    Widget schedList =  nameToWidget("schedulerTBL*schedList");
    char *buf = whichItemInXmListIsSelected(schedList);
    if(buf){
      char itemno[10],roName[256],runmode[20];
      sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, roName,runmode);
      if(strcmp("Movie", runmode) != 0){
	RunBase  *cat = scheduler->findRunObject(roName);
	insertStringtoInfo("Editing  <%s> \n",roName);
	popupSchedulerEditing(roName,cat,-1);
      }else
	popupModalInfoBox("Can not edit Movie Object directly");
    }else
      popupModalInfoBox("!!! Must select an item !!!");
  }
  if(flag == 1){ //it is a movie object editing
    Widget movieList  =  nameToWidget("schedulerTBL*movieList");
    int which = whichLineInXmListIsSelected(movieList);
    if(which >= 0){
      RunBase  *cat = scheduler->getOneMovieObject(which);
      popupSchedulerEditing(cat->name,cat,which);
    }else
      popupModalInfoBox("!!! Must select an item !!!\n");
  }
}
// if flag == -1 --- indicates it is a normal animation object
// if flag >=0   --- indicates the slot in the RunMovie Object.
void Xet::popupSchedulerEditing(char *runObj,RunBase  *cat,int flag){
  char buf[256];
  Widget  editRunTBL    = nameToWidget("editRunTBL");
  if(editRunTBL == NULL){
    editRunTBL = WcCreateRoot(Dpy,"editRunTBL");
  }else{
    popupModalInfoBox("!!! Must close the other echeduler editing !!!");
    return;
  }
  Widget runClipNameTF = nameToWidget("editRunTBL*runClipNameTF");

  if(cat->getclipList()){
    char *loln = cat->getclipList()->getListofNames();
    XmTextFieldSetString(runClipNameTF,loln);
    delete loln;
  }
  Widget runRunObjectTF = nameToWidget("editRunTBL*runRunObjectTF");
  Widget runRunObjectLB = nameToWidget("editRunTBL*runRunObjectLB");
  if(flag == -1)
    XmTextFieldSetString(runRunObjectTF,runObj);
  else{
    PutTextInLabelWidget(runRunObjectLB,"Slot in Movie Object:");
    sprintf(buf,"%d",flag);
    XmTextFieldSetString(runRunObjectTF,buf);
  }

  Widget runField1LB  = nameToWidget("editRunTBL*runField1LB");
  Widget runField2LB  = nameToWidget("editRunTBL*runField2LB");
  Widget runField3LB  = nameToWidget("editRunTBL*runField3LB");
  Widget runField4LB  = nameToWidget("editRunTBL*runField4LB");
  Widget runField5LB  = nameToWidget("editRunTBL*runField5LB");
  Widget runField6LB  = nameToWidget("editRunTBL*runField6LB");

  Widget runField1TF  = nameToWidget("editRunTBL*runField1TF");
  Widget runField2TF  = nameToWidget("editRunTBL*runField2TF");
  Widget runField3TF  = nameToWidget("editRunTBL*runField3TF");
  Widget runField4TF  = nameToWidget("editRunTBL*runField4TF");
  Widget runField5TF  = nameToWidget("editRunTBL*runField5TF");
  Widget runField6TF  = nameToWidget("editRunTBL*runField6TF");

  Widget runDspIntervalTF   = nameToWidget("editRunTBL*runDspIntervalTF");
  sprintf(buf,"%d",cat->interval);
  XmTextFieldSetString(runDspIntervalTF,buf);

  Widget drawarea = nameToWidget("editRunTBL*runDstRADIOBOX.lt");
  Widget rootarea = nameToWidget("editRunTBL*runDstRADIOBOX.mt");

  if(cat->getOutputWindow()->da_area)
    XmToggleButtonSetState(drawarea,True,False);
  else // if this field is NULL which means it is an animation object root
    XmToggleButtonSetState(rootarea,True,False);

  Widget  preserve = nameToWidget("editRunTBL*runTrailRADIOBOX.lt");
  Widget  wipeout  = nameToWidget("editRunTBL*runTrailRADIOBOX.mt");
  Widget  leaving  = nameToWidget("editRunTBL*runTrailRADIOBOX.rt");
  Widget  rainbow  = nameToWidget("editRunTBL*runTrailRADIOBOX.lm");
  
  TrailingMode tm = cat->getTrailingMode();
  switch(tm){
  case Preserve_E:
    XmToggleButtonSetState(preserve,True,False);
    break;
  case Wipeout_E:
    XmToggleButtonSetState(wipeout,True,False);
    break;
  case Leave_E:
    XmToggleButtonSetState(leaving,True,False);
    break;
  case Rainbow_E:
    XmToggleButtonSetState(rainbow,True,False);
    break;
  }

  Widget  wrap_bnd      = nameToWidget("editRunTBL*runBoundaryRADIOBOX.lt");
  Widget  bounce_bnd    = nameToWidget("editRunTBL*runBoundaryRADIOBOX.mt");
  Widget  disappear_bnd = nameToWidget("editRunTBL*runBoundaryRADIOBOX.rt");

  BoundaryMode bm = cat->getBoundaryMode();
  switch(bm){
  case Wrap_E:
    XmToggleButtonSetState(wrap_bnd,True,False);
    break;
  case BounceBack_E:
    XmToggleButtonSetState(bounce_bnd,True,False);
    break;
  case Disappear_E:
    XmToggleButtonSetState(disappear_bnd,True,False);
    break;
  }


  Widget runboundaryModeTBL = nameToWidget("editRunTBL*runboundaryModeTBL");
  Widget runtermCriteriaTBL = nameToWidget("editRunTBL*runtermCriteriaTBL");
  char *mode;
  switch(cat->getRunMode()){
  case Movie_E: // should never get this case here
    break;
  case Fixed_E:     mode = "Fixed"; 
		    XtSetSensitive(runboundaryModeTBL,False);
		    if(flag < 0)
		      XtSetSensitive(runtermCriteriaTBL,False);
		    else{
		      PutTextInLabelWidget(runField6LB,"Termination (Maximum Moves):");
		      XtSetSensitive(runtermCriteriaTBL,True);
		    }
		    PutTextInLabelWidget(runField1LB,"X Location:");
		    PutTextInLabelWidget(runField2LB,"Y Location:");
		    PutTextInLabelWidget(runField3LB,"-:");
		    PutTextInLabelWidget(runField4LB,"-:");
		    sprintf(buf,"%d",((RunFixed *)cat)->start_x);
		    XmTextFieldSetString(runField1TF,buf);
		    sprintf(buf,"%d",((RunFixed *)cat)->start_y);
		    XmTextFieldSetString(runField2TF,buf);
		    XmTextFieldSetString(runField3TF,"");
		    XmTextFieldSetString(runField4TF,"");
		    break;
  case Glide_E:    mode = "Glide"; 
		    XtSetSensitive(runboundaryModeTBL,False);
		    if(flag < 0)
		      XtSetSensitive(runtermCriteriaTBL,False);
		    else{
		      PutTextInLabelWidget(runField6LB,"Termination (Maximum Moves):");
		      XtSetSensitive(runtermCriteriaTBL,True);
		    }
		    PutTextInLabelWidget(runField1LB,"Initial X Location:");
		    PutTextInLabelWidget(runField2LB,"Initial Y Location:");
		    PutTextInLabelWidget(runField3LB,"Direction (dx dy):");
		    PutTextInLabelWidget(runField4LB,"Bounding Box (x y width height):");
		    sprintf(buf,"%d",((RunGlide *)cat)->start_x);
		    XmTextFieldSetString(runField1TF,buf);
		    sprintf(buf,"%d",((RunGlide *)cat)->start_y);
		    XmTextFieldSetString(runField2TF,buf);
		    sprintf(buf,"%d %d",((RunGlide *)cat)->start_dx,
			                ((RunGlide *)cat)->start_dy);
		    XmTextFieldSetString(runField3TF,buf);
		    sprintf(buf,"%d %d %d %d",
			    ((RunGlide *)cat)->bounding_x,
			    ((RunGlide *)cat)->bounding_y,
			    ((RunGlide *)cat)->bounding_width,
			    ((RunGlide *)cat)->bounding_height,
			  );
		    XmTextFieldSetString(runField4TF,buf);
		    break;
  case Random_E:    mode = "Random"; 
		    XtSetSensitive(runboundaryModeTBL,False);
		    if(flag < 0)
		      XtSetSensitive(runtermCriteriaTBL,False);
		    else{
		      PutTextInLabelWidget(runField6LB,"Termination (Maximum Moves):");
		      XtSetSensitive(runtermCriteriaTBL,True);
		    }
		    PutTextInLabelWidget(runField1LB,"Bounding Box X:");
		    PutTextInLabelWidget(runField2LB,"Bounding Box Y:");
		    PutTextInLabelWidget(runField3LB,"Bounding Box Size (Width height):");
		    PutTextInLabelWidget(runField4LB,"-:");
		    sprintf(buf,"%d",((RunRandom *)cat)->bounding_x);
		    XmTextFieldSetString(runField1TF,buf);
		    sprintf(buf,"%d",((RunRandom *)cat)->bounding_y);
		    XmTextFieldSetString(runField2TF,buf);
		    sprintf(buf,"%d %d",((RunRandom *)cat)->bounding_width,
			                ((RunRandom *)cat)->bounding_height);
		    XmTextFieldSetString(runField3TF,buf);
		    XmTextFieldSetString(runField4TF,"");
		    break;
  case Fly_E:       mode = "Fly"; 
		    XtSetSensitive(runboundaryModeTBL,True);
		    if(flag < 0)
		      XtSetSensitive(runtermCriteriaTBL,False);
		    else{
		      PutTextInLabelWidget(runField6LB,"Termination (Maximum Moves):");
		      XtSetSensitive(runtermCriteriaTBL,True);
		    }
		    PutTextInLabelWidget(runField1LB,"x(z) = :");
		    PutTextInLabelWidget(runField2LB,"y(z) = :");
		    PutTextInLabelWidget(runField3LB,"z = :");
		    PutTextInLabelWidget(runField4LB,"-:");  
		    sprintf(buf,"%s",((RunFly *)cat)->Xequation->getExprInString());
		    XmTextFieldSetString(runField1TF,buf);
		    sprintf(buf,"%s",((RunFly *)cat)->Yequation->getExprInString());
		    XmTextFieldSetString(runField2TF,buf);
		    sprintf(buf,"%d,%d,%d,%d",
			    ((RunFly *)cat)->fly_start_save,
			    ((RunFly *)cat)->fly_end_save,
			    ((RunFly *)cat)->fly_incr_save,
			    ((RunFly *)cat)->fly_current_save);
		    XmTextFieldSetString(runField3TF,buf);
		    XmTextFieldSetString(runField4TF,"");
		    break;
  case Blink_E:     mode = "Blink"; 
		    XtSetSensitive(runboundaryModeTBL,False);
		    if(flag < 0)
		      XtSetSensitive(runtermCriteriaTBL,False);
		    else{
		      PutTextInLabelWidget(runField6LB,"Termination (Maximum Moves):");
		      XtSetSensitive(runtermCriteriaTBL,True);
		    }
		    PutTextInLabelWidget(runField1LB,"X Location:");
		    PutTextInLabelWidget(runField2LB,"Y Location:");
		    PutTextInLabelWidget(runField3LB,"-:");
		    PutTextInLabelWidget(runField4LB,"-:");
		    sprintf(buf,"%d",((RunBlink *)cat)->start_x);
		    XmTextFieldSetString(runField1TF,buf);
		    sprintf(buf,"%d",((RunBlink *)cat)->start_y);
		    XmTextFieldSetString(runField2TF,buf);
		    XmTextFieldSetString(runField3TF,"");
		    XmTextFieldSetString(runField4TF,"");
		    break;
  case Locus_E:     mode = "Loucs"; 
		    XtSetSensitive(runboundaryModeTBL,True);
		    if(flag < 0)
		      XtSetSensitive(runtermCriteriaTBL,False);
		    else{
		      PutTextInLabelWidget(runField6LB,"Termination (Maximum Moves):");
		      XtSetSensitive(runtermCriteriaTBL,True);
		    }
		    PutTextInLabelWidget(runField1LB,"Locus Name:");
		    PutTextInLabelWidget(runField2LB,"Translation x:");
		    PutTextInLabelWidget(runField3LB,"Translation y:");
		    PutTextInLabelWidget(runField4LB,"-:");
		    sprintf(buf,"%s",((RunLocus *)cat)->locus->getName());
		    XmTextFieldSetString(runField1TF,buf);
		    sprintf(buf,"%d",((RunLocus *)cat)->trans_x);
		    XmTextFieldSetString(runField2TF,buf);
		    sprintf(buf,"%d",((RunLocus *)cat)->trans_y);
		    XmTextFieldSetString(runField3TF,buf);
		    XmTextFieldSetString(runField4TF,"");
		    break;
  }
  XmTextFieldSetString(runField5TF,mode);
  sprintf(buf,"editRunTBL.title: Edit %s Run Mode",mode);
  WcSetValue(editRunTBL,buf);

  if(flag >= 0){  
    TermCriteria tc = cat->getTermCriteria();
    Widget max_move   = nameToWidget("editRunTBL*runTermcritRADIOBOX.lt");
    Widget position   = nameToWidget("editRunTBL*runTermcritRADIOBOX.mt");
    Widget wall_clock = nameToWidget("editRunTBL*runTermcritRADIOBOX.rt");
    Widget time_intv  = nameToWidget("editRunTBL*runTermcritRADIOBOX.lm");
    switch(tc){
    case NOT_APPLICATBLE_E : 
      break;
    case MAX_MOVES_E:
      XmToggleButtonSetState(max_move,True,False);
      sprintf(buf,"%d",cat->tc_max_moves);
      XmTextFieldSetString(runField6TF,buf);
      break;
    case POSITION_E :
      XmToggleButtonSetState(position,True,False);
      sprintf(buf,"%d %d %d %d",cat->tc_x,cat->tc_y,cat->tc_width,cat->tc_height);
      XmTextFieldSetString(runField6TF,buf);
      break;
    case WALL_CLOCK_E:
      XmToggleButtonSetState(wall_clock,True,False);
      break;
    case TIME_INTERVAL_E:
      sprintf(buf,"%d",cat->tc_max_interval);
      XmTextFieldSetString(runField6TF,buf);
      XmToggleButtonSetState(time_intv,True,False);
      break;
    }

  }
}
void Xet::runFreeze(){
  Widget runRunObjectTF = nameToWidget("editRunTBL*runRunObjectTF");
  char *runRunObject=  XmTextFieldGetString(runRunObjectTF);
  scheduler->freeze(runRunObject);
  XtFree(runRunObject);
}
void Xet::runUnfreeze(){
  Widget runRunObjectTF = nameToWidget("editRunTBL*runRunObjectTF");
  char *runRunObject=  XmTextFieldGetString(runRunObjectTF);
  scheduler->unfreeze(runRunObject);
  XtFree(runRunObject);
}
void Xet::runReset(){
  Widget runRunObjectTF = nameToWidget("editRunTBL*runRunObjectTF");
  char *runRunObject=  XmTextFieldGetString(runRunObjectTF);
  RunBase  *cat = scheduler->findRunObject(runRunObject);
  if(cat)
    cat->reset();
  XtFree(runRunObject);
}
void Xet::runReverse(){
  Widget runRunObjectTF = nameToWidget("editRunTBL*runRunObjectTF");
  char *runRunObject=  XmTextFieldGetString(runRunObjectTF);
  RunBase  *cat = scheduler->findRunObject(runRunObject);
  if(cat)
    cat->reverse();
  XtFree(runRunObject);
}
void Xet::runReverseClip(){
  Widget runRunObjectTF = nameToWidget("editRunTBL*runRunObjectTF");
  char *runRunObject=  XmTextFieldGetString(runRunObjectTF);
  RunBase  *cat = scheduler->findRunObject(runRunObject);
  if(cat)
    cat->reverseClip();
  XtFree(runRunObject);
}

void Xet::runUpdate(){
  Widget runClipNameTF        = nameToWidget("editRunTBL*runClipNameTF");
  Widget runtermCriteriaTBL = nameToWidget("editRunTBL*runtermCriteriaTBL");
  Widget runDspIntervalTF   = nameToWidget("editRunTBL*runDspIntervalTF");
  Widget runRunObjectTF     = nameToWidget("editRunTBL*runRunObjectTF");
  Widget runField1TF        = nameToWidget("editRunTBL*runField1TF");
  Widget runField2TF        = nameToWidget("editRunTBL*runField2TF");
  Widget runField3TF        = nameToWidget("editRunTBL*runField3TF");
  Widget runField4TF        = nameToWidget("editRunTBL*runField4TF");
  Widget runField5TF        = nameToWidget("editRunTBL*runField5TF");
  Widget runField6TF        = nameToWidget("editRunTBL*runField6TF");
  Widget  preserve          = nameToWidget("editRunTBL*runTrailRADIOBOX.lt");
  Widget  wipeout           = nameToWidget("editRunTBL*runTrailRADIOBOX.mt");
  Widget  leaving           = nameToWidget("editRunTBL*runTrailRADIOBOX.rt");
  Widget  rainbow           = nameToWidget("editRunTBL*runTrailRADIOBOX.lm");
  Widget  wrap_bnd          = nameToWidget("editRunTBL*runBoundaryRADIOBOX.lt");
  Widget  bounce_bnd        = nameToWidget("editRunTBL*runBoundaryRADIOBOX.mt");
  Widget  disappear_bnd     = nameToWidget("editRunTBL*runBoundaryRADIOBOX.rt");

  int destination = 2;
  Widget drawarea = nameToWidget("editRunTBL*runDstRADIOBOX.lt");
  Widget rootarea = nameToWidget("editRunTBL*runDstRADIOBOX.mt");
  if(XmToggleButtonGetState(rootarea))    destination = 1;
  if(XmToggleButtonGetState(drawarea))    destination = 2;

  TrailingMode trailingMode;
  if(XmToggleButtonGetState(preserve))  trailingMode = Preserve_E;
  if(XmToggleButtonGetState(wipeout))   trailingMode =  Wipeout_E;
  if(XmToggleButtonGetState(leaving))   trailingMode =  Leave_E;
  if(XmToggleButtonGetState(rainbow))   trailingMode =  Rainbow_E;
  BoundaryMode boundaryMode;
  if(XmToggleButtonGetState(wrap_bnd))        boundaryMode = Wrap_E;
  if(XmToggleButtonGetState(bounce_bnd))      boundaryMode = BounceBack_E;
  if(XmToggleButtonGetState(disappear_bnd))   boundaryMode = Disappear_E;
  char *runDspInterval  =  XmTextFieldGetString(runDspIntervalTF);
  char *runRunObject    =  XmTextFieldGetString(runRunObjectTF);
  char *runClipName       =  XmTextFieldGetString(runClipNameTF);
  char *runField1       =  XmTextFieldGetString(runField1TF);
  char *runField2       =  XmTextFieldGetString(runField2TF);
  char *runField3       =  XmTextFieldGetString(runField3TF);
  char *runField4       =  XmTextFieldGetString(runField4TF);
  char *runField5       =  XmTextFieldGetString(runField5TF);
  char *runField6       =  XmTextFieldGetString(runField6TF);

  RunBase  *cat;
  int which;
  if(XtIsSensitive(runtermCriteriaTBL)){ // animation object in movie object
    sscanf(runRunObject,"%d",&which);
    cat = scheduler->getOneMovieObject(which);
  }else // normal animation object
    cat = scheduler->findRunObject(runRunObject);
  ClipList *cl = new ClipList(runClipName,clipsPool);
  if(cat == NULL || cl->Size() == 0){
    if(cl->Size() == 0){
      popupModalInfoBox("Cliplist <%s> is empty or bad clip names. \n",runClipName);
      delete cl;
    } 
    
    if(cat == NULL){
      popupModalInfoBox("This object is gone, probably you deleted it\n");
    }
    XtFree(runDspInterval);
    XtFree(runRunObject);  
    XtFree(runClipName);
    XtFree(runField1);     
    XtFree(runField2);     
    XtFree(runField3);     
    XtFree(runField4);     
    XtFree(runField5);     
    XtFree(runField6);
    return;
  }
  if(destination == 1)
    cat->setOutputWindow(rootArea);
  else
    cat->setOutputWindow(drawingArea);
  cat->setclipList(cl);
  int freezeOrNotToFreeze;
  if(scheduler->isFrozen(runRunObject) == 1)
    freezeOrNotToFreeze = 1;
  else{
    scheduler->freeze(runRunObject); // freeze it before making changes
    freezeOrNotToFreeze = 0;
  }
  Locus *l;
  if(strcmp(runField5,"Loucs") == 0){
    l = locusPool->findLocus(runField1);
    if(l != NULL){
      ((RunLocus *)cat)->setLocus(l);
      ((RunLocus *)cat)->trans_x = atoi(runField2);
      ((RunLocus *)cat)->trans_y = atoi(runField3);
    }else{
      popupModalInfoBox("Locus <%s> does not exist\n",runField1);
      return;
    }
  }
  if(strcmp(runField5,"Blink") == 0){
    ((RunBlink *)cat)->start_x = atoi(runField1);
    ((RunBlink *)cat)->start_y = atoi(runField2);
  }
  if(strcmp(runField5,"Fly") == 0){
    if(((RunFly *)cat)->setFlyEquation(runField1,runField2,runField3) != 1){
      insertStringtoInfo("Syntax error on fly equation\n");
      return;
    }
  }
  int xx,yy,ww,hh;
  unsigned int da_width = cat->drawingArea->da_width;
  unsigned int da_height = cat->drawingArea->da_height;
  if(strcmp(runField5,"Random") == 0){
    xx  = atoi(runField1);
    yy  = atoi(runField2);
    sscanf(runField3,"%d %d",&ww,&hh);
    if(xx <= 0) xx = 0;
    if(yy <= 0) yy = 0;
    if(ww <= 0 || (ww+xx) >= (int)da_width) ww = da_width - xx;
    if(hh <= 0 || (hh+yy) >= (int)da_height) hh = da_height - yy;
    ((RunRandom *)cat)->bounding_x       = xx;
    ((RunRandom *)cat)->bounding_y       = yy;
    ((RunRandom *)cat)->bounding_width   = ww;
    ((RunRandom *)cat)->bounding_height  = hh;
  }
  if(strcmp(runField5,"Glide") == 0){
    ((RunGlide *)cat)->start_x   = atoi(runField1);
    ((RunGlide *)cat)->start_y   = atoi(runField2);
    int aa,bb;
    sscanf(runField3,"%d %d",&aa,&bb);
    ((RunGlide *)cat)->start_dx  = aa;
    ((RunGlide *)cat)->start_dy  = bb;
    sscanf(runField4,"%d %d %d %d",&xx,&yy,&ww,&hh);
    if(xx <= 0) xx = 0;
    if(yy <= 0) yy = 0;
    if(ww <= 0 || (ww+xx) >= (int)da_width) ww = da_width - xx;
    if(hh <= 0 || (hh+yy) >= (int)da_height) hh = da_height - yy;
    ((RunGlide *)cat)->setBoundingBox(xx,yy,ww,hh);
  }
  if(strcmp(runField5,"Fixed") == 0){
    ((RunFixed *)cat)->start_x = atoi(runField1);
    ((RunFixed *)cat)->start_y = atoi(runField2);
  }
  cat->interval = atoi(runDspInterval);
  cat->setTrailingMode(trailingMode);
  cat->setBoundaryMode(boundaryMode);
  cat->reset();
  if(freezeOrNotToFreeze == 0)
    scheduler->unfreeze(runRunObject); 

  if(XtIsSensitive(runtermCriteriaTBL)){ // animation object in movie object
    Widget max_move      = nameToWidget("editRunTBL*runTermcritRADIOBOX.lt");
    Widget position      = nameToWidget("editRunTBL*runTermcritRADIOBOX.mt");
    Widget wall_clock    = nameToWidget("editRunTBL*runTermcritRADIOBOX.rt");
    Widget time_interval = nameToWidget("editRunTBL*runTermcritRADIOBOX.lm");
    TermCriteria termCriteria;
    if(XmToggleButtonGetState(max_move))       termCriteria = MAX_MOVES_E;
    if(XmToggleButtonGetState(position))       termCriteria = POSITION_E;
    if(XmToggleButtonGetState(wall_clock))     termCriteria = WALL_CLOCK_E;
    if(XmToggleButtonGetState(time_interval))  termCriteria = TIME_INTERVAL_E;
    cat->setTermCriteria(termCriteria,runField6);
    //update the movie XmList
    Widget movieList  =  nameToWidget("schedulerTBL*movieList");
    if(movieList){
      char buffer[199];
      char *tc = cat->getTermCondValueInString();
      sprintf(buffer,"%2d: ",which+1);
      sprintf(&buffer[strlen(buffer)],MOVIE_FORMAT,cat->getName(),
	      cat->getRunModeInString(),cat->interval,tc);
      XmString info = XmStringCreateSimple(buffer);
      XmListReplaceItemsPos(movieList,&info,1,which+1);
      XmStringFree(info);
    }
  }
  XtFree(runDspInterval);
  XtFree(runRunObject); 
  XtFree(runClipName);
  XtFree(runField1);     
  XtFree(runField2);     
  XtFree(runField3);     
  XtFree(runField4);     
  XtFree(runField5);     
  XtFree(runField6);     

}
void Xet::locusCancel(){
  Widget w = nameToWidget("*defLocus");
  XtSetSensitive(w,True);
  drawingStatus = Ready_E;
}
void Xet::locusPaste(){
  Widget locusList      = nameToWidget("locusTBL*locusList");
  char *buf = whichItemInXmListIsSelected(locusList);
  char name[256],itemno[10];
  if(buf){
    sscanf(buf,LOCUS_FORMAT_SCAN_NAME,itemno,name);
    Widget clipField1TF      = nameToWidget("clipSetupTBL*clipField1TF");
    if(clipField1TF){
      XmTextFieldSetString(clipField1TF,name);
    }
    Widget runField1TF  = nameToWidget("editRunTBL*runField1TF");
    if(runField1TF){
      XmTextFieldSetString(runField1TF,name);
    }
  }
}
void Xet::locusDup(){
  Widget locusList      = nameToWidget("locusTBL*locusList");
  char *buf = whichItemInXmListIsSelected(locusList);
  char name[256],itemno[10];
  if(buf){
    sscanf(buf,LOCUS_FORMAT_SCAN_NAME,itemno,name);
    Locus *ls = locusPool->dup(name);
  }
}
void Xet::locusDefine(){
  drawingStatus = DefineLocus_E;

  Widget locusNameTF       = nameToWidget("locusTBL*locusNameTF");
  Widget locuspointColorTF = nameToWidget("locusTBL*locuspointColorTF");
  Widget locuspointSizeTF  = nameToWidget("locusTBL*locuspointSizeTF");
  Widget locusmaxPointsTF  = nameToWidget("locusTBL*locusmaxPointsTF");
  
  char *locusName       = XmTextFieldGetString(locusNameTF);
  char *locuspointColor = XmTextFieldGetString(locuspointColorTF);
  char *locuspointSize  = XmTextFieldGetString(locuspointSizeTF);
  char *locusmaxPoints  = XmTextFieldGetString(locusmaxPointsTF);
  Widget l_def,l_accept,l_ignore;
  int pixval = atoi(locuspointColor);
  if(pixval == 0)
    pixval = (int) BlackPixel(Dpy,DefaultScreen(Dpy));
  int linewidth = atoi(locuspointSize);
  if(linewidth < 0)
    linewidth = 0;
  int maxpoints = atoi(locusmaxPoints);
  if(maxpoints <= 10)
    activeLocus = new Locus(DEFAULT_MAX_LOCUS_POINTS);
  else
    activeLocus = new Locus(maxpoints);
  XSetForeground(Dpy,locus_gc,(unsigned long) pixval);
  XSetLineAttributes(Dpy,locus_gc,(unsigned int) linewidth,LineSolid,CapRound,
		     JoinRound);
  Locus *l;
  if(strcmp(locusName,"") == 0){
    popupModalInfoBox("!!! must specify a locus name !!!");
    goto D99;
  }
  l = locusPool->findLocus(locusName);
  if(l != NULL){
    popupModalInfoBox("!!! Locus <%s> already exist !!!",locusName);
    goto D99;
  } 
  l_def    = nameToWidget("locusTBL*locusDefine");
  l_accept = nameToWidget("locusTBL*locusAccept");
  l_ignore = nameToWidget("locusTBL*locusIgnore");
  XtSetSensitive(l_def,False);
  XtSetSensitive(l_accept,True);
  XtSetSensitive(l_ignore,True);
  setCrossReverseCursor(drawingArea->da_area);
 D99:
  XtFree(locusName);
  XtFree(locuspointColor);
  XtFree(locuspointSize);
  XtFree(locusmaxPoints);
}

void Xet::locusIgnore(){
  Widget l_def    = nameToWidget("locusTBL*locusDefine");
  Widget l_accept = nameToWidget("locusTBL*locusAccept");
  Widget l_ignore = nameToWidget("locusTBL*locusIgnore");
  XtSetSensitive(l_def,True);
  XtSetSensitive(l_accept,False);
  XtSetSensitive(l_ignore,False);

  if(activeLocus){
    delete activeLocus;
    activeLocus = NULL;
  }
  normalCursor(drawingArea->da_area);
}
void Xet::locusAccept(){
  if(activeLocus){
    Widget locusList      = nameToWidget("locusTBL*locusList");
    Widget locusNameTF    = nameToWidget("locusTBL*locusNameTF");
    char *locusName       = XmTextFieldGetString(locusNameTF);
    if(strcmp(locusName,"") == 0){
      popupModalInfoBox("!!! Must specify a locus name !!!");
      XtFree(locusName);
      return;
    }
    Locus *l = locusPool->findLocus(locusName);
    if(l != NULL){
      popupModalInfoBox("!!! Locus <%s> already exist !!!",locusName);
      XtFree(locusName);
      return;
    } 

    if(locusPool->addLocus(locusName,activeLocus) == 1){
      activeLocus->setName(locusName);
      activeLocus = NULL; 
      Widget w1 = nameToWidget("*drawingStatus"); 
      drawingStatus = Ready_E;
      Widget l_def    = nameToWidget("locusTBL*locusDefine");
      Widget l_accept = nameToWidget("locusTBL*locusAccept");
      Widget l_ignore = nameToWidget("locusTBL*locusIgnore");
      XtSetSensitive(l_def,True);
      XtSetSensitive(l_accept,False);
      XtSetSensitive(l_ignore,False);
      normalCursor(drawingArea->da_area);
    }
    XtFree(locusName);
  }
}

void Xet::locusInfo(char *s){
  Widget locusList      = nameToWidget("locusTBL*locusList");
  if(locusList){
    if(s)
      placeToXmListInRightOrder(locusList,s);
    else 
      locusPool->buildLocusXmList(locusList);
  }
}

void Xet::locusShow(){
  // find out what is the current point width and color
  Widget locuspointColorTF  = nameToWidget("locusTBL*locuspointColorTF");
  Widget locuspointSizeTF   = nameToWidget("locusTBL*locuspointSizeTF");
  char  *locuspointColor    = XmTextFieldGetString(locuspointColorTF);
  char  *locuspointSize     = XmTextFieldGetString(locuspointSizeTF);
  int pixval = atoi(locuspointColor);
  if(pixval == 0)
    pixval = (int) BlackPixel(Dpy,DefaultScreen(Dpy));
  int linewidth = atoi(locuspointSize);
  if(linewidth < 0)
    linewidth = 0;
  XSetForeground(Dpy,locus_gc,(unsigned long) pixval);
  XSetLineAttributes(Dpy,locus_gc,(unsigned int) linewidth,LineSolid,CapRound,
		     JoinRound);

  XtFree(locuspointColor);
  XtFree(locuspointSize);

  Widget locusList      = nameToWidget("locusTBL*locusList");
  char *buf = whichItemInXmListIsSelected(locusList);
  char name[30],itemno[10];
  LocusPoint *pt;
  int i;
  if(buf){
    sscanf(buf,LOCUS_FORMAT_SCAN_NAME,itemno,name);
    Locus *ls = locusPool->findLocus(name);
    if(ls){
      for(i=0;i<ls->getSize();i++){
	if((pt = ls->getPoint(i)) == NULL)
	  break;
	XDrawLine(Dpy,drawingArea->window,locus_gc,pt->x,pt->y,pt->x,pt->y);
      }
    }else
      popupModalInfoBox("!!! Locus <%s> not found !!!",name);
  }
}
void Xet::locusClear(){
  rePaintBackground();
}
void Xet::locusDeleteAll(){
  Widget locusList      = nameToWidget("locusTBL*locusList");
  Widget areYouSureTBL  = createAreYouSureDialog("Delete all loci, Are you sure?");
  int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
  if(rtn){
    locusPool->deleteLocus();
    deleteAllItemsOfXmList(locusList);
  }
  destroyAreYouSureDialog();
}
void Xet::locusDelete(){
  Widget locusList      = nameToWidget("locusTBL*locusList");
  char *buf = whichItemInXmListIsSelected(locusList);
  char name[30], buffer[100],itemno[10];
  if(buf != NULL){
    sscanf(buf,LOCUS_FORMAT_SCAN_NAME,itemno,name);
    Locus *ls = locusPool->findLocus(name);
    if(ls->getRefCounter() > 0){
      popupModalInfoBox("Can not delete locus <%s> now, a run object is still using it",name);
      return;
    }
    sprintf(buffer,"Delete locus %s, Are you sure ?",name);
    Widget areYouSureTBL = createAreYouSureDialog(buffer);
    int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
    if(rtn){
      locusPool->deleteLocus(name);
      DeleteSelectedItemFromXmList(locusList);
    }
    destroyAreYouSureDialog();
  }else
    popupModalInfoBox("!!! Must select an item !!! ");
}

void Xet::configClipsShow(){
  Widget configList = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    Clip *clp = clipsPool->findClip(clipName);
    if(clp)
      clp->DisplayImage(drawingArea,0,0);
    else
      popupModalInfoBox("!!! Clip <%s> not found !!!",clipName);
  }
}
void Xet::configClipsDup(){
  Widget configList = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    Clip *clp = clipsPool->findClip(clipName);
    if(clp){
      clipsPool->dup(clipName);
    }else
      popupModalInfoBox("Clip <%s> not found",clipName);
  }
}
void Xet::configClipsClone(){
  Widget configList = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    Clip *clp = clipsPool->findClip(clipName);
    if(clp){
      clipsPool->clone(clipName);
    }else
      popupModalInfoBox("Clip <%s> not found",clipName);
  }
}

void Xet::configClipsSetStatus(){
  popupModalInfoBox("Not implemented yet");
}

void Xet::renameClipOk(){
  Widget renameClipOldTF     = nameToWidget("renameClipTBL*renameClipOldTF");
  Widget renameClipNewNameTF = nameToWidget("renameClipTBL*renameClipNewNameTF");
  char *newName = XmTextFieldGetString(renameClipNewNameTF);
  char *oldName = XmTextFieldGetString(renameClipOldTF);
  Clip *clp = clipsPool->findClip(oldName);
  if(clp){
    Clip *cp = clipsPool->findClip(newName);
    if(cp){
      popupModalInfoBox("Clip <%s> already exist",newName);
      goto D99;
    }else{
      Widget configList = nameToWidget("configClipsTBL*configList");
      clp->setClipName(newName);
      deleteItemInXmList(configList,oldName);
      clipsPool->renameKey(oldName,newName);
      insertStringtoInfo("Clip <%s> is renamed to <%s>\n",oldName,newName); 
    }
  }
  else
    popupModalInfoBox("It is impossible,Clip <%s> not found",oldName);
 D99:
  XtFree(newName);
  XtFree(oldName);
}

void Xet::configClipsRename(){
  Widget configClipsTBL = nameToWidget("configClipsTBL");
  Widget configList     = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    WcCreateRoot(Dpy,"renameClipTBL");
    Widget renameClipOldTF = nameToWidget("renameClipTBL*renameClipOldTF");
    XmTextFieldSetString(renameClipOldTF,clipName);
  }else{
    popupModalInfoBox("!!! Must select an item !!!");
    Widget w = nameToWidget("configClipsTBL*configClipsRename");
    XtSetSensitive(w,True);
  }
}

void Xet::configClipsDeleteAll(){
  if(scheduler->occupy() > 0){
    popupModalInfoBox("Can not delete all clips now, scheduler is not empty");
    return;
  }
  Widget areYouSureTBL = createAreYouSureDialog("Delete all clips, Are you sure ?");
  int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
  if(rtn){
    Widget configList = nameToWidget("configClipsTBL*configList");
    clipsPool->clearEverything();
    deleteAllItemsOfXmList(configList);
  }
  destroyAreYouSureDialog();
}
void Xet::setclipAttributes(){
  Widget configClipsTBL = nameToWidget("configClipsTBL");
  char buffer[100];
  Widget configList = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    Clip *clp = clipsPool->findClip(clipName);
    if(clp)
      WcCreateRoot(Dpy,"clipAttribTBL");
    // fillout clip attributes
    Widget clipAttribclipNameTF     = nameToWidget("clipAttribTBL*clipAttribclipNameTF");
    Widget clipAttribncolorsTF    = nameToWidget("clipAttribTBL*clipAttribncolorsTF");
    Widget clipAttribshapeMaskTF  = nameToWidget("clipAttribTBL*clipAttribshapeMaskTF");
    Widget clipAttribtypeTF       = nameToWidget("clipAttribTBL*clipAttribtypeTF");
    Widget clipAttribsizeTF       = nameToWidget("clipAttribTBL*clipAttribsizeTF");
    Widget clipAttribrefCounterTF = nameToWidget("clipAttribTBL*clipAttribrefCounterTF");
    Widget clipAttribstatusTF     = nameToWidget("clipAttribTBL*clipAttribstatusTF");
    Widget clipAttribimageFnameTF = nameToWidget("clipAttribTBL*clipAttribimageFnameTF");
    Widget clipAttribshapeFnameTF = nameToWidget("clipAttribTBL*clipAttribshapeFnameTF");
    XmTextFieldSetString(clipAttribclipNameTF, clp->getClipName());    
    XmTextFieldSetString(clipAttribimageFnameTF, clp->getDiskFileName());
    XmTextFieldSetString(clipAttribshapeFnameTF, clp->getShapeFileName());

    sprintf(buffer,"%d",clp->getNcolors());
    XmTextFieldSetString(clipAttribncolorsTF,buffer);   
    XmTextFieldSetString(clipAttribshapeMaskTF,(clp->getShapeMask()) ? "yes" : "no");
    sprintf(buffer,"%d x %d",clp->Width(),clp->Height());
    XmTextFieldSetString(clipAttribsizeTF,buffer);     
    sprintf(buffer,"%d",clp->getRefCounter());
    XmTextFieldSetString(clipAttribrefCounterTF,buffer);
    XmTextFieldSetString(clipAttribstatusTF,clp->getClipStatusInString());    
    if(clp->isClone() == 0)
      XmTextFieldSetString(clipAttribtypeTF,"Original");
    else{
      sprintf(buffer,"link to %s",
	      ((CloneImage *)clp)->getOriginalClip()->getClipName());
      XmTextFieldSetString(clipAttribtypeTF,buffer); 
    }
  }else{
    popupModalInfoBox("!!! Must select an item !!!");
    genericCancel("configClipsTBL*configClipsAttribute");
  }
}
void Xet::clipAttribApply(){
  Widget clipAttribclipNameTF     = nameToWidget("clipAttribTBL*clipAttribclipNameTF");
  Widget clipAttribimageFnameTF = nameToWidget("clipAttribTBL*clipAttribimageFnameTF");
  Widget clipAttribshapeFnameTF = nameToWidget("clipAttribTBL*clipAttribshapeFnameTF");
  char *clipAttribclipName        = XmTextFieldGetString(clipAttribclipNameTF);
  char *clipAttribimageFname    = XmTextFieldGetString(clipAttribimageFnameTF);
  char *clipAttribshapeFname    = XmTextFieldGetString(clipAttribshapeFnameTF);
  Clip *clp = clipsPool->findClip(clipAttribclipName);
  if(clp){
    clp->setDiskFileName(clipAttribimageFname);
    clp->setShapeFileName(clipAttribshapeFname);
  }else
    insertStringtoInfo("It is impossible, clip <%s> is no longer exist",clipAttribclipName);
  insertStringtoInfo("Clip <%s> attributes are saved \n",clipAttribclipName);
}

void Xet::configClipsDelete(){
  char buffer[100];
  Widget configList = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    Clip *clp = clipsPool->findClip(clipName);
    if(clp->getRefCounter() > 0){
      popupModalInfoBox("Can not delete clip <%s> now, a run object or a dup clips is still using it",clipName);
      return;
    }
    sprintf(buffer,"Delete clip <%s>, Are you sure ?",clipName);
    Widget areYouSureTBL = createAreYouSureDialog(buffer);
    int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
    if(rtn){
      clipsPool->deleteClip(clipName);
      DeleteSelectedItemFromXmList(configList);
      configClipsInfo(NULL);
      updateXetStatistic();
    }
    destroyAreYouSureDialog();
  }else
    popupModalInfoBox("!!! Must select an item !!!");
}

void Xet::configClipsDeleteMask(){
  char buffer[100];
  Widget configList = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    Clip *clp = clipsPool->findClip(clipName);
    sprintf(buffer,"Delete mask of clip <%s>, Are you sure ?",clipName);
    Widget areYouSureTBL = createAreYouSureDialog(buffer);
    int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
    if(rtn){
      clipsPool->deleteClipMask(clipName);
      replaceXmListHighlightContent(configList,"Has","No");
    }
    destroyAreYouSureDialog();
  }else
    popupModalInfoBox("!!! Must select an item !!!");
}
void Xet::writeClipImage(){
  Widget writeClipImageCnameTF = nameToWidget("writeClipImageTBL*writeClipImageCnameTF");
  Widget writeClipImageDnameTF = nameToWidget("writeClipImageTBL*writeClipImageDnameTF");
  char *writeClipImageCname = XmTextFieldGetString(writeClipImageCnameTF);
  char *writeClipImageDname = XmTextFieldGetString(writeClipImageDnameTF);
  if(isBlankString(writeClipImageCname) != 1){
    Clip *clp = clipsPool->findClip(writeClipImageCname);
    if(clp)
      clp->writeOut(writeClipImageDname);
    else
      popupModalInfoBox("Clip <%s> not found\n",writeClipImageCname);
  }else
    popupModalInfoBox("!!! Must specify a clip name !!!");
  XtFree(writeClipImageCname);
  XtFree(writeClipImageDname);
}
void Xet::preWriteClipImage(){
  Widget configList = nameToWidget("configClipsTBL*configList");
  char  *buf        = whichItemInXmListIsSelected(configList);
  char itemno[10],clipName[256];
  if(buf){
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
  }else{
    popupModalInfoBox("!!! Must select a clip !!!");
    genericCancel("configClipsTBL*configClipsWriteClip");
    return;
  }
  WcCreateRoot(Dpy,"writeClipImageTBL");
  Widget writeClipImageCnameTF = nameToWidget("writeClipImageTBL*writeClipImageCnameTF");
  XmTextFieldSetString(writeClipImageCnameTF,clipName);
}
void Xet::writeClipMask(){
  Widget writeClipMaskCnameTF = nameToWidget("writeClipMaskTBL*writeClipMaskCnameTF");
  Widget writeClipMaskDnameTF = nameToWidget("writeClipMaskTBL*writeClipMaskDnameTF");
  char *writeClipMaskCname = XmTextFieldGetString(writeClipMaskCnameTF);
  char *writeClipMaskDname = XmTextFieldGetString(writeClipMaskDnameTF);
  if(isBlankString(writeClipMaskCname) != 1){
    Clip *clp = clipsPool->findClip(writeClipMaskCname);
    if(clp){
      Pixmap    pm =  clp->getShapeMask();
      if(pm)
	clp->writeOutMask(writeClipMaskDname);
      else
	popupModalInfoBox("Clip <%s> doesn't have shape mask",writeClipMaskCname);
    }else
      popupModalInfoBox("Clip <%s> not found",writeClipMaskCname);
  }else
    popupModalInfoBox("!!! Must specify a clip name !!!");
  XtFree(writeClipMaskCname);
  XtFree(writeClipMaskDname);
}
void Xet::preWriteClipMask(){
  Widget configList = nameToWidget("configClipsTBL*configList");
  char *buf = whichItemInXmListIsSelected(configList);
  char itemno[10],clipName[256];
  if(buf){
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
  }else{
    popupModalInfoBox("!!! Must select a clip !!!");
    genericCancel("configClipsTBL*configClipsWriteMask");
    return;
  }

  WcCreateRoot(Dpy,"writeClipMaskTBL") ;

  Widget writeClipMaskCnameTF = nameToWidget("writeClipMaskTBL*writeClipMaskCnameTF");
  XmTextFieldSetString(writeClipMaskCnameTF,clipName);
}
void Xet::preshapeImportMask(){
  Widget shapebackgroundimageTF = nameToWidget("shapeTBL*shapebackgroundimageTF");
  char *shapebackgroundimage = XmTextFieldGetString(shapebackgroundimageTF);
  Clip *clp = clipsPool->findClip(shapebackgroundimage);
  if(clp){
    WcCreateRoot(Dpy,"shapeImportMaskTBL"); 
    Widget shapeImportMaskCnameTF = nameToWidget("shapeImportMaskTBL*shapeImportMaskCnameTF");
    XmTextFieldSetString(shapeImportMaskCnameTF,shapebackgroundimage);
  }else{
    popupModalInfoBox("Invalid clip <%s> name",shapebackgroundimage);
    genericCancel("shapeTBL*shapeImport");
    }
  XtFree(shapebackgroundimage);
}


void Xet::shapeImportMask(){
  Widget shapeImportMaskCnameTF = nameToWidget("shapeImportMaskTBL*shapeImportMaskCnameTF");
  Widget shapeImportMaskDnameTF = nameToWidget("shapeImportMaskTBL*shapeImportMaskDnameTF");
  char *shapeImportMaskCname    = XmTextFieldGetString(shapeImportMaskCnameTF);
  char *shapeImportMaskDname    = XmTextFieldGetString(shapeImportMaskDnameTF);
  Clip *clp = clipsPool->findClip(shapeImportMaskCname);
  if(clp){
    if(clp->getShapeMask()){
      Widget areYouSureTBL = createAreYouSureDialog("Clip already has a shape mask, replace it,  Are you sure ?");
      int rtn = performAsyncInteraction(Dpy,areYouSureTBL);
      if(!rtn){
	insertStringtoInfo("Mask is not attached to clip <%s>\n",shapeImportMaskCname);
	return;
      }
    }
    unsigned int width, height;
    int          x_hot, y_hot;
    Pixmap       bitmap;
    if(BitmapOpenFailed != XReadBitmapFile(Dpy,drawingArea->window,
					   shapeImportMaskDname,
					   &width,&height,&bitmap,&x_hot,&y_hot))
      {
	clp->setShapeMask(bitmap);
	insertStringtoInfo("Import mask is attached to clip <%s>\n",shapeImportMaskCname);
      }else
	popupModalInfoBox("Bitmap file <%s> not found\n",shapeImportMaskDname);
  }else
    popupModalInfoBox("It is impossible, clip <%s> no longer exist\n",shapeImportMaskCname);
  XtFree(shapeImportMaskCname);
  XtFree(shapeImportMaskDname);
}

void Xet::writeOutXetFileAs(){
  Widget filedirectoryTF  = nameToWidget("filenameTBL*filedirectoryTF");
  Widget filenameTF       = nameToWidget("filenameTBL*filenameTF");
  char *filename = XmTextGetString(filenameTF);
  char *filedirectory = XmTextGetString(filedirectoryTF);
  char filebuf[200];
  if(strcmp("",filename) == 0){
    popupModalInfoBox("!!! Must specify a file name !!!");
    return;
  }
  EFFECTIVE_DIRECTORY = calcEffectiveDirectory(filedirectory);
  if(EFFECTIVE_DIRECTORY == NULL){
    popupModalInfoBox("can not access directory <%s>\n",filedirectory);
    goto D99;
  }
  if(DirectoryExist(EFFECTIVE_DIRECTORY) == 1){
    sprintf(filebuf,"OverWrite <%s> !!, Are you sure ?",EFFECTIVE_DIRECTORY);
    Widget areYouSureTBL = createAreYouSureDialog(filebuf);
    int rtn;
    rtn = performAsyncInteraction(Dpy,areYouSureTBL);
    if(!rtn){
      goto D99;
    }
  }else
    if(mkdir(EFFECTIVE_DIRECTORY,0777) != 0){
      popupModalInfoBox("can not create directory <%s>",EFFECTIVE_DIRECTORY);
      goto D99;
    }
  sprintf(filebuf,"%s/%s",EFFECTIVE_DIRECTORY,filename);
  writeOutXetFile(filebuf);
 D99:
  XtFree(filename);
  XtFree(filedirectory);
}

void Xet::writeOutXetFile(char *filename){
  Widget all_opt     = nameToWidget("filenameTBL*filenameOptRADIOBOX.mt");
  Widget partial_opt = nameToWidget("filenameTBL*filenameOptRADIOBOX.rt");
  Widget S           = nameToWidget("filenameTBL*filenameOptRADIOBOX1.mt");
  Widget C           = nameToWidget("filenameTBL*filenameOptRADIOBOX1.rt");
  Widget L           = nameToWidget("filenameTBL*filenameOptRADIOBOX1.lm");
  Widget B           = nameToWidget("filenameTBL*filenameOptRADIOBOX1.lt");
    
  char buffer[100];
  time_t t = time(0);
  FILE *fp = fopen(filename,"w+");
  if(fp){
    setEffectiveCursor(writeCursor); 
    scheduler->suspend();
    fprintf(fp,"# Xet command file : %s\n", filename);
    fprintf(fp,"# user : %s \n", getenv("USER"));
    fprintf(fp,"# date : %s",(char *)ctime(&t));
    fprintf(fp,"# \n");
    
    fprintf(fp,"# directory : %s\n",EFFECTIVE_DIRECTORY);
    fprintf(fp,"# ------------------------------\n");
    sprintf(buffer,"%s/%s",EFFECTIVE_DIRECTORY,DEFAULT_BACKGROUND_FILE);
    if(XmToggleButtonGetState(all_opt) ||
       (XmToggleButtonGetState(partial_opt) && XmToggleButtonGetState(B))){
      fprintf(fp,"background:%s\n",skipDirectoryPath(buffer));
      writeBackgroundImage(buffer);
    }
    fprintf(fp,"# ------------------------------\n");
    if(XmToggleButtonGetState(all_opt) ||
       (XmToggleButtonGetState(partial_opt) && XmToggleButtonGetState(C)))
      clipsPool->writeOut(fp);
    fprintf(fp,"# ------------------------------\n");
    if(XmToggleButtonGetState(all_opt) ||
       (XmToggleButtonGetState(partial_opt) && XmToggleButtonGetState(L)))
      locusPool->writeOut(fp);
    fprintf(fp,"# ------------------------------\n");
     if(XmToggleButtonGetState(all_opt) ||
	(XmToggleButtonGetState(partial_opt) && XmToggleButtonGetState(S)))
       scheduler->writeOut(fp);
    fprintf(fp,"# \n");
    fprintf(fp,"# end of Xet command file. \n");
    fclose(fp);
    scheduler->resume();
    insertStringtoInfo("Wrote xet file <%s> completed.\n",filename);
    setSystemDefinedCursor(XC_left_ptr);
    XFlush(Dpy);
  }else
    popupModalInfoBox("Can not open file <%s> \n",filename);
}

void Xet::editLocusPoints(){
  Widget locusList      = nameToWidget("locusTBL*locusList");
  char *buf             = whichItemInXmListIsSelected(locusList);
  char name[256],item[10];
  LocusPoint *pt;
  char buffer[20];
  if(buf){
    sscanf(buf,LOCUS_FORMAT_SCAN_NAME,item,name);
    Locus *ls = locusPool->findLocus(name);
    if(ls){
      Widget  editLocusTBL    = nameToWidget("editLocusTBL");
      if(editLocusTBL == NULL){
	editLocusTBL = WcCreateRoot(Dpy,"editLocusTBL");

      }else{
	popupModalInfoBox("Must close the other editing of locus");
	return;
      }
      Widget editLocusPanelTXT = nameToWidget("editLocusTBL*editLocusPanelTXT");
      int ii;
      for(ii=0;ii<ls->getSize();ii++){
	if((pt = ls->getPoint(ii)) == NULL)
	  break;
        if(ii > 0 && ii % 4 == 0)
	  insertTXTText(editLocusPanelTXT,"\n");
	sprintf(buffer," (%d, %d) ", pt->x,pt->y);
	insertTXTText(editLocusPanelTXT,buffer);
      }
      XmTextSetEditable(editLocusPanelTXT,True);
      Widget editLocusPanelTF = nameToWidget("editLocusTBL*editLocusPanelTF");
      Widget editLocusnpsTF   = nameToWidget("editLocusTBL*editLocusnpsTF");
      XmTextFieldSetString(editLocusPanelTF,name);
      sprintf(buffer,"%d",ii);
      XmTextFieldSetString(editLocusnpsTF,buffer);
    }else
      popupModalInfoBox("Locus <%s> not found",name);
  }else
    popupModalInfoBox("!!! Must select an item !!!");
}

int Xet::buildLocusPoints(LocusPoint *lp,int npoints,char *asciipoints){
  int i; 
  char *ptr;
  ptr = asciipoints;
  for(i=0;i<npoints;i++){
    while(!isdigit(*ptr)){
      if(*ptr == '\0')
	goto D99;
      (void)*ptr++;
    }
    lp[i].x = atoi(ptr);
    while(isdigit(*ptr++));
    while(!isdigit(*ptr)){
      if(*ptr == '\0')
	goto D99;
      (void)*ptr++;
    }
    lp[i].y = atoi(ptr);
    while(isdigit(*ptr++));
  }
 D99:
  insertStringtoInfo("<%d> locus points is accepted. \n",i);
  return i;
}
static int howmanyPoints(char *s){
  int i; 
  char *ptr;
  ptr = s;
  for(i=0;;i++){
    while(!isdigit(*ptr)){
      if(*ptr == '\0')
	goto D99;
      (void)*ptr++;
    }
    while(isdigit(*ptr++));
    while(!isdigit(*ptr)){
      if(*ptr == '\0')
	goto D99;
      (void)*ptr++;
    }
    while(isdigit(*ptr++));
  }
 D99:
  return i;
}
void Xet::editLocusAccept(){
  Widget editLocusPanelTXT = nameToWidget("editLocusTBL*editLocusPanelTXT");
  Widget editLocusnpsTF    = nameToWidget("editLocusTBL*editLocusnpsTF");
  Widget editLocusPanelTF  = nameToWidget("editLocusTBL*editLocusPanelTF");
  char *editLocusnps       = XmTextFieldGetString(editLocusnpsTF);
  char *asciipoints        = XmTextGetString(editLocusPanelTXT);
  char *locusName          = XmTextFieldGetString(editLocusPanelTF);
  int npoints = atoi(editLocusnps); // can not trust this number
  int realnps;
  Locus *ls = locusPool->findLocus(locusName);
  if(ls){
    realnps = howmanyPoints(asciipoints);
    LocusPoint *lp = new LocusPoint[realnps]; 
    realnps = buildLocusPoints(lp,realnps,asciipoints);
    ls->depositPoints(realnps,lp);
    locusInfo(NULL);
  }else
    popupModalInfoBox("Locus <%s> is gone, probably you deleted it",locusName);
  XtFree(editLocusnps);
  XtFree(asciipoints);
  XtFree(locusName);
}
void Xet::editLocusUnique(){
  Widget editLocusPanelTXT = nameToWidget("editLocusTBL*editLocusPanelTXT");
  Widget editLocusnpsTF    = nameToWidget("editLocusTBL*editLocusnpsTF");
  Widget editLocusPanelTF  = nameToWidget("editLocusTBL*editLocusPanelTF");
  char *locusName          = XmTextFieldGetString(editLocusPanelTF);
  Locus *ls = locusPool->findLocus(locusName);
  LocusPoint *pt;
  char buffer[20];
  if(ls){
    ls->unique(ls->getSize());
    XmTextSetString(editLocusPanelTXT,"");
    int ii;
    for(ii=0;ii < ls->getSize();ii++){
      if((pt = ls->getPoint(ii)) == NULL)
	break;
      if(ii > 0 && ii % 4 == 0)
	insertTXTText(editLocusPanelTXT,"\n");
      sprintf(buffer," (%d, %d) ", pt->x,pt->y);
      insertTXTText(editLocusPanelTXT,buffer);
    }
    sprintf(buffer,"%d",ii);
    XmTextFieldSetString(editLocusnpsTF,buffer);
    locusInfo(NULL);
  }else
    popupModalInfoBox("Locus <%s> is gone, probably you deleted it",locusName);
  XtFree(locusName);
}

void Xet::writeBackgroundImage(char *fileName){
  writeXimageInPPM(Dpy,fileName,drawingArea->backgroundImage);
}

void Xet::setClipStatus(ClipStatus p){
  Widget configClipsTBL = nameToWidget("configClipsTBL");
  Widget configList     = nameToWidget("configClipsTBL*configList");
  char   *buf           = whichItemInXmListIsSelected(configList);
  if(buf){
    char itemno[10],clipName[256];
    sscanf(buf,CLIP_FORMAT_SCAN_NAME, itemno, clipName);
    int i = 0;
    while(buf[i++] != ',');
    while(buf[i++] != ',');
    while(buf[i++] != ',');
    char *status = &buf[i];
    i = 0;
    while(status[i++] != '>');
    status[i-1] = '\0';
    Clip *clp = clipsPool->findClip(clipName);
    if(clp){
      clp->setClipStatus(p);
      replaceXmListHighlightContent(configList,status,
				    clp->getClipStatusInString());
    }
  }else
    popupModalInfoBox("!!! Must select an item !!!");
}

void Xet::renameLocusOk(){
  Widget renameLocusOldTF     = nameToWidget("renameLocusTBL*renameLocusOldTF");
  Widget renameLocusNewNameTF = nameToWidget("renameLocusTBL*renameLocusNewNameTF");
  char *newName = XmTextFieldGetString(renameLocusNewNameTF);
  char *oldName = XmTextFieldGetString(renameLocusOldTF);
  Locus *clp = locusPool->findLocus(oldName);
  if(clp){
    Locus *ls = locusPool->findLocus(newName);
    if(ls){
      popupModalInfoBox("Locus <%s> already exist",newName);
      goto D99;
    }else{
      Widget locusList = nameToWidget("locusTBL*locusList");
      clp->setName(newName);
      deleteItemInXmList(locusList,oldName);
      locusPool->renameKey(oldName,newName);
      insertStringtoInfo("Locus <%s> is renamed to <%s>\n",oldName,newName); 
    }
  }
  else
    popupModalInfoBox("It is impossible,Locus <%s> not found\n",oldName);
 D99:
  XtFree(newName);
  XtFree(oldName);
}
void Xet::locusRename(){
  Widget locusTBL  = nameToWidget("LocusTBL");
  Widget locusList = nameToWidget("locusTBL*locusList");
  char *buf = whichItemInXmListIsSelected(locusList);
  if(buf){
    char itemno[10],locusid[256];
    sscanf(buf,LOCUS_FORMAT_SCAN_NAME, itemno, locusid);
    WcCreateRoot(Dpy,"renameLocusTBL");
    Widget renameLocusOldTF = nameToWidget("renameLocusTBL*renameLocusOldTF");
    XmTextFieldSetString(renameLocusOldTF,locusid);
  }else{
    popupModalInfoBox("!!! Must select an item !!!");
    Widget w = nameToWidget("locusTBL*locusRename");
    XtSetSensitive(w,True);
  }
}

void Xet::renameSchedOk(){
  Widget renameSchedOldTF     = nameToWidget("renameSchedTBL*renameSchedOldTF");
  Widget renameSchedNewNameTF = nameToWidget("renameSchedTBL*renameSchedNewNameTF");
  char *newName = XmTextFieldGetString(renameSchedNewNameTF);
  char *oldName = XmTextFieldGetString(renameSchedOldTF);
  RunBase *clp = scheduler->findRunObject(oldName);
  if(clp){
    RunBase *ls = scheduler->findRunObject(newName);
    if(ls){
      popupModalInfoBox("RunObject <%s> already exist",newName);
      goto D99;
    }else{
      Widget schedList = nameToWidget("schedulerTBL*schedList");
      deleteItemInXmList(schedList,oldName);
      scheduler->renameKey(oldName,newName);
      insertStringtoInfo("Scheduler <%s> is renamed to <%s>\n",oldName,newName); 
    }
  }
  else
    popupModalInfoBox("It is impossible, RunObject <%s> not found\n",oldName);
 D99:
  XtFree(newName);
  XtFree(oldName);
}

void Xet::schedulerRename(){
  Widget schedulerTBL = nameToWidget("SchedulerTBL");
  Widget schedList = nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],schedid[256];
    sscanf(buf,"%s %s ", itemno, schedid);
    WcCreateRoot(Dpy,"renameSchedTBL");
    Widget renameSchedOldTF = nameToWidget("renameSchedTBL*renameSchedOldTF");
    XmTextFieldSetString(renameSchedOldTF,schedid);
  }else{
    popupModalInfoBox("!!! Must select an item !!!");
    genericCancel("schedulerTBL*schedulerRename");
  }
}

